# File svdfit/esky.cl
# January 27, 2000
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# ESKY -- Remove sky from a spectrum by fitting eigenvectors for
#         a file or list of files

procedure esky (infile, outfile)

string	infile=""	{prompt="Input spectrum file"}
string	specnum="1"	{prompt="Range of spectra in multispec or 2D file"}
int	specband=1	{prompt="Band if multispec file"}
string	indir=""	{prompt="Input directory"}
string	outform="multispec" {prompt="Output format (multispec|oned|twod)"}
string	intfile="eigtemp1.fits"	{prompt="Intermediate spectrum file"}
string	intname="Output from ESKY pass 1" {prompt="Intermediate spectrum title"}
file	vecfile1="evec1.fits"	{prompt="First pass eigenvector file"}
string	vectors1="1,2"	{prompt="First pass eigenvectors"}
string	fixbad1="zero"	{prompt="Fix bad lines in first pass (zero fill no)"}
real	renorm1=1.0	{prompt="Renormalize (divide by mean) in first pass"}
int	order1=5	{prompt="First pass continuum order"}
int	avg1=1		{prompt="First pass continuum averaging"}
bool	delfile=yes	{prompt="Delete intermediate file (y or n)"}
string	outfile="eigtemp2.fits"	{prompt="Output spectrum file"}
string	outname="Output from ESKY pass 2" {prompt="Output spectrum title"}
file	vecfile2="evec2.fits"	{prompt="Second pass eigenvector file"}
string	fixbad2="zero"	{prompt="Fix bad lines in second pass (zero fill no)"}
real	renorm2=1.0	{prompt="Renormalize (divide by mean) in second pass"}
string	vectors2="1-9"	{prompt="Second pass eigenvectors"}
int	avg2=1		{prompt="Second pass continuum averaging"}
int	order2=15	{prompt="Second pass continuum order"}
file	badlines="badskynao.dat" {prompt="File with lines to eliminate"}
bool	plot=yes	{prompt="Plot results on display (y or n)"}
bool	cursor=no	{prompt="Wait after plot with cursor (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	debug=no	{prompt="Debugging mode (y or n)"}

begin

bool	vb, delf
int	i
file	filelist
file	localfile, ifile, jfile, ofile
file	listfile, eigtemp
string	tchar

	ifile = infile
	ofile = outfile
	vb = verbose
	jfile = intfile
	delf = delfile

	svdres.specband = specband
	svdres.specnum = specnum
	svdres.debug = debug
	svdres.res_plot = plot
	svdres.plot_int = cursor
	svdres.badlines = badlines

	set clobber = yes

#  Process data files from RFN or file of RFN's
	tchar = substr (ifile,1,1)
	if (tchar != '@') {
	    filelist = mktemp ("eiglist")
	    print (ifile,>filelist)
	    listfile = filelist
	    }
	else
	    listfile = substr(ifile,2,64)

	if (vb)
	    print ("ESKY: files from ",listfile)
 
#  Loop through list of reduced files
	list = listfile
	i = 0
	while (fscan (list, line) != EOF) {
	    i = i + 1
	    if (indir != "") {
		localfile = indir // "/" // line
		}
	    else {
		localfile = line
		}

	# If the reduced file is found, process a local copy of it
	    if (localfile != "") {

	# Fit first pass eigenvectors
		svdres.eigenfiles = vecfile1
		svdres.eigenum = vectors1
		svdres.fixbad = fixbad1
		svdres.renormalize = renorm1
		svdres.cont_remove = "subtract"
		svdres.outname = intname
		contres.order = order1
		contres.naverage = avg1
		svdres (localfile, jfile)

	# Fit second pass eigenvectors
		svdres.eigenfiles = vecfile2
		svdres.eigenum = vectors2
		svdres.fixbad = fixbad2
		svdres.renormalize = renorm2
		svdres.cont_remove = "subtract"
		svdres.outname = outname
		contres.order = order2
		contres.naverage = avg2
		svdres (jfile, ofile)

	# Delete intermediate file
		if (delf)
		    imdelete (jfile)
		}
	    }

	if (tchar != '@')
	    delete (filelist)
end

# May 20 1999	New task
# Jul 15 1999	Add continuum averaging parameters
# Jul 30 1999	Add per-pass renormalization and fixbad parameters
# Sep  8 1999	Set badlines
# Sep 17 1999	Change eigenres task to svdres
# Oct  8 1999	Fix bug which failed to set titles
