# File svdfit/esky2.cl
# February 24, 2000
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# ESKY2 -- Remove sky from spectra by fitting eigenvectors and removing the
#	  fit for a file or list of files

procedure esky1 (infile)

string	infile=""	{prompt="Input spectrum file"}
string	specnum="1"	{prompt="Range of spectra in multispec or 2D file"}
int	specband=1	{prompt="Band if multispec file"}
string	indir=""	{prompt="Input directory"}
string	outform1="twodspec" {prompt="First pass format (multispec|oned|twod)"}
string	intfile=""	{prompt="Intermediate spectrum file"}
file	vecdir="svdfit$lib/"	{prompt="Directory for eigenvectors"}
file	vecfile1="evec1.fits"	{prompt="First pass eigenvector file"}
string	vectors1="1-2"	{prompt="First pass eigenvectors"}
string	fixbad1="zero"	{prompt="Fix bad lines in first pass (zero fill no)"}
real	renorm1=1.0	{prompt="Renormalize (divide by mean) in first pass"}
int	order1=40	{prompt="First pass continuum order"}
bool	delfile=yes	{prompt="Delete intermediate files (y or n)"}
file	vecfile2="evec2.fits"	{prompt="Second pass eigenvector file"}
string	fixbad2="zero"	{prompt="Fix bad lines in second pass (zero fill no)"}
real	renorm2=0.0	{prompt="Renormalize (divide by mean) in second pass"}
string	vectors2="1-10"	{prompt="Second pass eigenvectors"}
int	order2=40	{prompt="Second pass continuum order"}
string	outform2="twodspec" {prompt="Second pass format (multispec|oned|twod)"}
file	badlines="badskynao.dat" {prompt="File with lines to eliminate"}
string	outfile=""	{prompt="Output spectrum file"}
string	outdir=""	{prompt="Output directory"}
string	outform=""	{prompt="Final spectrum format, if not outform2"}
bool	plot=yes	{prompt="Plot results on display (y or n)"}
bool	cursor=no	{prompt="Wait after plot with cursor (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	debug=no	{prompt="Debugging mode (y or n)"}

begin

bool	vb, delf, makeout, delint
int	i, nfile
file	filelist, odir, sfile, ffile
file	localfile, ifile, jfile, ofile
file	listfile, eigtemp
file	cfile, tfile, spectra
string	tchar, object, oform1, oform2, oform

	vb = verbose
	ifile = infile
	ofile = outfile
	odir = outdir
	oform1 = outform1
	oform2 = outform2
	oform = outform
	makeout = no
	if (ofile == "") {
	    makeout = yes
	    }
	delint = no
	sfile = "skytemp.fits"
	jfile = intfile
	if (jfile == "") {
	    jfile = "svdpass1out.fits"
	    delint = yes
	    }
	delf = delfile

	svdprep.save_names = no
	svdprep.svel_corr = "none"
	svdres.specband = specband
	svdres.specnum = specnum
	svdres.debug = debug
	svdres.res_plot = plot
	svdres.plot_int = cursor
	svdres.badlines = badlines

	set clobber = yes

#  Make file containing list of input files, if there is not already one
	tchar = substr (ifile,1,1)
	if (tchar != '@') {
	    filelist = mktemp ("eiglist")
	    print (ifile,>filelist)
	    listfile = filelist
	    }
	else
	    listfile = substr(ifile,2,64)

	if (vb)
	    print ("ESKY: files from ",listfile)
 
#  Count files in list
	list = listfile
	nfile = 0
	while (fscan (list, line) != EOF) {
	    nfile = nfile + 1
	    }
	svdprep.nspec = nfile

	svdprep.specband = specband
	svdprep.specnum = specnum
	if (tchar == '@') {
	    spectra = ifile
	    }
	else {
	    spectra = "@" // filelist
	    }
	cfile = "svdpass1in.fits"

	# Set up first pass continuum-removed file
	svdprep.compname = "object spectra - continuum"
	svdprep.fixbad = fixbad1
	svdprep.normout = renorm1
	contprep.order = order1
	svdprep.cont_remove = "subtract"
	svdprep (spectra, cfile)

	# Fit first pass eigenvectors
	if (nfile > 1)
	    svdres.specnum = "1-" // nfile
	svdres.outname = "SVD pass 1 residuals"
	svdres.eigendir = vecdir
	svdres.eigenfiles = vecfile1
	svdres.eigenum = vectors1
	svdres.fixbad = fixbad1
	svdres.renormalize = 0.0
	svdres.cont_remove = "no"
	if (nfile > 1)
	    svdres.output_format = "twod"
	else
	    svdres.output_format = oform1
	svdres (cfile, jfile)

	# Set up second pass continuum-removed file
	tfile = "svdpass2in.fits"
	if (nfile > 1)
	    svdprep.specnum = "1-" // nfile
	svdprep.compname = "SVD pass 1 residuals - continuum"
	svdprep.fixbad = fixbad2
	svdprep.normout = renorm2
	contprep.order = order2
	svdprep.cont_remove = "subtract"
	svdprep (jfile, tfile)

	# Fit second pass eigenvectors
	svdres.eigendir = vecdir
	svdres.eigenfiles = vecfile2
	svdres.eigenum = vectors2
	svdres.fixbad = fixbad2
	svdres.renormalize = 0.0
	svdres.cont_remove = "no"
	svdres.outname = "SVD pass 2 residuals"
	contres.order = order2
	if (nfile > 1)
	    svdres.output_format = "twod"
	else
	    svdres.output_format = oform2
	if (makeout) {
	    ofile = "svdpass2out.fits"
	    }
	svdres (tfile, ofile)
	if (delint) {
	    imdelete (tfile)
	    imdelete (cfile)
	    imdelete (jfile)
	    }

	# If multiple input files, asume they are one-dimensional
	if (nfile > 1) {
	    list = listfile
	    i = 0

	    # Extract files one at a time
	    while (fscan (list, localfile) != EOF) {
		fileroot (localfile)
		ffile = odir // fileroot.root // "res"
		i = i + 1
		if (verbose)
		    args = "-vfo " // ffile // " " // ofile // " " // i
		else
		    args = "-fo " // ffile // " " // ofile // " " // i
		imextract (args)

		# Write multispec file if requested to
		ffile = outdir // fileroot.root // "res.fits" 
		if (oform == "multispec") {
		    imarith (localfile, "-", ffile, sfile)
		    args = ofile // "," // ifile // "," // sfile
		    imstack (args)
		    rename (imstack.fits, ffile)
		    imdel (sfile)
		    }

		# Reset header parameters to match this file's input
		arg = localfile // " " // ffile
		arg = arg // " object ra dec epoch type fiber aprange fiber_x fiber_y"
		chead (arg)
		}
	    if (makeout)
		imdelete (ofile)
	    }
	else if (makeout) {
	    fileroot (localfile)
	    ffile = odir // fileroot.root // "res.fits"
	    rename (ofile, ffile)
	    }

	if (tchar != '@')
	    delete (filelist)
end

# Feb 24 2000	New task, based on esky1.cl
