# File svdfit/Svdprep/t_svdprep.x
# May 8, 2008
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 2008 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
 
# SVDPREP is an IRAF task for combining spectra with different velocities
# For arguments, see parameter file svdprep.par.
# Information is shared in common blocks defined in "rvsao.com".
 
include	<imhdr.h>
include	<imio.h>
include	<fset.h>
include "rvsao.h"
include "contin.h"

procedure t_svdprep ()

int	i
char	spectra[SZ_PATHNAME]	# List of input spectra
char	specfile[SZ_PATHNAME]	# Input spectrum file name
char	specdir[SZ_PATHNAME]	# Directory for input spectra
char	specpath[SZ_PATHNAME]	# Input spectrum path name
char	compfile[SZ_PATHNAME]	# Output spectrum file name
char	compdir[SZ_PATHNAME]	# Output spectrum directory
char	comppath[SZ_PATHNAME]	# Output spectrum pathname
#char	filt_type[SZ_LINE]	# Filter for transform
				# (ramp | welch | hanning | cos-bell)
char	svel_corr[SZ_LINE]	# Type of velocity correction for spectrum
				# (none | file | heliocentric | barycentric)
char	contstr[SZ_LINE]	# Remove continuum
  				# (no subtract divide)
char	fstring[SZ_LINE]
int	logfiles		# List of log files
char	logfile[SZ_PATHNAME]	# Log file name
char	wtitle[20]		# Title for wavelength plots of spectrum

int	mspec		# Object aperture to read from multispec file
int	mband		# Object band to read from multispec file

pointer	speclist	# List of spectrum files
pointer	compspec	# Summed spectrum
char	str[SZ_LINE]
int	fd
int	ldir

int	nmspec0		# Number of object multispec spectra
int	mspec_range[3,MAX_RANGES]
int	ip,jp,lfile	# Limits for multispec aperture decoding
double	zcomp
char	lbracket[3]	# "[({"
char	rbracket[3]	# "])}"
 
bool	clgetb()
real	clgetr()
int	clpopnu(), clgeti(), clgfil(), open()
double	clgetd()
int	strdic(), stridx(), stridxs()
int	decode_ranges(),get_next_number()
int	imtgetim(), imaccess(), strlen()
pointer	imtopen()
pointer compim		# Output spectrum header structure
int	ispec, jspec
bool	fproc, perspec
int	lcomp
int	nspec		# Number of spectra in output file
bool	specint, compint
double	minwav, maxwav
double	dindef

define	newspec_ 10
define	newap_	 20
define	endcomp_	 90

include	"rvsao.com"
include	"contin.com"
include	"sum.com"
 
begin
	call sprintf (lbracket,3,"[({")
	call sprintf (rbracket,3,"])}")
	call sprintf (wtitle,20,"Wavelength")
	c0 = 299792.5
	dindef = INDEFD
	nlogfd = 0
	compim = NULL
	velhc = dindef
	fproc = FALSE
	perspec = FALSE

# Get task parameters.

# Spectra to combine
	call clgstr ("spectra",spectra,SZ_PATHNAME)
	speclist = imtopen (spectra)

# Multispec spectrum apertures (use only first if multiple files)
	call clgstr ("specnum",specnums,SZ_LINE)
	if (decode_ranges (specnums, mspec_range, MAX_RANGES, nmspec0) == ERR)
	    call error (1, "SVDPREP: Illegal multispec list")
	call clgstr ("specdir",specdir,SZ_PATHNAME)
	ldir = strlen (specdir)
	if (specdir[1] != EOS && specdir[ldir] != '/') {
	    specdir[ldir+1] = '/'
	    specdir[ldir+2] = EOS
	    }
	mband = clgeti ("specband")

# Number of output spectra in output file
	nspec = clgeti ("nspec")

# File to which to write summed spectrum
	call clgstr ("compfile",compfile,SZ_PATHNAME)
	lcomp = strlen (compfile)
	call clgstr ("compdir",compdir,SZ_PATHNAME)
	ldir = strlen (compdir)
	if (ldir > 0) {
	    if (compdir[ldir] != '/') {
		compdir[ldir+1] = '/'
		compdir[ldir+2] = EOS
		}
	    call strcpy (compdir,comppath,SZ_PATHNAME)
	    call strcat (compfile,comppath,SZ_PATHNAME)
	    }
	else
	    call strcpy (compfile,comppath,SZ_PATHNAME)
	save_names = FALSE
	save_names = clgetb ("save_names")

# Optional intermediate data plot switches
	pltspec = clgetb ("spec_plot")
	plttemp = clgetb ("comp_plot")

# Print processing information
	debug  = clgetb ("debug")

# Continuum fit parameter pset
	call cprep_get_pars()
	pltcon  = clgetb ("cont_plot")

# Number of times to smooth (1-2-1) final data plot
	nsmooth = clgeti ("nsmooth")

# Minimum and maximum values for data in graphs
	ymin = clgetr ("ymin")
	ymax = clgetr ("ymax")

# Keep header from first spectrum
	copyhead = TRUE
	copyhead = clgetb ("copy_header")
	if (nspec == 1)
	    copyhead = TRUE

# Interact with display of final composite spectrum
	compint = clgetb ("comp_int")
	if (compint)
	    tsmooth = nsmooth
	else
	    tsmooth = -(nsmooth + 1)

# Interact with display of input spectra
	specint = clgetb ("spec_int")
	if (!specint)
	    nsmooth = -(nsmooth + 1)

# Type of heliocentric velocity correction to be used
	call clgstr ("svel_corr",svel_corr,SZ_LINE)
	svcor = strdic (svel_corr,svel_corr,SZ_LINE, HC_VTYPES)

# Optional wavelength limits for output spectrum
	minwav0 = clgetd ("st_lambda")
	maxwav0  = clgetd ("end_lambda")
	if (minwav0 == dindef || maxwav0 == dindef) {
	    call wlrange (spectra, minwav, maxwav)
	    if (minwav0 == dindef)
		minwav0 = minwav
	    if (maxwav0 == dindef)
		maxwav0 = maxwav
	    }

# Number of points in created spectrum
	npts = clgeti ("npts")
	if (debug || lcomp == 0 || npts < 3) {
	    call printf ("SVDPREP: %d points in spectrum from %.3fA to %.3fA\n")
		call pargi (npts)
		call pargd (minwav0)
		call pargd (maxwav0)
	    call flush (STDOUT)
	    }
	if (lcomp == 0 || npts < 3)
	    go to endcomp_

# Redshift velocity for composite spectrum
	velocity = clgetd ("velcomp")
	zcomp = clgetd ("zcomp")
	if (zcomp != dindef)
	    velocity = c0 * zcomp
	if (debug) {
	    call printf ("SVDPREP: Z = %.5f, CZ = %.2f\n")
		call pargd (zcomp)
		call pargd (velocity)
	    call flush (STDOUT)
	    }

# Spectrum continuum removal and line rejection
	schop = clgetb ("reject")
	emrej = clgetr ("em_reject")
	abrej = clgetr ("abs_reject")
	call clgstr ("cont_remove",contstr,SZ_LINE)
	conproc = strdic (contstr, contstr, SZ_LINE, CONT_TYPE)
	if (debug) {
	    call printf ("SVDPREP: Input spectra continuum ")
	    if (conproc == DIVCONT)
		call printf ("divided\n")
	    else if (conproc == SUBCONT)
		call printf ("subtracted\n")
	    else if (conproc == ZEROCONT)
		call printf ("divided - 1\n")
	    else
		call printf ("not removed\n")
	    }

# Bad line removal
        call clgstr ("fixbad", fstring, SZ_LINE)
	badproc = strdic (fstring, fstring, SZ_LINE, BAD_TYPE)
	if (debug) {
	    call printf ("SVDPREP: Input spectra bad lines ")
	    if (badproc == FILLBAD)
		call printf ("filled\n")
	    else if (badproc == ZEROBAD)
		call printf ("zeroed\n")
	    else if (badproc == NZEROBAD)
		call printf ("filled then zeroed\n")
	    else
		call printf ("not removed\n")
	    }

# Open log files and write a header.
	logfiles = clpopnu ("logfiles")
	call fseti (STDOUT, F_FLUSHNL, YES)
	i = 0
	call strcpy ("svdfit.svdprep",taskname,SZ_LINE)
	while (clgfil (logfiles, logfile, SZ_PATHNAME) != EOF) {
	    fd = open (logfile, APPEND, TEXT_FILE)
	    if (fd == ERR) break
	    call loghead (taskname,str)
	    call fprintf (fd, "%s\n")
		call pargstr (str)
	    i = i + 1
	    logfd[i] = fd
	    }
	nlogfd = i
	call clpcls (logfiles)
	ispec = 0

# Buffer vectors for continuum removal
	if (conproc != NOCONT) {
	    call malloc (contvec, npts, TY_REAL)
	    call malloc (contbuf, npts, TY_REAL)
	    call malloc (contwts, npts, TY_REAL)
	    call malloc (contrej, npts, TY_REAL)
	    }

# Get next object spectrum file name from the list
newspec_
	if (imtgetim (speclist, specfile, SZ_PATHNAME) == EOF)
	   go to endcomp_

# Check for specified apertures in multispec spectrum file
	ip = stridxs (lbracket,specfile)
	if (ip > 0) {
	    lfile = strlen (specfile)
	    specfile[ip] = EOS
	    ip = ip + 1
	    jp = 1
	    while (stridx (specfile[ip],rbracket) == 0 && ip <= lfile) {
		specnums[jp] = specfile[ip]
		specfile[ip] = EOS
		ip = ip + 1
		jp = jp + 1
		}
	    specnums[jp] = EOS
	    if (decode_ranges (specnums,mspec_range,MAX_RANGES,nmspec) == ERR)
		call error (1, "SVDPREP: Illegal multispec list")
	    }
	else
	    nmspec = nmspec0
	if (debug) {
	    call printf ("SVDPREP: next file is %s [%s] = %d aps\n")
		call pargstr (specfile)
		call pargstr (specnums)
		call pargi (nmspec)
	    call flush (STDOUT)
	    }

# Check for readability of object spectrum
	call strcpy (specdir,specpath,SZ_PATHNAME)
	call strcat (specfile,specpath,SZ_PATHNAME)
	if (imaccess (specpath, READ_ONLY) == NO) {
	    call eprintf ("SVDPREP: cannot read spectrum file %s \n")
		call pargstr (specpath)
	    go to newspec_
	    }

# Get next multispec number from list
	mspec = -1
newap_
	if (nmspec <= 0)
	    go to newspec_
	if (get_next_number (mspec_range, mspec) == EOF)
	    go to newspec_

	ispec = ispec + 1
	call addspec (specfile,specdir,mspec,mband,nspec,fproc,
		      compim,comppath,compspec,ispec,nspec,perspec)

	if (nspec > 1)
	    call tmp_write_iraf (compim, ispec, compspec, TY_REAL, debug)

# Move on to next aperture or next image
	nmspec = nmspec - 1
	if (nmspec > 0)
	    go to newap_
	go to newspec_
 
endcomp_

# If fewer than nspec spectra read, fill out the file with zeroes
	if (ispec < nspec) {
	    call aclrr (Memr[compspec], npts)
	    do jspec = ispec+1, nspec {
		call tmp_write_iraf (compim, ispec, compspec, TY_REAL, debug)
		}
	    }

# Close the log files
	if (nlogfd > 0) {
	    do i = 1, nlogfd {
		call close (logfd[i])
		}                                              
	    }

#  Close spectrum list
	call imtclose (speclist)

#  Close the output composite spectrum file
	if (compim != NULL) {
	    if (nspec > 1) {
		call imunmap (compim)
		call mfree (compspec, TY_REAL)
		}
	    else
		call tmp_close (compim,compspec,debug)
	    }

#  Free memory used by continuum buffers
	if (conproc != NOCONT) {
	    call mfree (contvec, TY_REAL)
	    call mfree (contbuf, TY_REAL)
	    call mfree (contwts, TY_REAL)
	    call mfree (contrej, TY_REAL)
	    }

end
# Sep 17 1999	New task copied from rvsao.sumspec

# Jan 18 2000	Set up continuum buffers outside of loop
# Jan 27 2000	Fix logging message
# Apr 25 2000	Fix bad line diagnostic print statements

# Nov 21 2001	Initialize velhc

# May  8 2008	Use latest version of addspec() from RVSAO
# May  8 2008   Add ymin and ymax to scale all graphs the same
