# File svdfit/Svdres/countspec.x
# April 28, 2000
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
 
# COUNTSPEC counts the number of spectra to be processed given a
# file list and aperture range.

# Copyright(c) 2000 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
 
include	<imhdr.h>
include	<imio.h>
include	<fset.h>
include	<smw.h>
include "rvsao.h"

procedure countspec (specfiles, specdir, specnums0, specband, debug, nspec)

char	specfiles[ARB]		# List of spectra
char	specdir[ARB]		# Directory containing spectra
char	specnums0[ARB]		# List of multispec spectra to read
int	specband		# Band of multispec spectrum
bool	debug
int	nspec			# Number of spectra (returned)

pointer	speclist		# List of spectra
char	specfile[SZ_PATHNAME]	# Spectrum file name
char	specpath[SZ_PATHNAME]	# Spectrum path name
char	specnums[SZ_LINE]	# List of multispec spectra to read
int     ntspec,ntspec0		# Number of spectrum multispec spectra
int     tspec_range[3,MAX_RANGES]
int	tspec			# Template spectrum to read from multispec file
int	ip, jp, lfile
char	lbracket[3]	# left bracket, parenthesis, brace
char	rbracket[3]	# right bracket, parenthesis, brace
char	cdot

int	stridx(), stridxs()
int	decode_ranges(),get_next_number()
int	imtgetim(), imaccess(), strlen()
pointer	imtopen()

define  newspec_ 40
define  newtap_  50
define  endspec_ 60

begin
	call sprintf (lbracket,3,"[({")
	call sprintf (rbracket,3,"])}")
	cdot = '.'

	if (decode_ranges (specnums0, tspec_range, MAX_RANGES, ntspec0) == ERR)
	    call error (1, "COUNTSPEC: Illegal multispec list")
	nspec = 0
	speclist = imtopen (specfiles)

# Initialize new spectrum file
newspec_
	tspec = -1

# Get next spectrum spectrum file name from the list
	if (imtgetim (speclist, specfile, SZ_PATHNAME) == EOF)
	    go to endspec_

# Check for specified apertures in multispec spectrum file
	ip = stridxs (lbracket,specfile)
	if (ip > 0) {
	    lfile = strlen (specfile)
	    specfile[ip] = EOS
	    ip = ip + 1
	    jp = 1
	    while (stridx (specfile[ip],rbracket) == 0 && ip <= lfile) {
		specnums[jp] = specfile[ip]
		specfile[ip] = EOS
		ip = ip + 1
		jp = jp + 1
		}
	    specnums[jp] = EOS
	    if (decode_ranges (specnums,tspec_range,MAX_RANGES,ntspec) == ERR)
		call error (1, "COUNTSPEC: Illegal spectrum multispec list")
	    }
	else {
	    call strcpy (specnums0,specnums,SZ_LINE)
	    if (decode_ranges (specnums,tspec_range,MAX_RANGES,ntspec) == ERR)
		call error (1, "COUNTSPEC: Illegal spectrum multispec list")
	    }
	if (debug) {
	    call printf ("COUNTSPEC: spectrum file %d is %s [%s] = %d aps\n")
		call pargi (nspec+1)
		call pargstr (specfile)
		call pargstr (specnums)
		call pargi (ntspec)
	    call flush (STDOUT)
	    }

# Check for readability of spectrum spectrum
	if (stridx ("/",specfile) == 1 || stridx ("$",specfile) > 0 ||
	    stridx (cdot,specfile) == 1)
	    call strcpy (specfile,specpath,SZ_PATHNAME)
	else {
	    call strcpy (specdir,specpath,SZ_PATHNAME)
	    call strcat (specfile,specpath,SZ_PATHNAME)
	    }
	if (imaccess (specpath, READ_ONLY) == NO) {
	    call eprintf ("COUNTSPEC: cannot read spectrum file %s \n")
		call pargstr (specpath)
	    go to newspec_
	    }
	if (debug) {
	    call printf ("COUNTSPEC: spectrum path %d is %s\n")
		call pargi (nspec+1)
		call pargstr (specpath)
	    call flush (STDOUT)
	    }

# Get next multispec aperture number from list
newtap_
	if (get_next_number (tspec_range, tspec) == EOF)
	    go to newspec_

# Add this spectrum to counter
	nspec = nspec + 1
	if (debug) {
	    call printf ("COUNTSPEC: %d spectra\n")
		call pargi (nspec)
	    call flush (STDOUT)
	    }

	ntspec = ntspec - 1
	if (ntspec > 0)
	    go to newtap_
	go to newspec_

endspec_
	call imtclose (speclist)
	if (debug) {
	    call printf ("COUNTSPEC: counted %d spectra\n")
		call pargi (nspec)
	    call flush (STDOUT)
	    }
	return

end

# Apr 28 2000	Separated for t_svdres.x
