# File svdfit/Svdres/putspec.x
# October 15, 2007
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1999-2007 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
 
# PUTSPEC writes a spectrum into a file
 
include	<imhdr.h>
include	<imio.h>
include	<fset.h>
include <smw.h>
include <error.h>
include <imset.h>
include <fio.h>
include <mach.h>
include "rvsao.h"

procedure putspec (imin,inpath,inap,inband,imout,outpath,spectrum,npix,
		   ispec,nspec)

pointer imin		# Input spectrum image header structure
char	inpath[ARB]	# Object spectrum path name
int	inap		# Input aperture
int	inband		# Input band
pointer imout		# Output image header structure
char	outpath[ARB]	# Out spectrum file name
real	spectrum[npix]	# Output spectrum
int	npix		# Number of pixels in output spectrum
int	ispec		# Number of this spectrum in output file
int	nspec		# Number of spectra stacked in output file

char	instrument[SZ_LINE]

char	tstring[SZ_LINE]
char	keyword[16]
char	temp[SZ_LINE]
char	outname[SZ_IMTITLE]	# Title for output spectrum

double	exp	# Exposure time in seconds
double	dindef
double	vsky
double	ra0,ra,dec0,dec
double	dtemp
char	dstr[SZ_LINE]
double	alt
int	isp
int	naxes

int	imaccess(), strlen(), imaccf(), immap()

define	endspec_	 90
include	"rvsao.com"
 
begin
	dindef = INDEFD
	c0 = 299792.5
	tstring[1] = EOS

	call logtime (dstr,SZ_LINE)
	if (debug) {
	    call printf ("PUTSPEC: Spectrum pointer is %d\n")
	 	call pargi (imout)
	    call flush (STDOUT)
	    }

# If output file is not yet open, open it
	if (imout == NULL) {
	    imout = immap (outpath, NEW_COPY, imin)
#	    call put_open (imout, outpath, npix, nspec)
	    if (imout == ERR) {
		call eprintf ("PUTSPEC: Error writing spectrum %s\n")
	 	    call pargstr (outpath)
		call flush (STDERR)
		go to endspec_
		}
	    naxes = 2
	    call imgipar (imout, "i_naxis", naxes)
	    call imaddi (imout, "i_naxis", 2)
#	    call imdelf (imout, "NAXIS3")
	    if (debug) {
		call printf ("PUTSPEC: Spectrum %s opened\n")
	 	    call pargstr (outpath)
		call flush (STDOUT)
		}

#  Write title or object name to output header
	    call clgstr ("resname",outname,SZ_IMTITLE)
	    if (strlen (outname) <= 0)
		call strcpy (IM_TITLE(imin),IM_TITLE(imout),SZ_IMTITLE)
	    else
		call strcpy (outname,IM_TITLE(imout),SZ_IMTITLE)

#  Write sky position to output header
	    ra0 = dindef
	    if (imaccf (imin, "RA") == YES) {
		call imgstr (imin, "RA", tstring,SZ_LINE)
		call imastr (imout, "RA", tstring)
		call sscan (tstring)
		    call gargd (ra0)
		}
	    else
		call strcpy ("0:00:00", tstring, SZ_LINE)
	    dec0 = dindef
	    if (imaccf (imin, "DEC") == YES) {
		call imgstr (imin, "DEC", tstring,SZ_LINE)
		call imastr (imout, "DEC", tstring)
		call sscan (tstring)
		    call gargd (dec0)
		}
	    else
		call strcpy ("0:00:00", tstring, SZ_LINE)
	    if (imaccf (imin, "EPOCH") == YES) {
		call imgdpar (imin, "EPOCH", dtemp)
		call imaddd (imout, "EPOCH", dtemp)
		}
	    if (imaccf (imin, "EQUINOX") == YES) {
		call imgdpar (imin, "EQUINOX", dtemp)
		call imaddd (imout, "EQUINOX", dtemp)
		}
	    else
		dtemp = 1950.0

# Write observatory and instrument to output header
	    if (imaccf (imin, "INSTRUME") == YES) {
		call imgstr (imin, "INSTRUME", instrument,SZ_LINE)
		call imastr (imout, "INSTRUME", instrument)
		}
	    if (imaccf (imin, "SITENAME") == YES) {
		call imgstr (imin, "SITENAME", tstring,SZ_LINE)
		call imastr (imout, "SITENAME", tstring)
		call imgstr (imin, "SITELONG", tstring,SZ_LINE)
		call imastr (imout, "SITELONG", tstring)
		call imgstr (imin, "SITELAT", tstring,SZ_LINE)
		call imastr (imout, "SITELAT", tstring)
		alt = 0.d0
		call imgdpar (imin, "SITEELEV",alt)
		call imaddd (imout, "SITEELEV", alt)
		}

# Write exposure time to output header
	    exp = 0.d0
	    call imgdpar (imin, "EXPTIME",exp)
	    if (exp .le. 0.d0)
		call imgdpar (imin, "EXPOSURE", exp)
	    if (exp .le. 0.d0)
		call imgdpar (imin, "ITIME", exp)
	    if (exp < .001d0)
		exp = 1.d0
	    call imaddd (imout, "EXPTIME", exp)


	    call imaddi (imout, "DISPAXIS", 1)
	    call imaddi (imout, "DC-FLAG", 0)
	    call imastr (imout, "CTYPE1","LINEAR")
	    call imaddr (imout, "CRPIX1", 1.)
	    call imaddd (imout, "CRVAL1", twl1[1])
	    call imaddd (imout, "CDELT1", dwl)
	    call imastr (imout, "CTYPE2","LINEAR")
	    call imaddi (imout, "CRPIX2", 1)
	    call imaddi (imout, "CRVAL2", 1)
	    call imaddi (imout, "CDELT2", 1)

	    call imaddi (imout, "NP1", 1)
	    call imaddi (imout, "NP2", npix)
	    call sprintf (tstring,SZ_LINE,"SVDRES 1.3 run %s")
		call pargstr (dstr)
	    call imastr (imout, "SVDRES", tstring)
	    if (imaccf (imin, "VSKY1") == YES) {
		do isp = 1, nspec {
		    call sprintf (keyword, 16, "VSKY%d")
			call pargi (isp)
		    call imgdpar (imin, keyword, vsky)
		    call imaddd (imout, keyword, vsky)
		    }
		}

	    if (debug) {
		call printf ("PUTSPEC: Spectrum header updated\n")
		call flush (STDOUT)
		}
	    }

	else if (debug) {
	    call printf ("PUTSPEC: Spectrum %s appears to be open already\n")
	 	call pargstr (outpath)
	    call flush (STDOUT)
	    }

	if (ispec <= 1) {

	    # Write input spectrum to file
	    call putspeciraf (imout, ispec, spectrum, debug)

	    # Note files if in debug mode
	    if (debug) {
		call printf ("PUTSPEC %s making %s\n")
		    call pargstr (dstr)
		    call pargstr (outpath)
		call printf ("PUTSPEC %d: %s")
		    call pargi (ispec)
		    call pargstr (inpath)
		if (inap > 0) {
		    if (inband > 0) {
			call printf ("[%d,%d]")
			    call pargi (inap)
			    call pargi (inband)
			}
		    else {
			call printf ("[%d]")
			    call pargi (inap)
			}
		    }
		call printf (" written to %s")
		    call pargstr (outpath)
		if (nspec > 1) {
		    call printf ("[%d]\n")
			call pargi (ispec)
		    }
		else
		    call printf ("\n")
		}
	    }

	else {

	# Compare positions; if different from last, delete position from header
	    if (imaccf (imin, "RA") == YES) {
		call imgstr (imin, "RA", tstring,SZ_LINE)
		call sscan (tstring)
		    call gargd (ra)
		}
	    else
		ra = ra0
	    if (imaccf (imin, "DEC") == YES) {
		call imgstr (imin, "DEC", tstring,SZ_LINE)
		call sscan (tstring)
		    call gargd (dec)
		}
	    else
		dec = dec0
	    if (ra != ra0 || dec != dec0) {
		if (imaccf (imout, "RA") == YES)
		    call imdelf (imout, "RA")
		if (imaccf (imout, "DEC") == YES)
		    call imdelf (imout, "DEC")
		if (imaccf (imout, "EPOCH") == YES)
		    call imdelf (imout, "EPOCH")
		if (imaccf (imout, "EQUINOX") == YES)
		    call imdelf (imout, "EQUINOX")
		}

	    # Write input spectrum to file
	    call putspeciraf (imout, ispec, spectrum, debug)

	    # Print diagnostic message
	    if (debug) {
		call printf ("PUTSPEC %d: %s")
		    call pargi (ispec)
		    call pargstr (inpath)
		if (inap > 0) {
		    if (inband > 0) {
			call printf ("[%d,%d]")
			    call pargi (inap)
			    call pargi (inband)
			}
		    else {
			call printf ("[%d]")
			    call pargi (inap)
			}
		    }
		call printf (" written to %s")
		    call pargstr (outpath)
		if (nspec > 1) {
		    if (ispec < nspec) {
			call printf ("[%d]\r")
			    call pargi (ispec)
			}
		    else {
			call printf ("[%d]\n")
			    call pargi (ispec)
			}
		    }
		else 
		    call printf ("\n")
		}
	    }

# Note addition of this spectrum to the composite spectrum
	call sprintf (tstring,SZ_LINE,"Add %s %s")
	    call pargstr (instrument)
	    call pargstr (inpath)
	if (inap > 0) {
	    if (inband > 0) {
		call sprintf (temp,SZ_LINE,"[%d,%d]")
		    call pargi (inap)
		    call pargi (inband)
		}
	    else {
		call sprintf (temp,SZ_LINE,"[%d]")
		    call pargi (inap)
		}
	    call strcat (temp, tstring, SZ_LINE)
	    }

# Check for writability if results are being saved in image header
	if (imaccess (outpath, READ_WRITE) == NO) {
	    call eprintf ("PUTSPEC: cannot write to %s; not saving results\n")
		call pargstr (outpath)
	    IM_UPDATE(imout) = NO
	    }
	else
	    IM_UPDATE(imout) = YES

endspec_
	return
end


procedure put_open (im, iraffile, npix, nspec)

pointer	im			# Pointer to output image structure (returned)
char	iraffile[ARB]		# IRAF template file name
int	npix			# Number of pixels in spectrum
int	nspec			# Number of spectra in output file

int	immap()
errchk	immap

begin

#  Create IRAF image header
	im = immap (iraffile, NEW_IMAGE, 0)
	if (im == NULL) return

# Initialize a few header parameters
	IM_PIXTYPE(im) = TY_REAL
	if (nspec < 1)
	    nspec = 1
	if (nspec > 1)
	    IM_NDIM(im) = 2
	else
	    IM_NDIM(im) = 1
	IM_LEN(im,1) = npix
	IM_PHYSLEN(im,1) = npix
	IM_LEN(im,2) = nspec
	IM_PHYSLEN(im,2) = nspec

	return
end


# PUTSPECIRAF --  Write spectrum pixels to IRAF image file

procedure putspeciraf (im, ispec, tempbuf, debug)

pointer	im		# IRAF image descriptor
int	ispec		# Number of spectrum in 2D spectrum to write
real	tempbuf[ARB]	# Spectrum data buffer
bool	debug		# True for diagnostic output

int	npix
long	v[IM_MAXDIM], nlines
pointer	outline
real	linemax, linemin
long	clktime()

int	impnlr()
errchk	malloc

begin
	if (IM_NDIM(im) == 0) {
	    call printf ("PUTSPECIRAF: No pixel file created\n")
	    return
	    }

	npix = IM_LEN(im, 1)
	nlines = 1

	IM_MAX(im) = -MAX_REAL
	IM_MIN(im) = MAX_REAL

	call amovkl (long(1), v, IM_MAXDIM)
	if (ispec > 1)
	    v[2] = long (ispec)

	if (debug) {
	    call printf ("PUTSPECIRAF:  %d pixels, type %d\n")
	    call pargi (npix)
	    call pargi (IM_PIXTYPE(im))
	    }

# Set output image line buffer
	if (impnlr (im, outline, v) == EOF)
	    call error (3, "PUTSPECIRAF: Error writing IRAF file")
	if (debug) {
	    call printf ("PUTSPECIRAF:  moving %d pixels to output buffer\n")
		call pargi (npix)
	    }

# Move pixels to output buffer
	call amovr (tempbuf, Memr[outline], npix)
	if (debug) {
	    call printf ("PUTSPECIRAF:  %d pixels moved to output buffer\n")
		call pargi (npix)
	    }

# Calculate image maximum and minimum
	call alimr (Memr[outline], npix, linemin, linemax)
	IM_MAX(im) = max (IM_MAX(im), linemax)
	IM_MIN(im) = min (IM_MIN(im), linemin)

	if (debug) {
	    call printf ("PUTSPECIRAF:  minimum is %f, maximum is %f\n")
	    call pargr (IM_MIN(im))
	    call pargr (IM_MAX(im))
	    }
	IM_CTIME(im) = clktime (long(0))
	return
end

# May 11 1999	New program, based on rvsao/Makespec/addspec.x and writetemp.x

# Jan  9 2000	Print correct program name in history
# May  4 2000	Read resname, not outname
# May 17 2000	Make all non-error messages contingent on debug flag

# Nov 21 2001	Add subscripts to twl1 and twl2, the wavelength limits

# May 31 2002	Copy VSKYn to from input file to residual file, if present

# Jan  4 2006	Initialize tstring so it comes out OK; add debugging code
# Jan  4 2006	Copy all of input header into output file

# Oct 15 2007	Change (debug == TRUE) to (debug)
