# File svdfit/Src/t_svdres2.x
# June 15, 2005
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
 
# SVDRES2 is an IRAF task for fitting eigenvectors to a spectrum, subtracting
# the residuals, removing the continuum and fitting another set of eigenvectors,
# and returning those residuals as a spectrum file.
# For arguments, see parameter file svdres2.par.
# Information is shared in common blocks defined in "rvsao.com".

# Copyright(c) 2000 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.
 
include	<imhdr.h>
include	<imio.h>
include	<fset.h>
include	<smw.h>
include "rvsao.h"
include "contin.h"
include "emv.h"
define	EIG_OUTFORM	"|none|oned|twod|multispec|"
define	EIG_NONE	1
define	EIG_ONED	2
define	EIG_TWOD	3
define	EIG_MULTISPEC	4

procedure t_svdres2 ()

char	specfiles[SZ_PATHNAME]	# Object spectrum file names
char	specfile[SZ_PATHNAME]	# Object spectrum file name
char	specpath[SZ_PATHNAME]	# Object spectrum path name
char	specdir[SZ_PATHNAME]	# Directory for object spectra
char	resfile[SZ_PATHNAME]	# Output spectrum file name
char	respath[SZ_PATHNAME]	# Output spectrum path name
char	resdir[SZ_PATHNAME]	# Directory for output spectra
char	coeffile[SZ_PATHNAME]	# Output fit coefficients filename
char	coefpath[SZ_PATHNAME]	# Output fit coefficients pathname
int	rmode			# Report format (1=normal,2=one-line)
int	logfiles		# List of log files
char	logfile[SZ_PATHNAME] # Log file name
char	wtitle[20]		# Title for wavelength plots of spectrum
int	specap			# Object aperture to read from multispec file
int	specband		# Object band to read from multispec file
pointer	speclist		# List of spectrum files
char	str[SZ_LINE]
int	fd
char	outform[SZ_LINE]	# Output file format
int	oform
int	nspecap,nspecap0	# Number of object multispec spectra
int	specap_range[3,MAX_RANGES]
int	ip,jp,lfile		# Limits for multispec aperture decoding
char	lbracket[3]		# "[({"
char	rbracket[3]		# "])}"
pointer	rebspec			# Allocated vector for rebinned spectra
pointer	rebwl			# Allocated vector for rebinned wavelengths
pointer	resspec1		# Allocated vector for spectra residuals
pointer	resspec2		# Allocated vector for spectra residuals
pointer	fitspec			# Allocated vector for removed eigenvector fit
int	ispec, nspec
int	lname, lroot
int	nbin, i
int	iv, ncv
pointer	imout
char	jdot
char	msfx[8]
bool	plotint
int	cfd1, cfd2
char	fstring[SZ_LINE]
 
bool	clgetb()
int	clpopnu(), clgeti(), clgfil(), open(), access()
int	strdic(), stridx(), stridxs(), strldx(), strmatch()
int	decode_ranges(),get_next_number()
int	imtgetim(), imaccess(), strlen(), strncmp(), ldir
pointer	imtopen()

define	newspec_ 10
define	newap_	 20
define	endeig_	 90

include	"rvsao.com"
include "svdvec.com"
include	"contin.com"
include	"emv.com"
 
begin
	c0 = 299792.5
	nfound = 0
	nctemp = 0
	ispec = 0
	imout = NULL
	do i = 1, 5 {
	    ncvec[i] = 0
	    nvec[i] = 0
	    }
	do i = 1, MAXTEMPS {
	    rebvec1[i] = NULL
	    rebvec2[i] = NULL
	    }

	jdot = '.'
	call strcpy (".ms.imh", msfx, 8)
	msfx[1] = jdot
	msfx[4] = jdot

	call sprintf (lbracket,3,"[({")
	call sprintf (rbracket,3,"])}")
	call sprintf (wtitle,20,"Wavelength")

# Get task parameters.

# Spectra to cross-correlate
	call clgstr ("spectra", specfiles,SZ_PATHNAME)

# Multispec spectrum numbers (use only first if multiple files)
	call clgstr ("specnum",specnums,SZ_LINE)
	if (decode_ranges (specnums, specap_range, MAX_RANGES, nspecap0) == ERR){
	    call sprintf (str, SZ_LINE, "SVDRES2: Illegal multispec list <%s>")
		call pargstr (specnums)
	    call error (1, str)
	    }
	call clgstr ("specdir",specdir,SZ_PATHNAME)
	ldir = strlen (specdir)
	if (specdir[1] != EOS && specdir[ldir] != '/') {
	    specdir[ldir+1] = '/'
	    specdir[ldir+2] = EOS
	    }
	specband = clgeti ("specband")

# Optional intermediate data plot switches
	pltspec = clgetb ("obj_plot")
	pltcon  = clgetb ("contsub_plot")
	plttemp = clgetb ("vec_plot")

# Print processing information
	debug  = clgetb ("debug")

# Number of times to smooth (1-2-1) final data plot
	nsmooth = clgeti ("nsmooth")

# Interact with graphics displays
        plotint = clgetb ("plot_int")
        if (plotint)
            nsmooth = nsmooth
        else
            nsmooth = -(nsmooth + 1)

# Format of output file
	call clgstr ("res_format",outform,SZ_LINE)
	oform = strdic (outform,outform,SZ_LINE, EIG_OUTFORM)
	if (debug) {
	    call printf ("SVDRES2: output file is type %d\n")
		call pargi (oform)
	    call flush (STDOUT)
	    }
	if (oform == EIG_NONE)
	    nspec = 0
	else if (oform == EIG_ONED)
	    nspec = 1
	else if (oform == EIG_MULTISPEC)
	    nspec = -3
	else {
	    call countspec (specfiles, specdir, specnums, specband, debug, nspec)
	    }

# Report mode for log file
	rmode = 1
	rmode = clgeti ("report_mode")

# Number of points into which to rebin spectrum and eigenvectors
	nbin = clgeti ("nbin")
	if (debug) {
	    call printf ("SVDRES2: nbin = %d\n")
		call pargi (nbin)
	    }

# Continuum fit parameters
	order1 =  clgeti ("cont_order1")
	order2 =  clgeti ("cont_order2")
	if (debug) {
	    call printf ("SVDRES2: Input spectra %d-order continuum subtracted\n")
		call pargi (order1)
	    call printf ("SVDRES2: Residual spectra %d-order continuum subtracted\n")
		call pargi (order2)
	    }

# Continuum fit parameter pset
	if (order1 > 0 || order2 > 0) {
	    call cres_get_pars()
	    if (order1 > 0)
		conproc1 = SUBCONT
	    else
		conproc1 = NOCONT
	    if (order2 > 0)
		conproc2 = SUBCONT
	    else
		conproc2 = NOCONT

# Allocate buffer vectors for continuum removal
	    call malloc (contvec, nbin, TY_REAL)
	    call malloc (contbuf, nbin, TY_REAL)
	    call malloc (contwts, nbin, TY_REAL)
	    call malloc (contrej, nbin, TY_REAL)
	    }
	else {
	    conproc1 = NOCONT
	    conproc2 = NOCONT
	    }
	if (debug) {
	    call printf ("SVDRES2: Input spectra continuum ")
	    if (order1 > 0)
		call printf ("subtracted before pass 1\n")
	    else
		call printf ("not removed before pass 1\n")
	    call printf ("SVDRES2: Input spectra continuum ")
	    if (order2 > 0)
		call printf ("subtracted before pass 2\n")
	    else
		call printf ("not removed before pass 2\n")
	    }

# Set bad line removal for first pass
	call clgstr ("fixbad1", fstring, SZ_LINE)
	badproc1 = strdic (fstring, fstring, SZ_LINE, BAD_TYPE)
	if (debug) {
	    call printf ("EIGENFIT2: Input spectra bad lines ")
	    if (badproc1 == FILLBAD)
		call printf ("filled\n")
	    else if (badproc1 == ZEROBAD)
		call printf ("zeroed\n")
	    else if (badproc1 == NZEROBAD)
		call printf ("filled then zeroed\n")
	    else
		call printf ("not removed\n")
	    }

# Set bad line removal for second pass
	call clgstr ("fixbad2", fstring, SZ_LINE)
	badproc2 = strdic (fstring, fstring, SZ_LINE, BAD_TYPE)
	if (debug) {
	    call printf ("EIGENFIT2: Residual spectra bad lines ")
	    if (badproc2 == FILLBAD)
		call printf ("filled\n")
	    else if (badproc2 == ZEROBAD)
		call printf ("zeroed\n")
	    else if (badproc2 == NZEROBAD)
		call printf ("filled then zeroed\n")
	    else
		call printf ("not removed\n")
	    }

# Allocate rebinned, wavelength, residual, and sky spectrum buffers
	call malloc (rebspec, nbin, TY_REAL)
	call malloc (rebwl, nbin, TY_REAL)
	call malloc (resspec1, nbin, TY_REAL)
	call malloc (resspec2, nbin, TY_REAL)
	call malloc (fitspec, nbin, TY_REAL)

# Initialize emission and absorption lines for labelling
	call eminit (FALSE)
 
# Open log files and write a header.
	logfiles = clpopnu ("logfiles")
	call fseti (STDOUT, F_FLUSHNL, YES)
	i = 0
	call strcpy ("svdfit.svdres",taskname,SZ_LINE)
	while (clgfil (logfiles, logfile, SZ_PATHNAME) != EOF) {
	    fd = open (logfile, APPEND, TEXT_FILE)
	    if (fd == ERR) break
	    if (rmode == 1) {
		call loghead (taskname,str)
		call fprintf (fd, "%s\n")
		    call pargstr (str)
		}
	    i = i + 1
	    logfd[i] = fd
	    }
	nlogfd = i
	call clpcls (logfiles)

# Spectra to cross-correlate
	speclist = imtopen (specfiles)

# Get next object spectrum file name from the list
newspec_
	if (imtgetim (speclist, specfile, SZ_PATHNAME) == EOF)
	   go to endeig_
	if (debug) {
	    call printf ("SVDRES2: next file is %s\n")
		call pargstr (specfile)
	    }

# Check for specified apertures in multispec spectrum file
	ip = stridxs (lbracket,specfile)
	if (ip > 0) {
	    lfile = strlen (specfile)
	    specfile[ip] = EOS
	    jp = 0
	    ip = ip + 1
	    while (stridx (specfile[ip],rbracket) == 0 && ip <= lfile) {
		jp = jp + 1
		specnums[jp] = specfile[ip]
		specfile[ip] = EOS
		ip = ip + 1
		}
	    if (jp > 0)
		specnums[jp+1] = EOS
	    else
		call strcpy ("0",specnums,SZ_LINE)
	    if (decode_ranges (specnums,specap_range,MAX_RANGES,nspecap) == ERR){
		call sprintf (str, SZ_LINE, "SVDRES2: Illegal multispec list <%s>")
		    call pargstr (specnums)
		call error (1, str)
		}
	    }
	else
	    nspecap = nspecap0
	if (debug) {
	    call printf ("SVDRES2: next file is %s [%s] = %d aps\n")
		call pargstr (specfile)
		call pargstr (specnums)
		call pargi (nspecap)
	    }

# Check for readability of object spectrum
	call strcpy (specdir,specpath,SZ_PATHNAME)
	call strcat (specfile,specpath,SZ_PATHNAME)
	if (imaccess (specpath, READ_ONLY) == NO) {
	    call eprintf ("SVDRES2: cannot read spectrum file %s \n")
		call pargstr (specpath)
	    go to newspec_
	    }

# Get or create name of output file
	call clgstr ("resfile",resfile,SZ_PATHNAME)
	lfile = strlen (resfile)
	if (lfile == 0) {
	    lname = strmatch (specfile, msfx)
	    if (lname > 0) {
		lname = lname - strlen (msfx)
		lroot = lname - 1
		if (nspec > 0)
		    lname = lname + 3
		}
	    else {
		lname = strldx (jdot, specfile)
		lroot = lname - 1
		}
	    if (lname == 0)
		lroot = lfile
	    call strcpy (specfile, resfile, lroot)
	    call strcat ("r2", resfile, SZ_PATHNAME)
	    if (lname > 0)
		call strcat (specfile[lname], resfile, SZ_PATHNAME)
	    }

# Get name of output directory
	call clgstr ("resdir",resdir,SZ_PATHNAME)
	ldir = strlen (resdir)

# Create directory if it does not already exist
	if (resdir[1] != EOS) {
	    if (resdir[ldir] != '/') {
		ldir = ldir + 1
		resdir[ldir+1] = EOS
		}
	    resdir[ldir] = EOS
	    if (access (resdir, 0, 0) == NO)
		call fmkdir (resdir)
	    resdir[ldir] = '/'
	    }

# Check for writability of output spectrum
	call strcpy (resdir,respath,SZ_PATHNAME)
	call strcat (resfile,respath,SZ_PATHNAME)

# Open file in which to save first pass coefficients
	cfd1 = -1
	call clgstr ("coeffile1",coeffile,SZ_PATHNAME)
	if (strlen (coeffile) <= 0) {
	    call strcpy (respath, coeffile, SZ_PATHNAME)
	    i = strmatch (coeffile, ".imh") 
	    if (i < 1)
		i = strmatch (coeffile, ".fit")
	    if (i > 0)
		coeffile[i-4] = char (0)
	    call strcat (".coeff1", coeffile, SZ_PATHNAME)
	    fcoef1 = FALSE
	    }
	else
	    fcoef1 = TRUE
	if (strncmp (coeffile, "none", 4) > 0) {
	    if (stridx ('/', coeffile) > 0)
		call strcpy (coeffile, coefpath)
	    else {
		call strcpy (resdir,coefpath,SZ_PATHNAME)
		call strcat (coeffile,coefpath,SZ_PATHNAME)
		}
	    call printf ("T_SVDRES2: Writing pass 1 coefficients to %s\n")
		call pargstr (coefpath)
	    cfd1 = open (coefpath, NEW_FILE, TEXT_FILE)
	    }

# Open file in which to save second pass coefficients
	cfd2 = -1
	call clgstr ("coeffile2",coeffile,SZ_PATHNAME)
	if (strlen (coeffile) <= 0) {
	    call strcpy (respath, coeffile, SZ_PATHNAME)
	    i = strmatch (coeffile, ".imh") 
	    if (i < 1)
		i = strmatch (coeffile, ".fit")
	    if (i > 0)
		coeffile[i-4] = char (0)
	    call strcat (".coeff2", coeffile, SZ_PATHNAME)
	    fcoef2 = FALSE
	    }
	else
	    fcoef2 = TRUE
	if (strncmp (coeffile, "none", 4) > 0) {
	    if (stridx ('/', coeffile) > 0)
		call strcpy (coeffile, coefpath)
	    else {
		call strcpy (resdir,coefpath,SZ_PATHNAME)
		call strcat (coeffile,coefpath,SZ_PATHNAME)
		}
	    call printf ("T_SVDRES2: Writing pass 2 coefficients to %s\n")
		call pargstr (coefpath)
	    cfd2 = open (coefpath, NEW_FILE, TEXT_FILE)
	    }

# Get next multispec number from list
	specap = -1
newap_
	if (nspecap <= 0) {
	    if (cfd1 > -1)
		call close (cfd1)
	    if (cfd2 > -1)
		call close (cfd2)
	    go to newspec_
	    }
	if (get_next_number (specap_range, specap) == EOF) {
	    if (cfd1 > -1)
		call close (cfd1)
	    if (cfd2 > -1)
		call close (cfd2)
	    go to newspec_
	    }
	ispec = ispec + 1

	if (debug) {
	    call printf ("SVDRES2: nbin = %d\n")
		call pargi (nbin)
	    }

	# One or two passes: binning, continuum removal, fitting, residuals
	call eigenfit2 (specfile, specdir, specap, specband, nbin,
		       Memr[rebspec], Memr[resspec1], Memr[resspec2],
		       Memr[fitspec], Memr[rebwl], ispec, nspec,
		       imout, respath, cfd1,cfd2)


	if (oform == EIG_ONED || oform == EIG_MULTISPEC) {
	    call imunmap (imout)
	    imout = NULL
	    }

# Move on to next aperture or next image
	nspecap = nspecap - 1
	if (nspecap > 0)
	    go to newap_
	if (cfd1 > -1)
	    call close (cfd1)
	if (cfd2 > -1)
	    call close (cfd2)
	go to newspec_
 
# Close the log files
endeig_	do i = 1, nlogfd {
	    call close (logfd[i])
	    }                                              

#  Close spectrum list
	call imtclose (speclist)
	if (imout != NULL && oform == EIG_TWOD)
	    call imunmap (imout)

# Close cached eignevector spectra
	ncv = ncvec[1] + ncvec[2]
	if (ncv > 0) {
	    do iv = 1, ncv {
		call close_image (tmpim[iv], tmpsh[iv])
		}
	    }
		    
#  Free memory used by spectrum buffers
	call mfree (rebspec, TY_REAL)
	call mfree (rebwl, TY_REAL)
	call mfree (resspec1, TY_REAL)
	call mfree (resspec2, TY_REAL)
	call mfree (skyspec, TY_REAL)
	if (conproc != NOCONT) {
	    call mfree (contvec, TY_REAL)
	    call mfree (contbuf, TY_REAL)
	    call mfree (contwts, TY_REAL)
	    call mfree (contrej, TY_REAL)
	    }

end
 
# May 18 2000	New task, based on t_svdres.x
# Jun 16 2000	Make residual directory if it does not exist already

# Jun 15 2005	Fix integer comparisons in if statements for Linux
