c*** File fitslib/hput.f
c*** April 19, 1993
c*** By Doug Mink

c  Implant value for variable VNAME into FITS header string
c  HSTRING, taking it from address IVAL.

c	HPUT   (HSTRING,VNAME,ITYPE,IVAL,COMMENT) sets any type
c	HPUTI2 (HSTRING,VNAME,IVAL) sets Integer*2 IVAL
c	HPUTI4 (HSTRING,VNAME,IVAL) sets Integer*4 IVAL
c	HPUTR4 (HSTRING,VNAME,RVAL) sets Real*4 RVAL
c	HPUTR8 (HSTRING,VNAME,DVAL) sets Real*8 DVAL
c	HPUTL  (HSTRING,VNAME,LVAL) sets Logical LVAL
c	HPUTS  (HSTRING,VNAME,CVAL) sets Character string CVAL adding ''
c	HPUTC  (HSTRING,VNAME,CVAL) sets Character string CVAL
c	HPUTCOM (HSTRING,VNAME,COMMENT) sets comment for parameter VNAME

	Subroutine HPUT (HSTRING,VNAME,ITYPE,IVAL,COMMENT)

	Character*(*) HSTRING
c		Character string containing FITS-style header information
c		in the format <VNAME>= <value> {/ <comment>}
c		Each Entry is padded with spaces to 80 characters

	Character*(*) VNAME
c		Character string containing the name of the variable
c		to be returned.  HPUT searches for a line beginning
c		with this string, and if it doesn't find one, creates one.
c		(THE FIRST 8 CHARACTERS OF A VARIABLE NAME MUST BE UNIQUE)

	Integer*4 ITYPE
c		Data type
c		  first ':' treated as '.', later ':'s and '.'s ignored.
c		  illegal characters, overflows, and underflows trapped. 
c		-1n: Integer n bytes long (Integer*n)
c		-2n: Floating point number n bytes long (Real*n)
c		-3n: Logical variable n bytes long (Logical*n) (t or f)
c		-4n: Logical variable n bytes long (Logical*n) (y or n)
c		1-99: Length of Character string returned, padded with spaces
c		101-199: Length (+100) of Character string returned
c		        padded with nulls
c		201-236: number (+200) of single characters
c			returned in array of 36 (A-Z,0-9) Integer*2
c			with array positions corresponding to letter
c			or number entered set to one.  Array is not
c			zeroed on entry.
c			Lower case letters are treated as upper case.
c		301-399: Length (+300) of Character string returned
c		        padded with nulls
c			Lower case letters are converted to upper case.

c	(Integer,Real) IVAL
c		Address for number, character, array, or string
c		If a Character string is to be set, call with
c		Equivalence'd Integer or use HPUTC; otherwise, Fortran adds
c		another invisible string length to the calling parameter list
c		and messes up everything.

	Character*(*) COMMENT
c		Character string containing comment for header

	Character*30 VALUE
c		Character string containing value from header

	Integer*2 IVAL(*)
	Integer*4 LVAL,IER

c  Translate value from binary to ASCII

	Call BINASC (LVAL,VALUE,ITYPE,IVAL,IER)

c  Put value and comment into header string

	Call HPUTC (HSTRING,VNAME,VALUE)
	Call HPUTCOM (HSTRING,VNAME,COMMENT)

c  Return to calling program

	Return
	End


	Subroutine HPUTI4 (HSTRING,VNAME,IVAL)

c  Extract value for variable VNAME from FITS header string
c  HSTRING, returning it to address IVAL.

	Character*(*) HSTRING
	Character*(*) VNAME
	Integer*4 IVAL
c		Address for number

	Character*30 VALUE

c  Translate value from binary to ASCII

	Call BINASCI4 (IVAL,VALUE)

c  Put value into header string

	Call HPUTC (HSTRING,VNAME,VALUE)

c  Return to calling program

	Return
	End


	Subroutine HPUTI2 (HSTRING,VNAME,IVAL)

c  Extract value for variable VNAME from FITS header string
c  HSTRING, returning it to address IVAL.

	Character*(*) HSTRING
	Character*(*) VNAME
	Integer*2 IVAL
c		Address for number

	Character*30 VALUE

c  Translate value from binary to ASCII

	Call BINASCI2 (IVAL,VALUE)

c  Put value into header string

	Call HPUTC (HSTRING,VNAME,VALUE)

c  Return to calling program

	Return
	End


	Subroutine HPUTR4 (HSTRING,VNAME,RVAL)

c  Extract value for variable VNAME from FITS header string
c  HSTRING, returning it to address RVAL.

	Character*(*) HSTRING
	Character*(*) VNAME
	Real*4 RVAL
c		Address for number

	Character*30 VALUE

c  Translate value from binary to ASCII

	Call BINASCR4 (RVAL,VALUE)

c  Put value into header string

	Call HPUTC (HSTRING,VNAME,VALUE)

c  Return to calling program

	Return
	End


	Subroutine HPUTR8 (HSTRING,VNAME,DVAL)

c  Extract value for variable VNAME from FITS header string
c  HSTRING, returning it to address DVAL.

	Character*(*) HSTRING
	Character*(*) VNAME
	Real*8 DVAL
c		Address for number

	Character*30 VALUE

c  Translate value from binary to ASCII

	Call BINASCR8 (DVAL,VALUE)

c  Put value into header string

	Call HPUTC (HSTRING,VNAME,VALUE)

c  Return to calling program

	Return
	End


	Subroutine HPUTL (HSTRING,VNAME,LVAL)

c  Extract value for variable VNAME from FITS header string
c  HSTRING, returning it to address IVAL.

	Character*(*) HSTRING
	Character*(*) VNAME
	Logical LVAL
c		Address for number

	Character*1 VALUE

c  Translate value from binary to ASCII

	If (LVAL) Then
	    VALUE = 'T'
	Else
	    VALUE = 'F'
	Endif

c  Put value into header string

	Call HPUTC (HSTRING,VNAME,VALUE)

c  Return to calling program

	Return
	End


	Subroutine HPUTS (HSTRING,VNAME,CVAL)

	Character*(*) HSTRING
	Character*(*) VNAME
	Character*(*) CVAL
c			Character string containing the value for variable
c			VNAME.  Trailing and leading blanks are removed.

	Character*1 SQUOT
	Character*70 VALUE
	Integer*4 LCVAL

	Include 'header.cmb'

	SQUOT = Char (39)

c  Find length of variable string

	Call VLEN (CVAL,LCVAL)
	If (LCVAL .gt. 67) LCVAL = 67

c  Put quotes around string

	VALUE(1:1) = SQUOT
	VALUE(2:LCVAL+1) = CVAL(1:LCVAL)
	VALUE(LCVAL+2:LCVAL+2) = SQUOT
	VALUE(LCVAL+3:LCVAL+3) = Char (0)
	Call PADNULL (VALUE)

c  Put value into header string

	Call HPUTC (HSTRING,VNAME,VALUE)

c  Return to calling program

	Return
	End


	Subroutine HPUTC (HSTRING,VNAME,VALUE)

	Character*(*) HSTRING
	Character*(*) VNAME
	Character*(*) VALUE
c			Character string containing the value for variable
c			VNAME.  Trailing and leading blanks are removed.

	Logical NOEQ
	Character*1 NULL,SQUOT
	Character*100 LINE
	Character*50 NEWCOM
	Integer*4 LVNAME,LCOM,LVAL,L0,LE,L1,LQ0,LQ1,LC0,I,LC

	Include 'header.cmb'

	SQUOT = Char (39)
	NULL = Char (0)
	FOUND = .false.

c  Find length of variable name

	Call VLEN (VNAME,LVNAME)

c  Search header string for variable name

	If ((VNAME(1:7) .eq. 'COMMENT') .or. (VNAME(1:7) .eq. 'HISTORY')) Then
	    L0 = 0
	    NOEQ = .true.
	Else
	    Call HSEARCH (HSTRING,VNAME,L0)
	    NOEQ = .false.
	Endif

c  Find END of header

	Call HSEARCH (HSTRING,'END',LE)

c  If parameter is not found, create a space for it

	If (L0 .le. 0) Then
	    L0 = LE
	    L1 = L0 + 79
	    HSTRING(L1+1:L1+79) = HSTRING(LE:LE+79)

c  Extract entry for this variable from the header

	Else
	    L1 = L0 + 79
	    LINE = HSTRING(L0:L1)

c  Check for quoted value

	    LQ0 = Index (LINE,SQUOT)
	    If (LQ0 .gt. 0) Then
		LQ1 = LQ0 + Index (LINE(LQ0+1:),SQUOT)
	    Else
		LQ1 = 0
	    Endif

c  Extract comment and remove trailing spaces

	    LC0 = Index (LINE(LQ1+1:),'/') + LQ1 + 1
	    If (LC0 .gt. 1) Then
		NEWCOM = LINE(LC0+1:)
	    Else
		NEWCOM = NULL
	    Endif
	    Call PADNULL (NEWCOM)
	Endif

c  Create new entry

	Call VLEN (NEWCOM,LCOM)
	Call VLEN (VALUE,LVAL)
	Do 300 I = L0, L1
300	    HSTRING(I:I) = ' '
	HSTRING(L0:L0+LVNAME-1) = VNAME(1:LVNAME)

c  Add COMMENT or HISTORY line just before END

	If (NOEQ) Then
	    HSTRING(L0+8:L0+9) = '  '
	    If (LVAL .lt. 70) Then
		HSTRING(L0+10:L0+10+LVAL) = VALUE(1:LVAL)
	    Else
		HSTRING(L0+10:L0+10+LVAL) = VALUE(1:69)
		L0 = L0 + 80
		L1 = L1 + 80
		Do 305 I = L0, L1
305		    HSTRING(I:I) = ' '
		HSTRING(L0:L0+LVNAME) = VNAME(1:LVNAME)
		HSTRING(L0+8:L0+9) = '  '
		HSTRING(L0+10:L0+10+LVAL) = VALUE(70:LVAL)
		L0 = L0 + 80
		L1 = L1 + 80
		HSTRING(L0:L0+2) = 'END'
		Do 306 I = L0+3, L1
306		    HSTRING(I:I) = ' '
	    Endif

c  Add any other parameter in the appropriate place

	Else
	    HSTRING(L0+8:L0+8) = '='
	    If (VALUE(1:1) .eq. SQUOT) Then
		HSTRING(L0+10:L0+10+LVAL) = VALUE(1:LVAL)
		If (LVAL+12 .gt. 31) Then
		    LC = LVAL + 12
		Else
		    LC = 40
		Endif
	    Else
		HSTRING(L0+30-LVAL:L0+30) = VALUE(1:LVAL)
		LC = 40
	    Endif
	Endif
	If (LCOM .gt. 0) Then
	    If (LC+2+LCOM .gt. 80) LCOM = 78 - LC
	    HSTRING(L0+LC:L0+LC) = '/'
	    HSTRING(L0+LC+2:L0+LC+2+LCOM) = NEWCOM(1:LCOM)
	    Endif

	If (VERBOSE) Then
	    If (LCOM .gt. 0) Then
		Write(*,400) VNAME(1:LVNAME),VALUE(1:LVAL),NEWCOM(1:LCOM)
400		Format('HPUT: ',A,' = ',A,' / ',A)
	    Else
		Write(*,405) VNAME(1:LVNAME),VALUE(1:LVAL)
405		Format('HPUT: ',A,' = ',A)
	    Endif
	    Endif

	Return
	End


	Subroutine HPUTCOM (HSTRING,VNAME,COMMENT)

	Character*(*) HSTRING
	Character*(*) VNAME
	Character*(*) COMMENT

	Character*1 SQUOT
	Character*100 LINE
	Integer*4 LVNAME,LCOM,LE,L0,L1,I,LQ0,LQ1,LC0,LC1,LC

	Include 'header.cmb'

	SQUOT = Char (39)
	FOUND = .false.

c  Find length of variable name

	Call VLEN (VNAME,LVNAME)

c  If parameter is COMMENT, always add it just before END

	If ((VNAME(1:7).eq.'COMMENT') .or. (VNAME(1:7).eq.'HISTORY')) Then

c	Find END of header

	    Call HSEARCH (HSTRING,'END',LE)
	    L0 = LE
	    L1 = L0 + 79
	    HSTRING(L1+1:L1+80) = HSTRING(LE:LE+79)

c  Put parameter name on line

	    Do 100 I = L0, L1
100		HSTRING(I:I) = ' '
	    HSTRING(L0:L0+LVNAME-1) = VNAME(1:LVNAME)
	    LC = 11

c  Search header string for variable name

	Else
	    Call HSEARCH (HSTRING,VNAME,L0)

c  If parameter is not found don't do anything

	    If (L0 .le. 0) Then
		If (VERBOSE) Write(*,*) 'HPUTCOM:',VNAME(1:LVNAME),'not found'
		Return
		Endif

c  Otherwise, extract entry for this variable from the header

	    L1 = L0 + 79
	    LINE = HSTRING(L0:L1)

c  Check for quoted value

	    LQ0 = Index (LINE,SQUOT)
	    If (LQ0 .gt. 0) Then
		LQ1 = LQ0 + Index (LINE(LQ0+1:),SQUOT)
	    Else
		LQ1 = 0
	    Endif
	    If (LQ1 .lt. 31) Then
		LC = 40
	    Else
		LC = LQ1 + 2
	    Endif
	    HSTRING(L0+LC:L0+LC+1) = '/ '
	Endif

c  Create new entry

	Call PADNULL (COMMENT)
	Call VLEN (COMMENT,LCOM)

	If (LCOM .gt. 0) Then
	    LC0 = L0 + LC + 2
	    If (LC0+LCOM .gt. L1) LCOM = L1 - (LC0 + LCOM)
	    Do 390 I = LC0, L1
390		HSTRING(I:I) = ' '
	    LC1 = LC0 + LCOM - 1
	    HSTRING(LC0:LC1) = COMMENT(1:LCOM)
	    Endif

	If (VERBOSE) Then
	    Write(*,400) VNAME(1:LVNAME),COMMENT(1:LCOM)
400	    Format('HPUTCOM: ',A,' / ',A)
	    Endif

	Return
	End

c Dec  9 1987	Drop COMMENT from HPUTxx argement lists
c		Add HPUTCOM for adding comment to header entries

c Mar  4 1988	Fix HPUTCOM and HPUTS so they work
c Apr 20 1988	Reposition values and comments to match IRAS FITS headers

c Jul  8 1990	Clean up code for COMMENT and HISTORY
c Oct  2 1990	Clean up code using flint

c Mar 21 1991	Declare all variables

c Jun  1 1992	Keep long comments from overwriting next line

c Feb 25 1993	Use typed versions of BINASC
c Apr 19 1993	Set value of logical parameters to T or F, not Y or N
