c*** File iolib/ascbin.f
c*** October 30, 1999
c*** By Doug Mink


c  ASCBIN (NCHAR,LINE,ITYPE,OUTVAL,IER) Translate string into binary variable
c  ASCBINI2 (NCHAR,LINE,HVAL,IER)	Translate string into Integer*2 variable
c  ASCBINI4 (NCHAR,LINE,IVAL,IER)	Translate string into Integer*4 variable
c  ASCBINL (LINE,LVAL,IER)		Translate string into Logical variable
c  ASCBINR4 (NCHAR,LINE,RVAL,IER)	Translate string into Real*4 variable
c  ASCBINR8 (NCHAR,LINE,DVAL,IER)	Translate string into Real*8 variable
c  ASCNUM (NCHAR,STRING,NUM)		NUM=1 STRING integer, NUM=2 STRING float

	Subroutine ASCBIN (NCHAR,LINE,ITYPE,OUTVAL,IER)

c  Translate ASCII string LINE into variable at address OUTVAL.
c   First ':' treated as '.'; later ':'s and '.'s ignored.
c   Illegal characters, overflows, and underflows trapped. 

	Integer*4 NCHAR
c		Number of characters in string to be translated

	Character*(*) LINE
c		Character string to be translated

	Integer*4 ITYPE
c		Type of variable to return (Integer*4)
c		-1n: Integer n bytes long (Integer*n)
c		-2n: Floating point number n bytes long (Real*n)
c		-3n: Logical variable n bytes long (Logical*n) (t or f)
c		-4n: Logical variable n bytes long (Logical*n) (y or n)
c		1-99: string length, returned in Character, padded with spaces
c		101-199: string length, returned as Character, padded with nulls
c		201-236: number of single characters (200 added)
c			returned in array of 36 (A-Z,0-9) Integer*2
c			with array positions corresponding to letter
c			or number entered set to one.  Array is not
c			zeroed on entry.
c			Lower case letters are treated as upper case.
c		301-399: string length, returned as Character, padded with nulls
c			Lower case letters are converted to upper case.

	Real*8 OUTVAL(*)
c		Address for returned number, character, array, or string
c		If a Character string is to be returned, call with
c		Equivalence'd Integer; otherwise, Fortran adds another
c		invisible string length to the calling parameter list
c		and messes up everything.
c		Integer*2 (returned)

	Integer*4 IER
c		Error code (0=OK) (returned)
c			    1=Exponent overflow
c			    2=Non-numeric input
c			    3=Integer overflow
c			    4=Illegal character

	Logical FRAC,EXPN,LVAL,QUOTED
	Character NULL*1,JCHAR*1,PADC*1,JQUOT*1,JTAB*1,TEMP*80
	Real*8 DNUM,DN,DXP,DSGN
	Integer*4 IDUM, IC, IC1, IC2, MCHAR, ICH, NB, ISGN, IVXP, N, I
	Integer*4 NCHAR1
	Common/ASCB/ DVAL,IDUM(24)
	Real*8 DVAL
	Real*4 RVAL
	Integer*4 KVAL
	Integer*2 JVAL(50)
	Character CVAL*100
	Equivalence (DVAL,RVAL,CVAL,KVAL,JVAL(1),LVAL)

	Include 'header.cmb'

	If (NCHAR .le. 0) Return

	IER = 0
	NULL = Char (0)
	JQUOT = Char (39)
	JTAB = Char (9)
	Call ZFILLC (CVAL,100)

	MCHAR = 99
	If (ITYPE .gt. 0) MCHAR = ITYPE - 100 * (ITYPE/100)
	If (MCHAR .gt. 99) MCHAR = 99
	If (ITYPE .eq. -1) MCHAR = 1
	If (ITYPE .lt. -30) MCHAR = 1

c  Decode logical variable

	If (ITYPE .lt. -30) Then
	    LVAL = .false.
	    If ((LINE(1:1).eq.'T') .or. (LINE(1:1).eq.'t')) LVAL = .true.
	    If ((LINE(1:1).eq.'Y') .or. (LINE(1:1).eq.'y')) LVAL = .true.
	    NB = -ITYPE - 10*(-ITYPE/10)
	    Call MOVEB (LVAL,OUTVAL,NB,0,0)

c  Decode number

	Elseif (ITYPE .lt. -1) Then
	    DNUM = 0.D0
	    FRAC = .false.
	    DXP = .1D0
	    DSGN = 1.D0
	    EXPN = .false.
	    ISGN = 1
	    IVXP = 0

c  Ignore leading and trailing blanks (or control characters)

	    Do 10 IC = 1, NCHAR
10		If (Ichar (LINE(IC:IC)) .gt. 32) Go to 11
11	    IC1 = IC
	    Do 20 IC = NCHAR, 1, -1
20		If (Ichar (LINE(IC:IC)) .gt. 32) Go to 21
21	    IC2 = IC

	    Do 250 IC = IC1, IC2
		JCHAR = LINE(IC:IC)
		ICH = Ichar (JCHAR)
		If (ICH .gt. 96) Then
		    ICH = ICH - 32
		    JCHAR = Char (ICH)
		    Endif

c  Add if character is digit from 0 to 9

		If ((ICH .ge. 48) .and. (ICH .le. 57)) Then
		    N = ICH - 48
		    DN = N

c  Add to exponent, checking for overflow

		    If (EXPN) Then
			IVXP = IVXP * 10 + (ICH - 48)
			If (ITYPE .eq. -11) Then
			    If (IVXP .gt. 2) Go to 390
			Elseif (ITYPE .eq. -12) Then
			    If (IVXP .gt. 4) Go to 390
			Elseif (ITYPE .eq. -14) Then
			    If (IVXP .gt. 8) Go to 390
			Elseif (ITYPE .eq. -24) Then
			    If (IVXP .gt. 38) Go to 390
			Elseif (ITYPE .eq. -28) Then
			    If (IVXP .gt. 308) Go to 390
			Endif

c  Add to fraction
		    Elseif (FRAC) Then
			DNUM = DNUM + DN * DXP
			DXP = DXP * .1D0

c  Add to number (mantissa)
		    Else
			DNUM = DNUM * 1.D1 + DN
		    Endif

c  Set decimal point if character is ':', '.', or ' '
		Elseif ((JCHAR.eq.':').or.(JCHAR.eq.'.').or.(JCHAR.eq.' ')) Then
		    If (.not. FRAC) DXP = .1D0
		    FRAC = .true.

c  Set exponent if 'D' or 'E'
		Elseif ((JCHAR .eq. 'D') .or. (JCHAR .eq. 'E')) Then
		    EXPN = .true.

c  Process - sign
		Elseif (JCHAR .eq. '-') Then

c  If in exponent, set sign of exponent
		    If (EXPN) Then
			ISGN = -1

c  If not first character of number, treat as decimal point (as for date)
		    Elseif (.not. FRAC) Then
			If (DNUM .gt. 0) Then
			    FRAC = .true.

c  Otherwise, set sign of mantissa
			Else
			    DSGN = -1.D0
			Endif
		    Endif

c  Print error message and process characters before this one
c  if anything else (but space or '+')
		Elseif ((JCHAR .ne. '+') .and. (JCHAR .ne. JQUOT) .and.
     1			(JCHAR .ne. JTAB)) Then
		    Write(TEMP,*) 'ASCBINR: Non-numeric input "',JCHAR,
     1			       '" ',LINE(1:NCHAR)
		    Call PSTRING (TEMP)
		    If (IC .gt. IC1) Then
			Go to 251
		    Else
			Go to 400
		    Endif
		Endif
250		Continue

c  Process string

251	    If (EXPN) DNUM = DNUM * (1.d1 ** (ISGN*IVXP))

c  Integer

	    If (ITYPE/10 .eq. -1) Then
		If (ITYPE .eq. -11) Then
		    If (DNUM .ge. 2.d0**8) Go to 410
		    NB = 1
		    KVAL = Ifix (Sngl (DNUM + .5D0))
		    If (DSGN .lt. 0) KVAL = -KVAL
		    CVAL(1:1) = Char (KVAL)
		Elseif (ITYPE .eq. -12) Then
		    If (DNUM .ge. 2.d0**15) Go to 410
		    NB = 2
		    JVAL(1) = Ifix (Sngl (DNUM + .5D0))
		    If (DSGN .lt. 0) JVAL(1) = -JVAL(1)
		Else
		    If (DNUM .ge. 2.d0**31) Go to 410
		    NB = 4
		    KVAL = Ifix (Sngl (DNUM + .5D0))
		    If (DSGN .lt. 0) KVAL = -KVAL
		Endif

c  Floating point

	    Else
		DNUM = DSGN * DNUM

c  Double precision floating point (REAL*8)

		If (ITYPE .eq. -28) Then
		    DVAL = DNUM
		    NB = 8

c  Single precision floating point (REAL*4)

		Else
		    RVAL = Sngl (DNUM)
		    NB = 4
		Endif
	    Endif

c  Move value to dummy variable

	    Call MOVEB (KVAL,OUTVAL,NB,0,0)
	    Return

c  Return switch array

	Elseif ((ITYPE .gt. 200) .and. (ITYPE .lt. 300)) Then
	    NB = 36 * 2
	    Do 305 I = 1, 36
305		JVAL(I) = 0
	    Do 310 IC = 1, NCHAR
		ICH = Ichar (LINE(IC:IC))
		If (ICH .gt. 96) ICH = ICH - 32
		If ((ICH .gt. 64) .and. (ICH .le. 90)) Then
		    ICH = ICH - 64
		Elseif ((ICH .ge. 48) .and. (ICH .le. 57)) Then
		    ICH = ICH - 21
		Else
		    Go to 420
		Endif
310		JVAL(ICH) = 1
	    Call MOVEB (KVAL,OUTVAL,NB,0,0)

c  Set up character string

	Else
	    IC = 1
	    QUOTED = .false.
	    Do 320 I = 1, NCHAR
		JCHAR = LINE(I:I)
		If ((I .eq. 1) .and. (JCHAR .eq. JQUOT)) Then
		    QUOTED = .true.
		Elseif ((QUOTED) .and. (JCHAR .eq. JQUOT)) Then
		    QUOTED = .false.
		Else
		    CVAL(IC:IC) = JCHAR
		    IC = IC + 1
		Endif
320		Continue

c	Pad string with apporpriate pad character

	    PADC = ' '
	    If (ITYPE .gt. 100) PADC = NULL
	    If (IC .le. MCHAR) Then
		NCHAR1 = IC
		Do 325 IC = NCHAR1, MCHAR
325		    CVAL(IC:IC) = PADC
		Endif

c	Move string to integer argument

	    Call MOVEB (KVAL,OUTVAL,MCHAR,0,0)
	Endif

	Return

c  Error return

390	Write(TEMP,*) 'ASCBIN: Exponent overflow = ',IVXP,' in ',
     1		   LINE(1:NCHAR)
	Call PSTRING (TEMP)
	IER = 1
	Return

c  Non-numeric input
400	IER = 2
	Return

410	Write(TEMP,*) 'ASCBIN: Integer overflow, N =',DNUM,' in ',
     1		   LINE(1:NCHAR)
	Call PSTRING  (TEMP)
	IER = 3
	Return

420	Write(TEMP,*) 'ASCBIN: Illegal character "',JCHAR,'" in ',
     1		   LINE(1:NCHAR)
	Call PSTRING  (TEMP)
	IER = 4
	Return
	End


c--- ASCBINI2 Translate ASCII string LINE into Integer*2 variable

	Subroutine ASCBINI2 (NCHAR,LINE,HVAL,IER)

	Integer*4 NCHAR
c			Number of characters in string to be translated
	Character*(*) LINE
c			Character string to be translated
	Integer*2 HVAL
c			Address for returned number
	Integer*4 IER
c			Error code (0=OK) (returned)

	Real*8 DNUM, DMAX
	Character*80 TEMP

	Include 'header.cmb'

	If (NCHAR .le. 0) Return

	Call ASCBINR8 (NCHAR,LINE,DNUM,IER)

c  Check for overflow

	DMAX = (2.d0 ** 15) - 1.d0
        If (Dabs (DNUM) .gt. DMAX) Then
	    Write(TEMP,*) 'ASCBINI2: Integer overflow, N =',DNUM
	    Call PSTRING (TEMP)
	    Return
	    Endif

	HVAL = Idnint (DNUM)

	Return
	End


	Subroutine ASCBINI4 (NCHAR,LINE,IVAL,IER)

c  Translate ASCII string LINE into Integer*4 variable.

	Integer*4 NCHAR
c			Number of characters in string to be translated
	Character*(*) LINE
c			Character string to be translated
	Integer*4 IVAL
c			Address for returned number
	Integer*4 IER
c			Error code (0=OK) (returned)

	Real*8 DNUM, DMAX
	Character*80 TEMP

	Include 'header.cmb'

	If (NCHAR .le. 0) Return

	Call ASCBINR8 (NCHAR,LINE,DNUM,IER)

c  Check for overflow

	DMAX = (2.d0 ** 31) - 1.d0
        If (Dabs (DNUM) .gt. DMAX) Then
	    Write(TEMP,*) 'ASCBINI4: Integer overflow, N =',DNUM
	    Call PSTRING (TEMP)
	    Return
	    Endif

	IVAL = Idnint (DNUM)

	Return
	End


c--- Translate ASCII string LINE into Logical variable.

	Subroutine ASCBINL (LINE,LVAL,IER)

	Character*(*) LINE
c			Character string to be translated
	Logical LVAL
c			Address for returned number
	Integer*4 IER
c			Error code (0=OK) (returned)

	Character*80 TEMP

	Include 'header.cmb'

c  Decode logical variable

	If (LINE(1:1) .eq. 'T') Then
	    LVAL = .true.
	Elseif (LINE(1:1) .eq. 't') Then
	    LVAL = .true.
	Elseif (LINE(1:1) .eq. 'Y') Then
	    LVAL = .true.
	Elseif (LINE(1:1) .eq. 'y') Then
	    LVAL = .true.
	Elseif (LINE(1:1) .eq. 'F') Then
	    LVAL = .false.
	Elseif (LINE(1:1) .eq. 'f') Then
	    LVAL = .false.
	Elseif (LINE(1:1) .eq. 'N') Then
	    LVAL = .false.
	Elseif (LINE(1:1) .eq. 'n') Then
	    LVAL = .false.
	Else
	    IER = 4
	    Write(TEMP,*) 'ASCBINL: Illegal character = "',LINE(1:1),'"'
	    Call PSTRING (TEMP)
	    Return
	Endif

	IER = 0
	Return
	End


	Subroutine ASCBINR4 (NCHAR,LINE,RVAL,IER)

c  Translate ASCII string LINE into Real*4 variable.

	Integer*4 NCHAR
c			Number of characters in string to be translated
	Character*(*) LINE
c			Character string to be translated
	Real*4 RVAL
c			Address for returned number
	Integer*4 IER
c			Error code (0=OK) (returned)

	Real*8 DNUM

	If (NCHAR .le. 0) Return

	Call ASCBINR8 (NCHAR,LINE,DNUM,IER)

c  Multiply by sign

	RVAL = Sngl (DNUM)

	Return
	End


	Subroutine ASCBINR8 (NCHAR,LINE,DVAL,IER)

c  Translate ASCII string LINE into Real*8 variable.

	Integer*4 NCHAR
c			Number of characters in string to be translated
	Character*(*) LINE
c			Character string to be translated
	Real*8 DVAL
c			Address for returned number
	Integer*4 IER
c			Error code (0=OK) (returned)

	Logical FRAC,EXPN
	Character NULL*1,JCHAR*1,JQUOT*1,JTAB*1
	Real*8 DNUM,DN,DXP,DSGN
	Integer*4 ICH, IC1, IC2, IC, ISGN, IVXP, N
	Character*80 TEMP

	Include 'header.cmb'

	If (NCHAR .le. 0) Return

	IER = 0
	NULL = Char (0)
	JQUOT = Char (39)
	JTAB = Char (9)

c  Ignore leading and trailing blanks (or control characters)

	Do 10 IC = 1, NCHAR
10	    If (Ichar (LINE(IC:IC)) .gt. 32) Go to 11
11	IC1 = IC
	Do 20 IC = NCHAR, 1, -1
20	    If (Ichar (LINE(IC:IC)) .gt. 32) Go to 21
21	IC2 = IC

c  Decode number

	DNUM = 0.D0
	FRAC = .false.
	DXP = .1D0
	DSGN = 1.D0
	EXPN = .false.
	ISGN = 1
	IVXP = 0

	Do 250 IC = IC1, IC2
	    JCHAR = LINE(IC:IC)
	    ICH = Ichar (JCHAR)
	    If (ICH .gt. 96) Then
		ICH = ICH - 32
		JCHAR = Char (ICH)
		Endif

c  Add if character is digit from 0 to 9

	    If ((ICH .ge. 48) .and. (ICH .le. 57)) Then
		N = ICH - 48
		DN = N

c  Add to exponent, checking for overflow

		If (EXPN) Then
		    IVXP = IVXP * 10 + (ICH - 48)
		    If (IVXP .gt. 308) Go to 390

c  Add to fraction

		Elseif (FRAC) Then
		    DNUM = DNUM + DN * DXP
		    DXP = DXP * .1D0

c  Add to number (mantissa)

		Else
		    DNUM = DNUM * 1.D1 + DN
		Endif

c  Set decimal point if character is ':', '.', or ' '

	    Elseif ((JCHAR.eq.':').or.(JCHAR.eq.'.').or.(JCHAR.eq.' ')) Then
		If (.not. FRAC) DXP = .1D0
		FRAC = .true.

c  Set exponent if 'D' or 'E'

	    Elseif ((JCHAR .eq. 'D') .or. (JCHAR .eq. 'E')) Then
		EXPN = .true.

c  Set sign if '-', ignore if '+'

	    Elseif (JCHAR .eq. '-') Then
		If (EXPN) Then
		    ISGN = -1
		Else
		    DSGN = -1.D0
		Endif

c  Print error message and reprompt if anything else (but space or '+')

	    Elseif ((JCHAR.ne.'+').and.(JCHAR.ne.JQUOT).and.
     1		(JCHAR.ne.JTAB) .and. (JCHAR.ne.NULL)) Then
		Write(*,*) 'ASCBINR8: Non-numeric input "',JCHAR,
     1			   '" ',LINE(1:NCHAR)
		Call PNEWLINE
		If (IC .gt. IC1) Then
		    Go to 251
		Else
		    Go to 400
		Endif
	    Endif
250	    Continue

251	If (EXPN) DNUM = DNUM * (1.D1 ** (ISGN*IVXP))

c  Multiply by sign

	DVAL = DSGN * DNUM

	Return

c  Error return

390	Write(TEMP,*) 'ASCBINR8: Exponent overflow = ',IVXP,' ',
     1		    LINE(1:NCHAR)
	Call PSTRING (TEMP)
	IER = 1
	Return

c  Non-numeric input
400	IER = 2
	Return

	End


	Subroutine ASCNUM (NCHAR,STRING,NUM)

c  Return NUM 1 or 2 if STRING is a number, or else 0

	Integer*4 NCHAR
c			Number of characters in string to be translated
	Character*(*) STRING
c			Character string to be translated
	Integer*4 NUM
c			=1: STRING is an integer
c			=2: STRING is a floating point number
c			=0: STRING is a not a legal number

	Character NULL*1,JCHAR*1,JQUOT*1,JTAB*1
	Integer*4 ICH, IC1, IC2, IC, NC

	Include 'header.cmb'

	NUM = 0
	NC = NCHAR
	If (NC .le. 0) Then
	    Call VLEN (STRING,NC)
	    If (NC .le. 0) Return
	    Endif

	NULL = Char (0)
	JQUOT = Char (39)
	JTAB = Char (9)

c  Ignore leading blanks (or control characters)

	Do 10 IC = 1, NC
10	    If (Ichar (STRING(IC:IC)) .gt. 32) Go to 11
11	IC1 = IC

c  Ignore trailing blanks (or control characters)

	Do 20 IC = NC, 1, -1
20	    If (Ichar (STRING(IC:IC)) .gt. 32) Go to 21
21	IC2 = IC

c  Decode number

	Do 250 IC = IC1, IC2
	    JCHAR = STRING(IC:IC)
	    ICH = Ichar (JCHAR)
	    If (ICH .gt. 96) Then
		ICH = ICH - 32
		JCHAR = Char (ICH)
		Endif

c  Add if character is digit from 0 to 9

	    If ((ICH .ge. 48) .and. (ICH .le. 57)) Then
		If (NUM .eq. 0) NUM = 1

c  Set decimal point if character is ':', '.', or ' '

	    Elseif ((JCHAR.eq.':').or.(JCHAR.eq.'.').or.(JCHAR.eq.' ')) Then
		NUM = 2

c  Set exponent if 'D' or 'E'

	    Elseif ((JCHAR .eq. 'D') .or. (JCHAR .eq. 'E')) Then
		NUM = 2

c  Set sign if '-', ignore if '+'

	    Elseif (JCHAR .eq. '-') Then

c  Print error message and reprompt if anything else (but space or '+')

	    Elseif ((JCHAR.ne.'+').and.(JCHAR.ne.JQUOT).and.
     1		(JCHAR.ne.JTAB) .and. (JCHAR.ne.NULL)) Then
		NUM = 0
		Return
	    Endif
250	    Continue

	Return
	End


	Subroutine ISNUM (STRING, NUM)

	Character*(*) STRING
c				Character string to test
	Logical NUM
c				True if number, else false (returned)

	Integer*4 LSTR,I,ND, ISTR
	Character*1 CSTR

	Call VLEN (STRING,LSTR)
	ND = 0
	Do I = 1, LSTR
	    CSTR = STRING(I:I)
	    ISTR = Ichar (CSTR)
	    If (((ISTR .lt. 48) .or. (ISTR .gt. 57)) .and.
     1		(CSTR .ne. '+') .and. (CSTR .ne. '-') .and.
     2		(CSTR .ne. 'D') .and. (CSTR .ne. 'd') .and.
     3		(CSTR .ne. 'E') .and. (CSTR .ne. 'e') .and.
     4		(CSTR .ne. '.')) Then
		NUM = .false.
		Return
	    Elseif ((ISTR .ge. 47) .and. (ISTR .le. 57)) Then
		ND = ND + 1
	    Endif
	    Enddo
	If (ND .gt. 0) Then
	    NUM = .true.
	Else
	    NUM = .false.
	Endif
	Return
	End

c Nov 15 1984	Ignore ' in numeric strings

c Apr 11 1985	Remove unnecessary re-assignment of LINE

c Jul 14 1986	Ignore ' in character strings

c Jun  4 1987	Change meaning of IER to 1=OK from 0=OK
c Jun 10 1987	Ignore tabs as well as spaces and quotes
c Aug 12 1987	Ignore quotes only if first character in string is quote (')

c Apr 11 1989	Cleanup code for RISC machine by separating out f.p. calls

c Mar 14 1990	Fix code for Integer*1 and Integer*2 with non-VAX byte order
c Apr 24 1990	Fix code for Real*4
c Aug 10 1990	Add ASCBINI4 for Integer*4 numbers
c		Ignore leading blanks

c Jul 11 1991	Eliminate extraneous error message

c Aug 21 1992	Add non-zero error codes and print better error messages
c Sep  3 1992	Add ASCBINL to return Logical value
c Sep  9 1992	Add ASCBINI2 to return Integer*2 value
c Sep 11 1992	Declare undeclared variables
c Nov 30 1992	If non-number encountered after numbers, process anyway

c Mar  3 1993	Print error messages using PSTRING

c Oct 11 1995	Add ASCNUM to check for numeracity of a string

c Dec  3 1996	If nchar in ascnum is zero, find length of string

c Oct  8 1997	Add isnum() to test for numbers

c Aug 20 1998	Declare undeclared variables

c Oct 30 1999	Process - as decimal point for ISO dates
