c*** File iolib/dcon.f
c*** December 1, 2004
c*** By Doug Mink

c  Declination conversion subroutines

c	DCONR (HDEC,DEC) dd.mmssss to radians
c	DCONH (DEC,HDEC) radians to dd.mmssss
c	DCOND (HDEC,DDEC) dd.mmssss to fractional degrees
c	DCONF (HDEC,DDEC)  fractional degrees to dd.mmssss
c	DCONS (DEC,ISGN,ID,IM,DSEC) radians to deg, min, sec
c	DCONT (ISGN,ID,IM,DSEC,DEC) deg, min, sec to radians
c	DCONI (DEC,ISGN,ID,IM,ISEC) radians to deg, min, int. sec
c	DCSTR (DEC,NDEC,NPAD,UNITS,STRING) radians to string
c	DCSTRH (DEC,NDEC,STRING) radians to string (dd.mmssss); drop zeroes
c	DCSTRT (DEC,NDEC,STRING) radians to string (d'"); drop zeroes
c	DCSTRC (DEC,NDEC,STRING) radians to string (:); drop zeroes
c	DCSTRS (DEC,NDEC,STRING) radians to seconds; drop zeroes
c	DCSTRP (DEC,PREC,STRING) radians to string with superscript labels
c	D2R (DEC) fractional degrees to radians
c	R2D (DEC) radians to fractional degrees

c--- DCONR  Convert from DEC as dd.mmssss to radians

	Subroutine DCONR (HDEC,DEC)

	Real*8 HDEC
c			DEC as dd.mmsssss
	Real*8 DEC
c			DEC in radians (returned)

	Call HCONR (HDEC,DEC)

	Return
	End


c--- DCONH  Convert declination in radians to dd.mmssssss.

	Subroutine DCONH (DEC,HDEC)

	Real*8 DEC
c			Declination in radians
	Real*8 HDEC
c			Declination as dd.mmsssss (returned)

	Call HCONS (DEC,HDEC)

	Return
	End


c--- DCOND  Convert from DEC as dd.mmssss to degrees

	Subroutine DCOND (HDEC,DDEC)

	Real*8 HDEC
c			DEC as dd.mmsssss
	Real*8 DDEC
c			DEC in degrees (returned)

	Call HCON (HDEC,DDEC)

	Return
	End


c--- DCONF  Convert declination in fractional degrees to dd.mmssssss.

	Subroutine DCONF (DDEC,HDEC)

	Real*8 DDEC
c			Declination in degrees
	Real*8 HDEC
c			Declination as dd.mmsssss (returned)

	Real*8 DEC1

	Include 'constants.cmb'

	If (RCON .lt. 0.01d0) Call CONSET
	DEC1 = DDEC * RCON
	Call HCONS (DEC1,HDEC)

	Return
	End


c--- DCONS  Convert DEC in radians to degrees, minutes, and seconds.

	Subroutine DCONS (DEC,ISGN,IDD,IDM,DSEC)

	Real*8 DEC
c			Declination in radians
	Integer*4 ISGN
c			Sign of DEC
	Integer*4 IDD
c			Degrees (returned)
	Integer*4 IDM
c			Minutes (returned)
	Real*8 DSEC
c			Seconds (returned)

	Real*8 DEC1

	Include 'constants.cmb'

	If (RCON .lt. 0.01d0) Call CONSET
	DEC1 = DEC / RCON
	Call HCON1 (DEC1,ISGN,IDD,IDM,DSEC)

	Return
	End


c--- DCONT  Convert DEC in degrees, minutes, and seconds to radians.

	Subroutine DCONT (ISGN,IDD,IDM,DSEC,DEC)

	Character*1 ISGN
c			Sign of DEC
	Integer*4 IDD
c			Degrees
	Integer*4 IDM
c			Minutes
	Real*8 DSEC
c			Seconds
	Real*8 DEC
c			Declination in radians (returned)

	Real*8 HDEC

	HDEC = Dble (IDD) + (1.d-2 * Dble (IDM)) + (1.d-4 * DSEC)
	If (ISGN .eq. '-') HDEC = -HDEC
	Call DCONR (HDEC, DEC)

	Return
	End


c--- DCONI  Convert Dec in radians to degrees, minutes, and integer seconds.

	Subroutine DCONI (DEC,ISGN,IDD,IDM,IDS)

	Real*8 DEC
c			Declination in radians
	Integer*4 ISGN
c			Sign of DEC
	Integer*4 IDD
c			Degrees (returned)
	Integer*4 IDM
c			Minutes (returned)
	Integer*4 IDS
c			Seconds (returned)

	Real*8 DEC1
	Include 'constants.cmb'

	If (RCON .lt. 0.01d0) Call CONSET
	DEC1 = DEC / RCON
	Call HCON3 (DEC1,ISGN,IDD,IDM,IDS)

	Return
	End


c--- DCSTR  Convert declination to string

	Subroutine DCSTR (DEC,NDEC,NPAD,UNITS,STRING)

	Real*8 DEC
c			Fractional degrees or hours
	Integer*4 NDEC
c			Number of decimal places in seconds
	Integer*4 NPAD
c			Number of spaces between entries
	Logical UNITS
c			If true, print units
	Character*(*) STRING
c			String (returned)

	Integer*4 NFIELD
	Character*10 LFORMAT,TEMP
	Character*1 JDQ,JSQ
	Integer*4 IDD,IDM,ISGN,ISEC,I
	Real*8 DSEC

	JDQ = Char (34)
	JSQ = Char (39)
	If (NDEC .gt. 0) Then
	    Call DCONS (DEC,ISGN,IDD,IDM,DSEC)
	Else
	    Call DCONI (DEC,ISGN,IDD,IDM,ISEC)
	Endif

c  Degrees

10	If (ISGN .lt. 1) Then
	    STRING = '-$'
	Else
	    STRING = ' $'
	Endif
	Call PADNULL (STRING)
	If (IDD .lt. 100) Then
	    Write(TEMP,15) IDD
15	    Format(I2,'$')
	Else
	    Write(TEMP,16) IDD
16	    Format(I3,'$')
	Endif
	If (TEMP(1:1) .eq. ' ') TEMP(1:1) = '0'
	Call CONSTR (STRING,TEMP)
	If (UNITS) Call CONSTR (STRING,'d')
	If (NPAD .gt. 0) Then
	    Do I = 1, NPAD
		Call CONSTR (STRING,' $')
		Enddo
	    Endif

c  Minutes

20	Write(TEMP,25) IDM
25	Format(I2,'$')
	If (TEMP(1:1) .eq. ' ') TEMP(1:1) = '0'
	Call CONSTR (STRING,TEMP)
	If (UNITS) Call CONSTR (STRING,JSQ)
	If (NPAD .gt. 0) Then
	    Do I = 1, NPAD
		Call CONSTR (STRING,' $')
		Enddo
	    Endif

c  Seconds

	If (NDEC .le. 0) Then
	    Write(TEMP,'(I2)') ISEC
	Else
	    NFIELD = NDEC + 3
	    If (NFIELD .gt. 9) Then
		Write(LFORMAT,35) NFIELD,NDEC
35		Format('(F',I2,'.',I1,')')
	    Else
		Write(LFORMAT,36) NFIELD,NDEC
36		Format('(F',I1,'.',I1,')')
	    Endif
	    Write(TEMP,LFORMAT) DSEC
	    Do I = NFIELD, 4, -1
		If (TEMP(I:I) .eq. ' ') TEMP(I:I) = '0'
		Enddo
	Endif
	If (TEMP(1:1) .eq. '6') Then
	    DSEC = 0.d0
	    IDM = IDM + 1
	    If (IDM .gt. 59) Then
		IDM = IDM - 1
		IDD = IDD + 1
		Endif
	    Go to 10
	    Endif
	If (TEMP(1:1) .eq. ' ') TEMP(1:1) = '0'
	Call PADNULL (TEMP)
	Call CONSTR (STRING,TEMP)
	If (UNITS) Call CONSTR (STRING,JDQ)
	If (NPAD .gt. 0) Then
	    Do I = 1, NPAD
		Call CONSTR (STRING,' $')
		Enddo
	    Endif

c  Remove space between sign and first digit

	If (STRING(1:2) .eq. '- ') STRING(1:2) = '-0'

	Return
	End


c--- DCSTRT  Convert declination to string and drop trailing zeroes

	Subroutine DCSTRT (DEC,NDEC0,STRING)

	Real*8 DEC
c			Fractional degrees or hours
	Integer*4 NDEC0
c			Number of decimal places in seconds
c			* If negative, do not drop trailing zeroes
	Character*(*) STRING
c			String (returned)
	Integer*4 LT, I
	Integer*4 NPAD, NDEC
	Logical UNITS
	Character*1 NULL, JDQ

	NULL = Char (0)
	JDQ = Char (34)
	If (NDEC0 .lt. 0) Then
	    NDEC = -NDEC0
	Else
	    NDEC = NDEC0
	Endif

	NPAD = 0
	UNITS = .true.
	Call DCSTR (DEC,NDEC,NPAD,UNITS,STRING)

	If (NDEC0 .ge. 0) Then

c  Remove trailing zeroes in seconds

	    Call VLEN (STRING,LT)
	    If (STRING(LT-4:LT-1) .eq. '.000') Then
		STRING(LT-4:LT-4) = JDQ
		STRING(LT-3:LT-3) = NULL
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Elseif (STRING(LT-3:LT-1) .eq. '.00') Then
		STRING(LT-3:LT-3) = JDQ
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Elseif (STRING(LT-2:LT-1) .eq. '.0') Then
		STRING(LT-2:LT-2) = JDQ
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Endif

c  Remove arcseconds if zero

	    Call VLEN (STRING,LT)
	    If (STRING(LT-2:LT-1) .eq. '00') Then
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
		Endif

c  Remove arcminutes if zero

	    Call VLEN (STRING,LT)
	    If (STRING(LT-2:LT-1) .eq. '00') Then
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
		Endif
	    Endif

c  Eliminate leading space if trailing zeroes are dropped

	If ((NDEC0 .ge. 0) .and. (STRING(1:1) .eq. ' ')) Then
	    Do I = 2, LT
		STRING(I-1:I-1) = STRING(I:I)
		Enddo
	    STRING(LT:LT) = NULL
	    LT = LT - 1
	    Endif

	Return
	End


c--- DCSTRC  Convert declination to colon-separated string; drop trailing zeroes

	Subroutine DCSTRC (DEC,NDEC0,STRING)

	Real*8 DEC
c			Fractional degrees or hours
	Integer*4 NDEC0
c			Number of decimal places in seconds
c			* If negative, do not drop trailing zeroes
	Character*(*) STRING
c			String (returned)
	Integer*4 LT,I
	Integer*4 NPAD, NDEC
	Logical UNITS
	Character*1 NULL

	NULL = Char (0)
	If (NDEC0 .lt. 0) Then
	    NDEC = -NDEC0
	Else
	    NDEC = NDEC0
	Endif

	NPAD = 1
	UNITS = .false.
	Call DCSTR (DEC,NDEC,NPAD,UNITS,STRING)

	If (NDEC0 .ge. 0) Then

c  Remove trailing zeroes in seconds

	    Call VLEN (STRING,LT)
	    If (STRING(LT-4:LT-1) .eq. '.000') Then
		STRING(LT-4:LT-4) = NULL
		STRING(LT-3:LT-3) = NULL
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Elseif (STRING(LT-3:LT-1) .eq. '.00') Then
		STRING(LT-3:LT-3) = NULL
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Elseif (STRING(LT-2:LT-1) .eq. '.0') Then
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Endif

c  Remove arcseconds if zero

	    Call VLEN (STRING,LT)
	    If (STRING(LT-2:LT-1) .eq. '00') Then
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
		Endif

c  Remove arcminutes if zero

	    Call VLEN (STRING,LT)
	    If (STRING(LT-2:LT-1) .eq. '00') Then
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
		Endif
	    Endif

	Call VLEN (STRING,LT)
	STRING(LT+1:LT+1) = NULL

c  Eliminate leading space if trailing zeroes are dropped

	If ((NDEC0 .ge. 0) .and. (STRING(1:1) .eq. ' ')) Then
	    Do I = 2, LT
		STRING(I-1:I-1) = STRING(I:I)
		Enddo
	    STRING(LT:LT) = NULL
	    LT = LT - 1
	    Endif

c  Replace spaces with colons

	Do I = 3, LT-1
	    If (STRING(I:I) .eq. ' ') STRING(I:I) = ':'
	    Enddo

	Return
	End


c--- DCSTRH  Convert declination to vigesimal string; drop trailing zeroes

	Subroutine DCSTRH (DEC,NDEC,STRING)

	Real*8 DEC
c			Fractional degrees or hours
	Integer*4 NDEC
c			Number of decimal places in seconds
c			* If negative, do not drop trailing zeroes
	Character*(*) STRING
c			String (returned)
	Integer*4 IC, I, LSTR

	Call DCSTRC (DEC,NDEC,STRING)

c  Change first : to . ; return if there is none

	IC = Index (STRING,':')
	If (IC .le. 0) Return
	STRING(IC:IC) = '.'

c  Delete second :

	IC = Index (STRING,':')
	If (IC .gt. 0) Then
	    Call VLEN (STRING,LSTR)
	    STRING(IC:IC) = STRING(IC+1:IC+1)
	    STRING(IC+1:IC+1) = STRING(IC+2:IC+2)

c  If there are decimal seconds, delete .

	    If (STRING(IC+2:IC+2) .eq. '.') Then
		Do I = IC+3, LSTR
		    STRING(I-2:I-2) = STRING(I:I)
		    Enddo
		STRING(LSTR-1:LSTR-1) = '$'
	    Else
		STRING(IC+2:IC+2) = '$'
	    Endif
	    Call PADNULL (STRING)
	    Endif

	Return
	End


c--- DCSTRS  Convert declination to seconds string and drop leading spaces

	Subroutine DCSTRS (DEC,NDEC,STRING)

	Real*8 DEC
c			Declination in radians
	Integer*4 NDEC
c			Number of decimal places in seconds
	Character*(*) STRING
c			String (returned)
	Real*8 SDEC
	Integer*4 LSTR, ND, NF, LNUM, LSP, I, ISEC
	Character*16 FRMT

	Include 'constants.cmb'

c  Convert from radians to arcseconds

	If (RCON .lt. 0.01d0) Call CONSET
	SDEC = 3.6d3 * DEC / RCON

c  Maximum length of output string

	LSTR = Len (STRING)

c  Set length out output string (include sign and decimal point)

	NF = NDEC
	If (SDEC .ge. 0) Then
	    ND = Int (Dlog10 (SDEC)) + 1
	Else
	    ND = Int (Dlog10 (-SDEC)) + 1
	Endif
	If (ND .lt. 1) ND = 1
	If (NDEC .gt. 0) Then
	    LNUM = NDEC + ND + 2
	Else
	    LNUM = NDEC + ND
	Endif
	If (LNUM .gt. LSTR) Then
	    NF = NDEC - (LNUM - LSTR)
	    If (NF .lt. 0) Then
		Do I = 1, LSTR-1
		    STRING(I:I) = '*'
		    Enddo
		STRING(LSTR:LSTR) = Char (0)
		Return
		Endif
	    LNUM = LSTR
	Else
	    NF = NDEC
	Endif

c  Set up output format

	If (NF .eq. 0) Then
	    If (LNUM .lt. 10) Then
		Write(FRMT,10) LNUM
10		Format('(I',I1,')')
	    Else
		Write(FRMT,11) LNUM
11		Format('(I',I2,')')
	    Endif
	Elseif (NF .lt. 10) Then
	    If (LNUM .lt. 10) Then
		Write(FRMT,20) LNUM, NF
20		Format('(F',I1,'.',I1,')')
	    Else
		Write(FRMT,21) LNUM, NF
21		Format('(F',I2,'.',I1,')')
	    Endif
	Else
	    Write(FRMT,30) LNUM, NF
30	    Format('(F',I2,'.',I2,')')
	Endif

c  Write number to string

	If (NF .gt. 0) Then
	    Write(STRING,FRMT) SDEC
	Else
	    ISEC = Idnint (SDEC)
	    Write(STRING,FRMT) ISEC
	Endif

c  Remove leading spaces

	LSP = 0
	Do I = 1, LNUM
	    If (STRING(I:I) .eq. ' ') LSP = LSP + 1
	    Enddo
	If (LSP .gt. 0) Then
	    LNUM = LNUM - LSP
	    Do I = 1, LNUM
		STRING(I:I) = STRING(I+LSP:I+LSP)
		Enddo
	    STRING(LNUM+1:LNUM+1) = '$'
	    Endif
	Call PADNULL (STRING)

c  Remove hanging decimal point
	Call VLEN (STRING, LSTR)
	If (STRING(LSTR:LSTR) .eq. '.') Then
	    STRING(LSTR:LSTR) = Char (0)
	    Endif

	Return
	End


	Subroutine DCSTRP (DEC,PREC,STRING)

c--- Write DEC in degrees to minimum-length dms string with superscript labels

	Real*8 DEC
c				Declination in radians
	Integer*4 PREC
c				0= Drop both minutes and seconds if 0
c				1= Drop seconds, keep minutes if 0
c				2= Keep seconds even if 0
	Character*(*) STRING
c				Output string, terminated by "$" (returned)

	Real*8 DS
	Integer*4 ISGN,IDD,IDM,LTEMP,LSTR,LD,LM,LZ,L,I,L0,ND
	Character TEMP*60,DLAB*8,MLAB*8
	Include 'plx1.cmb'

c  Convert declination to degrees, minutes, and seconds

	Call DCONS (DEC,ISGN,IDD,IDM,DS)

c  Set labels

	If (DRAW) then
	    DLAB = '|sup{o}'
	    LD = 7
	    MLAB = "' "
	    LM = 2
	    LTEMP = 18
	Else
	    DLAB = 'd '
	    LD = 2
	    MLAB = "' "
	    LM = 2
	    LTEMP = 13
	Endif

c  Set up string and fill in blanks with zeroes

c  If seconds are to be printed

	If ((PREC .eq. 2) .or. (DS .gt. 0.005d0)) Then
	    If (IDD .gt. 99) Then
		If (ISGN .gt. 0) Then
		    Write(TEMP,10) IDD,DLAB(1:LD),IDM,MLAB(1:LM),DS
10		    Format(I3,A,I2,A,F5.2,'$')
		    ND = 3
		Else
		    Write(TEMP,11) IDD,DLAB(1:LD),IDM,MLAB(1:LM),DS
11		    Format('-',I3,A,I2,A,F5.2,'$')
		    ND = 4
		Endif
	    Elseif (IDD .gt. 9) Then
		If (ISGN .gt. 0) Then
		    Write(TEMP,10) IDD,DLAB(1:LD),IDM,MLAB(1:LM),DS
12		    Format(I2,A,I2,A,F5.2,'$')
		    ND = 2
		Else
		    Write(TEMP,13) IDD,DLAB(1:LD),IDM,MLAB(1:LM),DS
13		    Format('-',I2,A,I2,A,F5.2,'$')
		    ND = 3
		Endif
	    Else
		If (ISGN .gt. 0) Then
		    Write(TEMP,14) IDD,DLAB(1:LD),IDM,MLAB(1:LM),DS
14		    Format('-',I1,A,I2,A,F5.2,'$')
		    ND = 2
		Else
		    Write(TEMP,15) IDD,DLAB(1:LD),IDM,MLAB(1:LM),DS
15		    Format(I1,A,I2,A,F5.2,'$')
		    ND = 3
		Endif
	    Endif
	    LZ = LD + LM + ND + 3
	    If (TEMP(LZ:LZ) .eq. ' ') TEMP(LZ:LZ) = '0'
	    LZ = LD + ND + 1
	    If (TEMP(LZ:LZ) .eq. ' ') TEMP(LZ:LZ) = '0'
	    Call PADNULL (TEMP)
	    Call VLEN (TEMP,LTEMP)

c  Remove trailing zeroes of fractional part of seconds

	    If (TEMP(LTEMP:LTEMP) .eq. '0') Then
		TEMP(LTEMP:LTEMP) = ' '
		LTEMP = LTEMP - 1
		Endif
	    If (TEMP(LTEMP:LTEMP) .eq. '0') Then
		TEMP(LTEMP-1:LTEMP) = '  '
		LTEMP = LTEMP - 2
		Endif
	    TEMP(LTEMP+1:LTEMP+1) = '$'
	    Call PADNULL (TEMP)
	    Call CONSTR (TEMP,'"')

c  If seconds are zero, don't bother with them; print minutes

	Elseif ((PREC .eq. 1) .or. (IDM .gt. 0)) Then
	    If (IDD .gt. 99) Then
		If (ISGN .gt. 0) Then
		    Write(TEMP,20) IDD,DLAB(1:LD),IDM,MLAB(1:LM)
20		    Format(I3,A,I2,A,'$')
		    ND = 3
		Else
		    Write(TEMP,21) IDD,DLAB(1:LD),IDM,MLAB(1:LM)
21		    Format('-',I3,A,I2,A,'$')
		    ND = 4
		Endif
	    Elseif (IDD .gt. 9) Then
		If (ISGN .gt. 0) Then
		    Write(TEMP,22) IDD,DLAB(1:LD),IDM,MLAB(1:LM)
22		    Format(I2,A,I2,A,'$')
		    ND = 2
		Else
		    Write(TEMP,23) IDD,DLAB(1:LD),IDM,MLAB(1:LM)
23		    Format('-',I2,A,I2,A,'$')
		    ND = 3
		Endif
	    Else
		If (ISGN .gt. 0) Then
		    Write(TEMP,24) IDD,DLAB(1:LD),IDM,MLAB(1:LM)
24		    Format(I1,A,I2,A,'$')
		    ND = 1
		Else
		    Write(TEMP,25) IDD,DLAB(1:LD),IDM,MLAB(1:LM)
25		    Format('-',I1,A,I2,A,'$')
		    ND = 2
		Endif
	    Endif
	    LZ = LD + ND + 1
	    If (TEMP(LZ:LZ) .eq. ' ') TEMP(LZ:LZ) = '0'

c  If minutes and seconds are zero, print only degrees

	Else
	    If (.not. DRAW) LD = 0
	    If (IDD .gt. 99) Then
		If (ISGN .gt. 0) Then
		    Write(TEMP,30) IDD,DLAB(1:LD)
30		    Format(I3,A,'$')
		Else
		    Write(TEMP,31) IDD,DLAB(1:LD)
31		    Format('-',I3,A,'$')
		Endif
	    Elseif (IDD .gt. 9) Then
		If (ISGN .gt. 0) Then
		    Write(TEMP,32) IDD,DLAB(1:LD)
32		    Format(I2,A,'$')
		Else
		    Write(TEMP,33) IDD,DLAB(1:LD)
33		    Format('-',I2,A,'$')
		Endif
	    Else
		If (ISGN .gt. 0) Then
		    Write(TEMP,34) IDD,DLAB(1:LD)
34		    Format(I1,A,'$')
		Else
		    Write(TEMP,35) IDD,DLAB(1:LD)
35		    Format('-',I1,A,'$')
		Endif
	    Endif
	Endif

c  Move temporary string to return string

	LSTR = Len (STRING)
	Call VLEN (TEMP,LTEMP)
	Do 40 I = 1, LSTR
	    If (TEMP(I:I) .ne. ' ') Then
		L0 = I
		LTEMP = LTEMP - L0 + 1
		Go to 50
		Endif
40	    Continue
50	L = LTEMP
	If (LSTR .lt. LTEMP) L = LSTR
	Call ZSTRING (STRING)
	STRING = TEMP(L0:L)
	Call PADNULL (STRING)

	Return
	End


c--- D2R  Convert declination in fractional degrees to radians

	Real*8 Function D2R (DEC)

	Real*8 DEC
c				Declination in degrees

	Include 'constants.cmb'

	If (RCON .lt. 0.01d0) Call CONSET
	D2R = DEC * RCON
	Return
	End


c--- R2D  Convert declination in radians to fractional degrees

	Real*8 Function R2D (DEC)

	Real*8 DEC
c				Declination in radians

	Include 'constants.cmb'

	If (RCON .lt. 0.01d0) Call CONSET
	R2D = DEC / RCON
	Return
	End

c Sep  2 1992	Convert DEC to degrees before calling HCON programs
c Oct 16 1992	Declare undeclared variables
c Nov 18 1992	Fix bug in DCOND

c Jan 26 1993	Add DCONF to convert from degrees to dd.mmsss

c Feb  7 1995	Add DCSTRT to drop unused part of strings
c Feb  8 1995	Add DCSTRC to drop unused part of strings with colons
c Sep 18 1995	Add DCSTRS to produce seconds string
c Dec  8 1995	Add DCSTRH to produce vigesimal string
c Dec  8 1995	Remove hanging decimal point in seconds string

c Jan 16 1996	Do not drop trailing zeroes if ndec is negative
c Feb  2 1996	Fix colon-separated signed declination
c Sep 13 1996	Add DCSTRP to produce string with plotted superscripts

c Apr 16 1997	Declare undeclared variable nd in DCSTRP
c Jun  3 1997	Add third digit in dcstr for non-declination degrees > 99

c Apr  9 1998	Add DCONT to convert from table entry to radians
c Nov 23 1998	Add R2D() and D2R() functions

c Dec  1 2004	Include from local directory
