/*** File iolib/ibmcon.c
 *** December 16, 1985
 *** By Doug Mink
 */

#include <stdio.h>
#include <stdlib.h>

static double xexp[127];
static double r4norm = 16777216.0;
static double r8norm = 4294967296.0;

ibmr4_ (buff,xnum)

/*-- Convert IBM Real*4 tape floating point number to VAX floating point format

/*	Call IBMR4 (BUFF0,XNUM)

	Integer*4 BUFF0		4 bytes to convert
	Real*4 XNUM		4-byte VAX floating point number (returned)
*/

unsigned char buff[4];
float *xnum;
{
	double mantis;		/* Mantissa for floating point */
	double dsgn;		/* Sign value */
	int iexpt; 		/* Exponent for floating point */
	int inum;		/* Value of mantissa */
	int lbuff[4];
	int i;

/*  Compute all possible exponent values to avoid exponentiation */

	if (xexp[64] == 0) {
	    xexp[64] = 1.0;
	    for (i=1; i < 32; i++) {
		xexp[64+i] = xexp[64+i-1] * 16.0;
		xexp[64-i] = 1.0 / xexp[64+i];
		}
	    }
	for (i=0; i<4; i++) lbuff[i] = (int) buff[i];

	/*  Set sign  */

	if (lbuff[0] > 127) {
	    dsgn = -1.0; }
	else {
	    dsgn = 1.0; }

	/*  Exponent (excess 64)  */

	iexpt = lbuff[0] & 127;

	inum = (((lbuff[1] << 8) + lbuff[2]) << 8) + lbuff[3];

/*  If input is hard zero, return it unchanged  */

	if (inum == 0) *xnum = 0.0;

/*  Otherwise translate it  */

	else {

	/*  Mantissa  */

	    mantis = ((double) inum) / r4norm;

/*  Compute number */

	    if (iexpt > 95) {
		fprintf (stderr,"IBMR4:  Exponent overflow = %d \n",iexpt);
		*xnum = 0.;  }
	    else if (iexpt < 33)
		*xnum= 0.;
	    else
		*xnum = dsgn * mantis * xexp[iexpt];
	    }

	return;
}


ibmr8_ (buff,dnum)

/*-- Convert IBM Real*8 tape floating point number to VAX floating point format

/*	Call IBMR8 (BUFF,DNUM)

	Integer*4 BUFF(2)		4 bytes to convert
	Real*8 DNUM		4-byte VAX floating point number (returned)
*/

unsigned char buff[8];
double *dnum;
{
	double mantis;		/* Mantissa for floating point */
	double mantis2;		/* Mantissa for floating point */
	double dsgn;		/* Sign value */
	int iexpt; 		/* Exponent for floating point */
	int inum;		/* Value of mantissa */
	int inum2 ;		/* Value of mantissa */
	int lbuff[8];
	int i;

/*  Compute all possible exponent values to avoid exponentiation */

	if (xexp[64] == 0) {
	    xexp[64] = 1.0;
	    for (i=1; i < 32; i++) {
		xexp[64+i] = xexp[64+i-1] * 16.0;
		xexp[64-i] = 1.0 / xexp[64+i];
		}
	    }

	for (i=0; i<8; i++) lbuff[i] = (int) buff[i];

	/*  Set sign  */

	if (lbuff[0] > 127) {
	    dsgn = -1.0; }
	else {
	    dsgn = 1.0; }

	/*  Exponent (excess 64)  */

	iexpt = lbuff[0] & 127;

	inum = (((lbuff[1] << 8) + lbuff[2]) << 8) + lbuff[3];

/*  If input is hard zero, return it unchanged  */

	if (inum == 0) *dnum = 0.0;

/*  Otherwise translate it  */

	else {

	/*  Mantissa  */

	    mantis = (double) inum;
	    inum2 = (((lbuff[4] << 8) + lbuff[5]) << 8) + lbuff[6];
	    mantis2 = ((double) inum2 * 256.0) + (double) lbuff[7];
	    mantis = (mantis + (mantis2 / r8norm)) / r4norm;

/*  Compute number */

	    if (iexpt > 95) {
		fprintf (stderr,"IBMR8:  Exponent overflow = %d \n",iexpt);
		*dnum = 0.;
		}

	    else if (iexpt < 33) {
		fprintf (stderr,"IBMR8:  Exponent underflow = %d \n",iexpt);
		*dnum= 0.;
		}

	    else {
		*dnum = dsgn * mantis * xexp[iexpt];
		}

	    }

	return;
}
