c*** File iolib/isocon.f
c*** December 2, 2004
c*** By Doug Mink

c  DATE2JD (STRING, DJ)
c	Convert date string to Julian Date

c  DATE2SEC (STRING,TSEC)
c	Convert date string to seconds since 1950

c  DATE2VIG (STRING,DATE,TIME) 
c	Convert date string to date as yyyy.ddmm and time as hh.mmsss

c  DATE2SEC-- returns seconds since 1950

c  DATE2JD-- returns Julian Date

	Subroutine DATE2JD (STRING, JD)

	Character*(*) STRING
c		Character string containing ISO-format date string
	Real*8 JD
c		Julian Date (returned)

	Real*8 DATE, TIME

	Call DATE2VIG (STRING, DATE, TIME)
	Call VCONJ (DATE, TIME, JD)

	Return
	End


c  DATE2SEC-- returns seconds since 1950

	Subroutine DATE2SEC (STRING,TSEC)

	Character*(*) STRING
c		Character string containing ISO-format date string
	Real*8 TSEC
c		Seconds since 1950 (returned)

	Real*8 DATE, TIME

	Call DATE2VIG (STRING, DATE, TIME)
	Call VCON (DATE, TIME, TSEC)
	Return
	End


c DATE2VIG-- returns date as yyyy.mmdd and time as hh.mmssss

	Subroutine DATE2VIG (STRING,DATE,TIME)

	Character*(*) STRING
c		Character string containing ISO-format date string
c		or Old FITS dd/mm/[cc]yy
c		or Bad FITS dd-mm-[cc]yy
c		or vigesimal date [cc]yy.mmdd
c		or epoch [yy]yy.fff, [cc]yy.fffff
c		[cc] defaults to 19
	Real*8 DATE
c		UT date as yyyy.mmdd (returned)
	Real*8 TIME
c		UT time as hh.mmssss (returned)
c		* 0:00:00 returned if not set

	Character*1 SLASH, DASH, COLON, DOT
	Integer*4 IX1, IX2, IX3, IX4, IX5, IX6, NC, LV, N1, N2, N3
	Real*8 YR, MN, DAY, DFRAC, TSEC, TSEC0, TSEC1, YDATE0, YDATE1
	Real*8 HR, MIN, SEC
	Integer*4 IER
	Include 'header.cmb'

	YR = 0
	MN = 0
	DAY = 0
	TIME = 0.d0
	SLASH = '/'
	DASH = '-'
	COLON = ':'
	DOT = '.'

c  Check for old FITS date format (dd/mm/yy)
	IX1 = Index (STRING,SLASH)
	IF (IX1 .gt. 0) Then

c  Extract date from old FITS date format (allowing optional 4-digit year)
	    NC = IX1 - 1
	    Call ASCBINI4 (NC,STRING,N1,IER)
	    IX2 = IX1 + Index (STRING(IX1+1:),SLASH)
	    NC = IX2 - IX1 - 1
	    Call ASCBINI4 (NC,STRING(IX1+1:),N2,IER)
	    Call VLEN (STRING, LV)
	    NC = LV - IX2
	    Call ASCBINI4 (NC,STRING(IX2+1:),N3,IER)
	    If (N1 .gt. 31) Then
		YR = N1
		MN = N2
		DAY = N3
	    Elseif (N2 .lt. 13) Then
		DAY = N1
		MN = N2
		YR = N3
	    Else
		DAY = N2
		MN = N1
		YR = N3
	    Endif
	    If (YR .lt. 100.d0) YR = YR + 1900.d0
	    DATE = Dble (YR) + (1.d-2 * Dble (MN)) + (1.d-4 * Dble (DAY))
	    Return
	    Endif

c  Check for ISO date format (ccyy-mm-dd[Thh:mm:ss[.sss...]])
	IX1 = Index (STRING, DASH)
	IF (IX1 .gt. 0) Then

c  Extract date from ISO format (or bad fits format) (2-digit year optional)
	    NC = IX1 - 1
	    Call ASCBINI4 (NC,STRING,N1,IER)
	    IX2 = IX1 + Index (STRING(IX1+1:),DASH)
	    NC = IX2 - IX1 - 1
	    Call ASCBINI4 (NC,STRING(IX1+1:),N2,IER)
	    Call VLEN (STRING, LV)
	    IX3 = Index (STRING, 'T')
	    If (IX3 .gt. 0) Then
		NC = IX3 - IX2 - 1
	    Else
		NC = LV - IX2 + 1
	    Endif
	    Call ASCBINI4 (NC,STRING(IX2+1:),N3,IER)
	    If (N3 .gt. 31) Then
		If (N2 .lt. 13) Then
		    YR = N3
		    MN = N2
		    DAY = N1
		Else
		    YR = N3
		    MN = N1
		    DAY = N2
		Endif
	    Else
		YR = N1
		MN = N2
		DAY = N3
	    Endif
	    If (YR .lt. 100) YR = YR + 1900
	    DATE = Dble (YR) + (1.d-2 * Dble (MN)) + (1.d-4 * Dble (DAY))
	    If (IX3 .le. 0) Return

c  Extract time from ISO format, if it is present
	    HR = 0.d0
	    MIN = 0.d0
	    SEC = 0.d0
	    IX4 = Index (STRING,COLON)
	    NC = IX4 - IX3 - 1
	    Call ASCBINR8 (NC,STRING(IX3+1:),HR,IER)
	    IX5 = IX4 + Index (STRING(IX4+1:),COLON)
	    If (IX5 .gt. IX4) Then
		NC = IX5 - IX4 - 1
		Call ASCBINR8 (NC,STRING(IX4+1:),MIN,IER)
	    	IX6 = Index (STRING,'Z')
		If (IX6 .gt. 0) Then
		    NC = IX6 - IX5
		Else
		    NC = LV - IX5
		Endif
		If (NC .gt. 0) Then
		    Call ASCBINR8 (NC,STRING(IX5+1:),SEC,IER)
		    Endif
		Endif
	    TIME = HR + (1.d-2 * MIN) + (1.d-4 * SEC)
	    Return
	    Endif

c  Return first of year as date if integer year
	IX1 = Index (STRING, DOT) + 1
	Call VLEN (STRING, NC)
	Call ASCBINR8 (NC,STRING,DATE,IER)
	If (DATE .lt. 100.0) DATE = DATE + 1900.d0
	If (IX1 .lt. 2) Then
	    DATE = DATE + .0101d0
	    Return
	    Endif

c  Date as fraction of year if not four decimal places
	Call VLEN (STRING(IX1:), NC)
	If (NC .ne. 4) Then
	    DFRAC = Dmod (DATE,1.d0)
	    YDATE0 = DATE - DFRAC + 0.0101d0
	    Call VCON (YDATE0, 0.d0, TSEC0)
	    YDATE1 = YDATE0 + 1.d0
	    Call VCON (YDATE1, 0.d0, TSEC1)
	    TSEC = TSEC0 + DFRAC * (TSEC1 - TSEC0)
	    Call VCON1 (TSEC, DATE, TIME)
	    Return
	    Endif
	
c  Date as vigesimal [yy]yy.mmdd if four decimal places
	TIME = 0.d0
	Return
	End

c Feb  8 1998	New subroutine
c Feb 27 1998	Fix bugs found by Alpha compilation

c Jun 25 1999	Fix bug handling times in date2vig(); add date2jd()

c Dec  2 2004	Fix bug computing vigesimal time
