c*** File iolib/pcon.f
c*** February 22, 1993
c*** By Doug Mink

c  Sky position conversion subroutines

c	PCONH (HRA,HDEC,RA,DEC) hh.mmssss dd.mmssss to radians.
c	PCONH1 (RA,DEC,HRA,HDEC) radians to hh.mmssss dd.mmssss.
c	PCONHS (RA,DEC,NDRA,STRING) hh.mmssss dd.mmssss to string
c	PCOND (HRA,HDEC,THETA,PHI) hh.mmssss dd.mmssss to degrees.
c	PCOND1 (THETA,PHI,HRA,HDEC) degrees to hh.mmssss dd.mmssss
c	PCONDS (RA,DEC,NDRA,STRING) degrees to string
c	PCONRS (RA,DEC,NDRA,STRING) radians to string

c--- PCONH  Convert RA and Dec from hh.mmssss dd.mmssss to radians.

	Subroutine PCONH (HRA,HDEC,RA,DEC)

	Real*8 HRA
c			Right ascension (hh.mmssss)
	Real*8 HDEC
c			Declination (dd.mmssss)
	Real*8 RA
c			Right ascension (radians)
	Real*8 DEC
c			Declination (radians)

	If (HRA .ne. 0.d0) Then
	    Call HCONR (HRA,RA)
	    RA = RA * 15.d0
	Else
	    RA = 0.d0
	Endif
	If (HDEC .ne. 0.d0) Then
	    Call HCONR (HDEC,DEC)
	Else
	    DEC = 0.d0
	Endif
	Return
	End


c--- PCONH1  Convert RA and Dec from radians to hh.mmssss dd.mmssss.

	Subroutine PCONH1 (RA,DEC,HRA,HDEC)

	Real*8 RA
c			Right ascension (radians)
	Real*8 DEC
c			Declination (radians)
	Real*8 HRA
c			Right ascension (hh.mmssss)
	Real*8 HDEC
c			Declination (dd.mmssss)

	Real*8 RA1

	If (RA .ne. 0.d0) Then
	    RA1 = RA / 15.d0
	    Call HCONS (RA1,HRA)
	Else
	    HRA = 0.d0
	Endif
	If (DEC .ne. 0.d0) Then
	    Call HCONS (DEC,HDEC)
	Else
	    HDEC = 0.d0
	Endif
	Return
	End


c--- PCOND  Convert RA and Dec from hh.mmssss dd.mmssss to degrees.

	Subroutine PCOND (HRA,HDEC,THETA,PHI)

	Real*8 HRA
c			Right ascension (hh.mmssss)
	Real*8 HDEC
c			Declination (dd.mmssss)
	Real*8 THETA
c			Right ascension (degrees)
	Real*8 PHI
c			Declination (degrees)

	If (HRA .ne. 0.d0) Then
	    Call HCON (HRA,THETA)
	    THETA = THETA * 15.d0
	Else
	    THETA = 0.d0
	Endif
	If (HDEC .ne. 0.d0) Then
	    Call HCON (HDEC,PHI)
	Else
	    PHI = 0.d0
	Endif
	Return
	End


c--- PCOND1  Convert RA and Dec from degrees to hh.mmssss dd.mmssss.

	Subroutine PCOND1 (THETA,PHI,HRA,HDEC)

	Real*8 THETA
c			Right ascension (radians)
	Real*8 PHI
c			Declination (radians)
	Real*8 HRA
c			Right ascension (hh.mmssss)
	Real*8 HDEC
c			Declination (dd.mmssss)

	Real*8 THETA1

	If (THETA .ne. 0.d0) Then
	    THETA1 = THETA / 15.d0
	    Call HCON2 (THETA1,HRA)
	Else
	    HRA = 0.d0
	Endif
	If (PHI .ne. 0.d0) Then
	    Call HCONS (PHI,HDEC)
	Else
	    HDEC = 0.d0
	Endif
	Return
	End


c--- PCONHS  Convert RA and Dec from hh.mmssss dd.mmssss to string.

	Subroutine PCONHS (HRA,HDEC,NDRA,STRING)

	Real*8 HRA
c			Right ascension (hh.mmssss)
	Real*8 HDEC
c			Declination (dd.mmssss)
	Integer*4 NDRA
c			Number of decimal places in seconds of RA
	Character*(*) STRING
c			Position string (returned)

	Real*8 RA,DEC

	Call HCON (HRA,RA)
	RA = RA * 15.d0
	Call HCON (HDEC,DEC)

	Call PCONDS (RA,DEC,NDRA,STRING)

	Return
	End


c--- PCONRS  Convert RA and Dec from radians to string.

	Subroutine PCONRS (RA,DEC,NDRA,STRING)

	Real*8 RA
c			Right ascension (radians)
	Real*8 DEC
c			Declination (radians)
	Integer*4 NDRA
c			Number of decimal places in seconds of RA
	Character*(*) STRING
c			Position string (returned)

	Real*8 DEC1,RA1

	Include 'constants.cmb'

c  Convert from radians to degrees

	RA1 = RA / RCON
	DEC1 = DEC / RCON

	Call PCONDS (RA1,DEC1,NDRA,STRING)

	Return
	End


c--- PCONDS  Convert RA and Dec from degrees to string.

	Subroutine PCONDS (RA,DEC,NDRA,STRING)

	Real*8 RA
c			Right ascension (hh.mmssss)
	Real*8 DEC
c			Declination (dd.mmssss)
	Integer*4 NDRA
c			Number of decimal places in seconds of RA
	Character*(*) STRING
c			Position string (returned)

	Real*8 RA1,RS,DS
	Character LINE*80,TEMP*30,JSGN*1,LFORMAT*20
	Integer*4 NFIELD,ISGN,IRH,IRM,IRS,IDD,IDS,IDM,I,NDDEC

	Include 'constants.cmb'

c  Set output format for seconds of RA

	NFIELD = NDRA + 4
	If (NDRA .le. 0) Then
	    LFORMAT = '(I3)'
	Elseif (NFIELD .gt. 9) Then
	    Write(LFORMAT,10) NFIELD,NDRA
10	    Format('(F',I2,'.',I1,')')
	Else
	    Write(LFORMAT,11) NFIELD,NDRA
11	    Format('(F',I1,'.',I1,')')
	Endif

c  Convert right ascension to hours, minutes, and seconds

	RA1 = RA / 15.d0
	If (NDRA .gt. 0) Then
	    Call HCON1 (RA1,ISGN,IRH,IRM,RS)
	Else
	    Call HCON3 (RA1,ISGN,IRH,IRM,IRS)
	Endif

c  Write hours and minutes to string

	Write(LINE,20) IRH,IRM
20	Format(I2,I3,'$')
	Call PADNULL (LINE)
	If (LINE(4:4) .eq. ' ') LINE(4:4) = '0'

c  Write seconds to string

	If (NDRA .gt. 0) Then
	    Write(TEMP,LFORMAT) RS
	Else
	    Write(TEMP,'(I3)') IRS
	Endif
	If (TEMP(2:2) .eq. ' ') TEMP(2:2) = '0'
	If ((NDRA .gt. 0) .and. (TEMP(5:5) .eq. ' ')) Then
	    Do 25 I = 5, NDRA+4
25		TEMP(I:I) = '0'
	    Endif
	Call PADNULL (TEMP)
	Call CONSTR (LINE,TEMP)
	Call CONSTR (LINE,'  $')

c  Set output format for seconds of RA

	NDDEC = NDRA - 1
	If (NDDEC .le. 0) Then
	    LFORMAT = '(I3)'
	Elseif (NDDEC+4 .gt. 9) Then
	    Write(LFORMAT,40) NDDEC+4,NDDEC
30	    Format('(F',I2,'.',I1,')')
	Else
	    Write(LFORMAT,31) NDDEC+4,NDDEC
31	    Format('(F',I1,'.',I1,')')
	Endif

c  Convert declination to degrees, minutes, and seconds

	If (NDDEC .gt. 0) Then
	    Call HCON1 (DEC,ISGN,IDD,IDM,DS)
	Else
	    Call HCON3 (DEC,ISGN,IDD,IDM,IDS)
	Endif

c  Write sign, degrees, and minutes to string

	JSGN = ' '
	If (ISGN .lt. 0) JSGN = '-'
	Write(TEMP,40) JSGN,IDD,IDM
40	Format(A1,I2,I3,' $')
	Call PADNULL (TEMP)
	If (TEMP(2:2) .eq. ' ') Then
	    TEMP(2:2) = TEMP(1:1)
	    TEMP(1:1) = ' '
	    Endif
	If (TEMP(5:5) .eq. ' ') TEMP(5:5) = '0'
	Call PADNULL (TEMP)
	Call CONSTR (LINE,TEMP)

c  Write seconds to string

	If (NDDEC .gt. 0) Then
	    Write(TEMP,LFORMAT) DS
	Else
	    Write(TEMP,LFORMAT) IDS
	Endif
	If (TEMP(2:2) .eq. ' ') TEMP(2:2) = '0'
	If ((NDRA .gt. 0) .and. (TEMP(5:5) .eq. ' ')) Then
	    Do 50 I = 5, NDRA+3
50		    TEMP(I:I) = '0'
	    Endif
	Call PADNULL (TEMP)
	Call CONSTR (LINE,TEMP)

	STRING = LINE

	Return
	End

c Feb 22 1993	Fix pconds by using ndra instead of ndecra and assigning string
