/*
 *	pstek - convert Tektronix 4014 escape sequences to PostScript
 *
 *	Written by:
 *		Edward Moy
 *		Academic Computing Services
 *		University of California
 *		Berkeley, CA  94720
 *
 *		edmoy@opal.Berkeley.EDU
 *		ucbvax!opal!edmoy
 *
 *	Requires Tekparse.h and Tekparsetable.c, which is part of the
 *	xterm 6.6* distribution.
 *
 *	Modified by:
 *		Mic Kaczmarczik
 *		User Services Digital Support Group
 *		Computation Center
 *		University of Texas at Austin
 *		Austin, TX 78712
 *
 *		mic@emx.utexas.edu
 *		...!cs.utexas.edu!ut-emx!mic
 *
 *		-- Modified the "noerase" flag to do a showpage if the
 *		   page is not to be erased.  This seems more useful
 *		   for the applications we have.
 *		-- Added 'd' flag do disable showpage's and document comments
 *		-- Added 'i' flag to ignore initial screen clear sequence
 *		-- Modified the exit() to avoid generating a useless
 *		   and annoying VMS status message.
 *		-- Shortened some names in order to be compiled using
 *		   the TOPS-20 KCC C compiler (6-character identifiers)
 *		-- Added -o flag (makes life a lot easier under VMS)
 *		-- Added -n option (for scaling pen size)
 *		-- Remove assumption that open() always returns lowest
 *		   file descriptor
 *		-- Add %begin(plot), %end(plot), and %%BoundingBox
 *
 *	Modified by:
 *		Bryon Hance
 *		User Services VAX/VMS Support
 *		Computation Center
 *		University of Texas at Austin
 *		Austin, TX 78712
 *
 *		-- Modified the PostScript prolog to divide the screen
 *		   into smaller graphic objects.  This was done to fix
 *		   a problem handling complicated tek screens (large).
 *		   This modification was made to the /L def statement,
 *		   and is only a temperary fix.
 *      Modified by:
 *              Susan Deniso
 *		User Services VAX/VMS Support
 *		Computation Center
 *		University of Texas at Austin
 *		Austin, TX 78712
 *
 *              --Corrected type conflict between atof() function and
 *                the double defined options -w, -h, -x, -y, and -n.
 *
 *	Modified yet again by MPK 7/23/89 to modify the ``document''
 *	flag to output something closer to Encapsulated PostScript
 *
 *	Modified to print and renamed PSTEK, February 16, 1993
 *	by Doug Mink, Harvard-Smithsonian Center for Astrophysics
 *
 *	Added -b option to supress burst page, June 26, 1995
 *	by Doug Mink, Harvard-Smithsonian Center for Astrophysics
 *
 *	Changed non-ANSI C rindex to strrchr, July 26, 1995
 *	by Doug Mink, Harvard-Smithsonian Center for Astrophysics
 */
#include "Tekparse.h"
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#define	COURIER		(10. / 6.)	/* ratio of height to width */
#define	DEF_HEIGHT	10.0
#define	DEF_WIDTH	7.5
#define	FALSE		0
#define	MAXPATH		1000
#define	PAGE_HEIGHT	11.0
#define	PAGE_WIDTH	8.5
#define	PSINCHES	72
#define	S_H		010
#define	S_W		04
#define	S_X		01
#define	S_Y		02
#define	TRUE		1

/* Tek defines */

#define	BEL		07
#define	CANCEL		030
#define	DOTDASHEDLINE	2
#define	DOTTEDLINE	1
#define	EAST		01
#define	ETX		03
#define	LARGEFONT	0
#define	LINEMASK	07
#define	LONGDASHEDLINE	4
#define	MARGIN1		0
#define	MARGIN2		1
#define MAX_PTS		150
#define MAX_VTX		300
#define	NAK		025
#define	NORTH		04
#define	PENDOWN		1
#define	PENUP		0
#define	SHORTDASHEDLINE	3
#define	SMALLFONT	3
#define	SOLIDLINE	0
#define	SOUTH		010
#define	TEKBOTTOMPAD	24
#define	TEKFULLHEIGHT	(TEKHEIGHT + TEKTOPPAD + TEKBOTTOMPAD)
#define	TEKFULLWIDTH	(TEKWIDTH + TEKPAD)
#define	TEKHEIGHT	3072
#define	TEKHOME		((TekChar[fontsize].nlines - 1)\
			 * TekChar[fontsize].vsize)
#define	TEKNUMFONTS	4
#define	TEKPAD		58
#define	TEKTOPPAD	34
#define	TEKWIDTH	4096
#define	THREEFONT	2
#define	TWOFONT		1
#define	WEST		02

#ifdef	TOPS20
/* Make global identifiers unique to 6 (uppercase) characters */
#define	Tpushback Tpshbck
#define	TCursorUp TCUp
#define	TCursorDown TCDown
#define	TCursorBack TCBack
#define	TCursorForward TCForward
#define	cur_X curXXX
#define cur_Y curYYX
#define psprolog2 pspr2
#endif

#define	GOOD	0		/* good exit status */

#ifdef	VMS
#include <ssdef.h>
#undef	GOOD
#define	GOOD	SS$_NORMAL
#endif

int Tbcnt = 0;
char Tbuffer[BUFSIZ];
char *Tbptr = Tbuffer;
char Tpushb[BUFSIZ];
char *Tpushback = Tpushb;

/* use comma operator so these macros act like one line (hack!!!) */
#define TekFlush()	fputs("d\n", Tekout), nplot = 0
#define TekMove(x,y)	cur_X = (x), cur_Y = (y)
#define	input()		Tinput()
#define	unput(c)	*Tpushback++ = (c)

static struct Tek_Char {
	int hsize;	/* in Tek units */
	int vsize;	/* in Tek units */
	int charsperline;
	int nlines;
} TekChar[TEKNUMFONTS] = {
	{56, 88, 74, 35},	/* large */
	{51, 82, 81, 38},	/* #2 */
	{34, 53, 121, 58},	/* #3 */
	{31, 48, 133, 64},	/* small */
};

static int cur_X;
static int cur_Y;
static int cur_x;
static int cur_y;
static int fontsize;
static lastfont = -1;
static int linetype;
static int margin;
static int noerase;
static int document = FALSE;
static double penscale = 1.0;
static int ignorefirst = FALSE;
static int keeptemp = FALSE;
static int printfile = TRUE;
static int pageno = 1;
static int nplot;
static int pen;
static int infd = 0;

extern int Talptable[];
extern int Tbestable[];
extern int Tbyptable[];
extern int Tesctable[];
extern int Tipltable[];
extern int Tplttable[];
extern int Tpttable[];
extern int Tspttable[];

static int *curstate = Talptable;
static int *Tparsestate = Talptable;

FILE	*Tekout = stdout;
char	*printname;
char	*lprflags[10];
char	hccom[32];
int	nflags = 0;
int	cpid;

char **filearg;
char *myname;
int nfiles;

main(argc, argv)
int argc;
char **argv;
{
	register int landscape = TRUE;
	register int set = 0;
	register double x, y, w, h;
        double atof();
	char *strrchr();
	char arg[20];
	char nohead[4];

	strcat (hccom, "/usr/ucb/lpr");
	lprflags[0] = hccom;
	strcat (nohead, "-h");

	if(myname = strrchr(*argv, '/'))
		myname++;
	else
		myname = *argv;
	for (argc--, argv++ ; argc > 0 && **argv == '-' ; argc--, argv++)
		switch((*argv)[1]) {
		 case 'b':	/* suppress burst page */
			lprflags[++nflags] = nohead;
			break;
		 case 'c':	/* hard copy command */
			if (argc < 2)
			    Usage();
			lprflags[0] = *++argv;
		 case 'd':	/* document */
			document = TRUE;
			break;
		 case 'h':
			if(argc < 2)
			    Usage();
			h = (double)atof(*++argv);
			argc--;
			set |= S_H;
			break;
		 case 'i':	/* ignore *first* PAGE */
			ignorefirst = TRUE;
			break;
		 case 'k':	/* keep temporary file if printing */
			keeptemp = TRUE;
			break;
		 case 'l':	/* landscape */
			landscape = TRUE;
			break;
		 case 'n':      /* scale the pen nib */
			if (argc < 2)
			    Usage();
			penscale = (double)atof(*++argv);
			argc--;
			break;
		 case 'o':
			if (argc < 2)
			    Usage();
			Tekout = fopen(*++argv,"w");
			if (Tekout == NULL) {
			    fprintf(stderr,"%s: can't open %s\n",
				    myname,*argv);
			    exit(1);
			    }
			printfile = FALSE;
			argc--;
			break;
		 case 'p':	/* Don't erase on PAGE */
			noerase = TRUE;
			break;
		 case 'r':	/* portrait */
			landscape = FALSE;
			break;
		 case 'w':
			if(argc < 2)
			    Usage();
			w = (double)atof(*++argv);
			argc--;
			set |= S_W;
			break;
		 case 'x':
			if(argc < 2)
			    Usage();
			x = (double)atof(*++argv);
			argc--;
			set |= S_X;
			break;
		 case 'y':
			if(argc < 2)
			    Usage();
			y = (double)atof(*++argv);
			argc--;
			set |= S_Y;
			break;
		 case 'P':	/* set printer name if not lp */
			if (argc < 2)
			    Usage();
			if ((*argv)[2] != NULL)
			    lprflags[++nflags] = *argv;
			else {
			    strcpy (arg,*argv);
			    lprflags[++nflags] = strcat (arg,*++argv);
			    argc--;
			    }
			break;

		 default:
			Usage();
		}

	/* Make temporary output file name and open it */
	printname = tempnam ("/tmp","pstek");
	Tekout = fopen (printname,"w");
	if (Tekout == NULL) {
	    fprintf(stderr,"%s: can't open %s\n",
		    myname,printname);
	    exit(1);
	    }
	else if (keeptemp == TRUE)
	    fprintf(stderr,"%s: writing to %s\n",
		    myname,printname);

	init (landscape, set, x, y, w, h, penscale);
	filearg = argv;
	if((nfiles = argc) > 0)
		nextfile();
	cur_X = 0;
	cur_Y = TEKHOME;
	Tekparse();

}

Usage()
{
	fprintf(stderr, "Usage: %s\n", myname);
	fprintf(stderr,"[-b suppress burst page>]\n");
	fprintf(stderr,"[-c <hard copy command>]\n");
	fprintf(stderr,"[-d (set document flag)]\n");
	fprintf(stderr,"[-h <height in inches>]\n");
	fprintf(stderr,"[-i (ignore first PAGE)]\n");
	fprintf(stderr,"[-k (keep temporary file if printing)]\n");
	fprintf(stderr,"[-l (landscape=default)]\n");
	fprintf(stderr,"[-n <pen nib scale>]\n");
	fprintf(stderr,"[-o <Postscript output file>]\n");
	fprintf(stderr,"[-p (Don't erase on PAGE)]\n");
	fprintf(stderr,"[-r (portrait)]\n");
	fprintf(stderr,"[-w <width in inches>]\n");
	fprintf(stderr,"[-x <lower left x in inches>]\n");
	fprintf(stderr,"[-y <lower left y in inches>]\n");
	fprintf(stderr,"[-P <printer name>]\n");
	exit(1);
}

char pshead[] = "\
%%%%Creator: %s\n\
%%%%CreationDate: %s\
%%%%DocumentFonts: Courier\n\
%%%%Pages: 1\n\
";

char psfont[] =
	"/f%d {/Courier findfont [%.2f 0 0 %d 0 0] makefont setfont} def\n";

char *psprolog[] = {
	"/in {72 mul} def\n",
	"/L {1 index 1 index moveto stroke moveto}def\n",
	"/l /lineto load def\n",
	"/d /stroke load def\n",
	"/p {3 1 roll moveto show} def\n",
	"/t0 {[] 0 setdash} def\n",
	"/t1 {[1 20] 0 setdash} def\n",
	"/t2 {[1 10 40 10] 0 setdash} def\n",
	"/t3 {[30] 0 setdash} def\n",
	"/t4 {[60 20] 0 setdash} def\n",
	"/initpage {\n",
	"  /s save def\n",
	0
};

char *psprolog2[] = {
	"  1 penscale mul setlinewidth 1 setlinecap\n",
	"} def\n",
	0
};

char *psprolog3[] = {
	"initpage\n",
	0
};

double def_width[2] = {DEF_WIDTH, DEF_WIDTH * TEKFULLWIDTH / TEKFULLHEIGHT};
double def_height[2] = {DEF_WIDTH * TEKFULLHEIGHT / TEKFULLWIDTH, DEF_WIDTH};
double page_width[2] = {PAGE_WIDTH, PAGE_HEIGHT};
double page_height[2] = {PAGE_HEIGHT, PAGE_WIDTH};
int    bbox[4] = {0, 0, 612, 792};

init(landscape, set, x, y, w, h, penscale)
int landscape, set;
double x, y, w, h, penscale;
{
	register int i;
	register char **ptr;
	register double sx, sy;
	long l;
	char *ctime();

	fprintf(Tekout,"%%!PS-Adobe-2.0%s\n", document ? " EPSF-2.0" : "");
	time(&l);
	fprintf (Tekout,pshead, myname, ctime(&l));

	switch(set & (S_W | S_H)) {
	 case 0:
		w = def_width[landscape];
		h = def_height[landscape];
		break;
	 case S_W:
		h = w * TEKFULLHEIGHT / TEKFULLWIDTH;
		break;
	 case S_H:
		w = h * TEKFULLWIDTH / TEKFULLHEIGHT;
		break;
	}
	sx = PSINCHES * w / TEKFULLWIDTH;
	sy = PSINCHES * h / TEKFULLHEIGHT;
	switch(set & (S_X | S_Y)) {
	 case 0:
		x = (page_width[landscape] - w) / 2.0;
		y = (page_height[landscape] - h) / 2.0;
		break;
	 case S_X:
		y = (page_height[landscape] - h) / 2.0;
		break;
	 case S_Y:
		x = (page_width[landscape] - w) / 2.0;
		break;
	}

	/* calculate the PostScript bounding box */
	bbox[0] = 0;
	bbox[1] = 0;
	bbox[2] = PSINCHES * w;
	bbox[3] = PSINCHES * h;
	if (!document) {
		bbox[2] += (bbox[0] = PSINCHES * x);
		bbox[3] += (bbox[1] = PSINCHES * y);
	}
	fprintf(Tekout,"%%begin(plot)\n%%%%BoundingBox: %d %d %d %d\n%%\n",
	       bbox[0],bbox[1],bbox[2],bbox[3]);

	fputs("%%EndComments\n",Tekout);

	/* define the fonts we use */
	for(i = 0 ; i < 4 ; i++)
		fprintf(Tekout,psfont, i, COURIER * TekChar[i].hsize,TekChar[i].vsize);

	for(ptr = psprolog ; *ptr ; ptr++)
		fputs(*ptr, Tekout);

	/* This stuff is inside the definition of initpage */

	fprintf(Tekout,"  /document %s def\n", document ? "true" : "false");
	fprintf(Tekout,"  /landscape %s def\n", landscape ? "true" : "false");
	fprintf(Tekout,"  /penscale %f def\n",penscale);
	fprintf(Tekout,"  document not {\n");
	fprintf(Tekout,"    landscape { 8.5 in 0 translate 90 rotate } if\n");
	fprintf(Tekout,"    %.2f %.2f translate\n", PSINCHES * x, PSINCHES * y);
	fprintf(Tekout,"  } if\n");
	fprintf(Tekout,"  %f %f scale\n", sx, sy);
	for(ptr = psprolog2 ; *ptr ; ptr++)
		fputs(*ptr, Tekout);

	fputs("%%EndProlog\n",Tekout);
	if (!document)
		fputs("%%Page: 1 1\n",Tekout);
	for(ptr = psprolog3 ; *ptr ; ptr++)
		fputs(*ptr, Tekout);

}

Tekparse()
{
	register int c, x, y;
	register char *cp;
	char text[BUFSIZ];

	for( ; ; )
		switch(Tparsestate[c = input()]) {
		 case CASE_REPORT:	/* report address */
		 case CASE_VT_MODE:	/* special return to vt102 mode */
		 case CASE_BEL:		/* BEL */
		 case CASE_COPY:	/* make copy */
		 case CASE_CURSTATE:
			Tparsestate = curstate;	/* clear bypass condition */
			break;

		 case CASE_SPT_STATE:	/* Enter Special Point Plot mode */
			Tparsestate = curstate = Tspttable;
			break;

		 case CASE_GIN:		/* Do Tek GIN mode */
			Tparsestate = Tbyptable;	/* Bypass mode */
			break;

		 case CASE_BS:		/* BS */
			Tparsestate = curstate;	/* clear bypass condition */
			TCursorBack();
			break;

		 case CASE_PT_STATE:	/* Enter Tek Point Plot mode */
			Tparsestate = curstate = Tpttable;
			break;

		 case CASE_PLT_STATE:	/* Enter Tek Plot mode */
			Tparsestate = curstate = Tplttable;
			if((c = input()) == BEL)
				pen = PENDOWN;
			else {
				unput(c);
				pen = PENUP;
			}
			break;

		 case CASE_TAB:		/* HT */
			Tparsestate = curstate;	/* clear bypass condition */
			TCursorForward();
			break;

		 case CASE_IPL_STATE:	/* Enter Tek Incremental Plot mode */
			Tparsestate = curstate = Tipltable;
			break;

		 case CASE_ALP_STATE:	/* Enter Tek Alpha mode */
			/* if in one of graphics states, move alpha cursor */
			if(nplot > 0) /* flush line Tbuffer */
				TekFlush();
			Tparsestate = curstate = Talptable;
			break;

		 case CASE_UP:		/* cursor up */
			Tparsestate = curstate;	/* clear bypass condition */
			TCursorUp();
			break;

		 case CASE_PAGE:	/* Page Function */
			TekPage();	/* clear bypass condition */
			break;

		 case CASE_BES_STATE:	/* Byp: an escape char */
			Tparsestate = Tbestable;
			break;

		 case CASE_BYP_STATE:	/* set bypass condition */
			Tparsestate = Tbyptable;
			break;

		 case CASE_IGNORE:	/* Esc: totally ignore CR, ESC, LF, ~ */
			break;

		 case CASE_ASCII:	/* Select ASCII char set */
			/* ignore for now */
			Tparsestate = curstate;
			break;

		 case CASE_APL:		/* Select APL char set */
			/* ignore for now */
			Tparsestate = curstate;
			break;

		 case CASE_CHAR_SIZE:	/* character size selector */
			fontsize = c & 03;
			Tparsestate = curstate;
			break;

		 case CASE_BEAM_VEC:	/* beam and vector selector */
			/* only line types */
			if((c &= LINEMASK) != linetype) {
				if(nplot > 0)
					TekFlush();
				linetype = c;
			}
			Tparsestate = curstate;
			break;

		 case CASE_PENUP:	/* Ipl: penup */
			pen = PENUP;
			break;

		 case CASE_PENDOWN:	/* Ipl: pendown */
			pen = PENDOWN;
			break;

		 case CASE_IPL_POINT:	/* Ipl: point */
			x = cur_X;
			y = cur_Y;
			if(c & NORTH)
				y++;
			else if(c & SOUTH)
				y--;
			if(c & EAST)
				x++;
			else if(c & WEST)
				x--;
			if(pen == PENDOWN)
				TekDraw(x, y);
			else
				TekMove(x, y);
			break;

		 case CASE_PLT_VEC:	/* Plt: vector */
			unput(c);
			if(getpoint()) {
				if(pen == PENDOWN)
					TekDraw(cur_x, cur_y);
				else
					TekMove(cur_x, cur_y);
				pen = PENDOWN;
			}
			break;

		 case CASE_PT_POINT:	/* Pt: point */
			unput(c);
			if(getpoint()) {
				TekMove(cur_x, cur_y);
				TekDraw(cur_x, cur_y);
			}
			break;

		 case CASE_SPT_POINT:	/* Spt: point */
			/* ignore intensity character in c */
			if(getpoint()) {
				TekMove(cur_x, cur_y);
				TekDraw(cur_x, cur_y);
			}
			break;

		 case CASE_CR:		/* CR */
			if(nplot > 0)	/* flush line Tbuffer */
				TekFlush();
			cur_X = margin == MARGIN1 ? 0 :
			 TEKWIDTH / 2;
			Tparsestate = curstate = Talptable;
			break;

		 case CASE_ESC_STATE:	/* ESC */
			Tparsestate = Tesctable;
			break;

		 case CASE_LF:		/* LF */
			TCursorDown();
			break;

		 case CASE_SP:		/* SP */
			TCursorForward();
			break;

		 case CASE_PRINT:	/* printable character */
			if(lastfont != fontsize)
				fprintf(Tekout,"f%d\n", lastfont = fontsize);
			fprintf(Tekout,"%d %d (", cur_X, y = cur_Y);
			cp = text;
			x = sizeof(text) - 1;
			unput(c);
			while(x-- > 0 && y == cur_Y) {
				if(!isprint(c = input())) {
					unput(c);
					break;
				}
				if(c == '(' || c == ')' || c == '\\') {
					if(x <= 0) {
						unput(c);
						break;
					}
					*cp++ = '\\';
					x--;
				}
				*cp++ = c;
				TCursorForward();
			}
			*cp = 0;
			fprintf(Tekout,"%s) p\n", text);
			break;
		 case CASE_OSC:		/* do osc escape */
			while(!iscntrl(input()));	/* ignore */
			Tparsestate = curstate;
			break;
		}
}

Tinput()
{
	register int c;

	if(Tpushback > Tpushb)
		return(*--Tpushback);
again:
	if(Tbcnt-- <= 0) {
		if(((Tbcnt = read(infd, Tbptr = Tbuffer, BUFSIZ)) <= 0)
		 && nextfile())
			goto again;
		Tbcnt--;
	}
	if(!(c = (*Tbptr++ & 0177)))
		goto again;
	return(c);
}

nextfile()
{
	static int status = GOOD;
	static int first = TRUE;
	int i;
	char command[64];
again:
	if(nfiles-- > 0) {
	        if (!first)
			close(infd);
		if((infd = open(*filearg, 0)) < 0) {
			fprintf(stderr, "%s: can't open %s\n", myname,
			 *filearg++);
			status++;
			goto again;
		}
		filearg++;
		/* (MPK) does this code do anything? */
		if(!first)
			fputs("s restore /s save def\n", Tekout);
		first = FALSE;
		return(TRUE);	/* keep going */
	}
	fputs("%%Trailer\n",Tekout);
	fputs("document not { showpage } if \ns restore\n", Tekout);
	fputs("%end(plot)\n",Tekout);	/* for DVIALW compatability	*/
	if (Tekout != stdout)
	    fclose (Tekout);

	/* Print converted Postscript file */
	if (printfile) {
	    cpid = vfork ();
	    lprflags[++nflags] = printname;
	    lprflags[++nflags] = 0;
	    if (cpid == 0) {
		/*i = 0;
		while (lprflags[i] != NULL) {
		    if (i > 0)
			strcat (command, " ");
		    strcat (command, lprflags[i++]);
		    }
		fprintf (stderr,"Printing using %s\n",command); */
		execv (lprflags[0], lprflags);
		}
	    wait (0);
	    if (keeptemp == FALSE)
		unlink (printname);
	    free (printname);
	    }

	exit (status);
}

TekPage()
{
	/* if ignorefirst is TRUE, totally ignore first PAGE command */
	if (ignorefirst) {
		ignorefirst = FALSE;
		return;
	}
	pageno++;
	if(!noerase)
		fputs("document not { erasepage } if\n", Tekout);
	else {
		fputs("document not { showpage } if\n", Tekout);
		fputs("s restore\n", Tekout);
		if (!document)
			fprintf(Tekout,"%%%%Page: %d %d\n",pageno, pageno);
		fputs("initpage\n", Tekout);
	}
	cur_X = 0;
	cur_Y = TEKHOME;
}

#define	EXTRABITS	017
#define	FIVEBITS	037
#define	HIBITS		(FIVEBITS << SHIFTHI)
#define	LOBITS		(FIVEBITS << SHIFTLO)
#define	SHIFTHI		7
#define	SHIFTLO		2
#define	TWOBITS		03

getpoint()
{
	register int c, x, y, e, lo_y = 0;

	x = cur_x;
	y = cur_y;
	for( ; ; ) {
		if((c = input()) < ' ') {	/* control character */
			if (c == '\n' || c == '\r' || c == '\0')
				continue;	/* ignore CR, LF */
			unput(c);
			return(0);
		}
		if(c < '@') {	/* Hi X or Hi Y */
			if(lo_y) {	/* seen a Lo Y, so this must be Hi X */
				x &= ~HIBITS;
				x |= (c & FIVEBITS) << SHIFTHI;
				continue;
			}
			/* else Hi Y */
			y &= ~HIBITS;
			y |= (c & FIVEBITS) << SHIFTHI;
			continue;
		}
		if(c < '`') {	/* Lo X */
			x &= ~LOBITS;
			x |= (c & FIVEBITS) << SHIFTLO;
			cur_x = x;
			cur_y = y;
			return(1);	/* OK */
		}
		/* else Lo Y */
		if(lo_y) {	/* seen a Lo Y, so other must be extra bits */
			e = (y >> SHIFTLO) & EXTRABITS;
			x &= ~TWOBITS;
			x |= e & TWOBITS;
			y &= ~TWOBITS;
			y |= (e >> SHIFTLO) & TWOBITS;
		}
		y &= ~LOBITS;
		y |= (c & FIVEBITS) << SHIFTLO;
		lo_y++;
	}
}

TCursorBack()
{
	register struct Tek_Char *t;
	register int x, l;

	x = (cur_X -= (t = &TekChar[fontsize])->hsize);
	if(margin == MARGIN1 && x < 0 || margin == MARGIN2
	 && x < TEKWIDTH / 2) {
		if((l = (cur_Y + (t->vsize - 1)) / t->vsize + 1) >=
		 t->nlines) {
			margin = !margin;
			l = 0;
		}
		cur_Y = l * t->vsize;
		cur_X = (t->charsperline - 1) * t->hsize;
	}
}

TCursorForward()
{
	register struct Tek_Char *t;
	register int l;

	if((cur_X += (t = &TekChar[fontsize])->hsize) > TEKWIDTH) {
		if((l = cur_Y / t->vsize - 1) < 0) {
			margin = !margin;
			l = t->nlines - 1;
		}
		cur_Y = l * t->vsize;
		cur_X = margin == MARGIN1 ? 0 : TEKWIDTH / 2;
	}
}

TCursorUp()
{
	register struct Tek_Char *t;
	register int l;

	t = &TekChar[fontsize];
	if((l = (cur_Y + (t->vsize - 1)) / t->vsize + 1) >= t->nlines) {
		l = 0;
		if((margin = !margin) != MARGIN1) {
			if(cur_X < TEKWIDTH / 2)
				cur_X += TEKWIDTH / 2;
		} else if(cur_X >= TEKWIDTH / 2)
			cur_X -= TEKWIDTH / 2;
	}
	cur_Y = l * t->vsize;
}

TCursorDown()
{
	register struct Tek_Char *t;
	register int l;

	t = &TekChar[fontsize];
	if((l = cur_Y / t->vsize - 1) < 0) {
		l = t->nlines - 1;
		if((margin = !margin) != MARGIN1) {
			if(cur_X < TEKWIDTH / 2)
				cur_X += TEKWIDTH / 2;
		} else if(cur_X >= TEKWIDTH / 2)
			cur_X -= TEKWIDTH / 2;
	}
	cur_Y = l * t->vsize;
}

TekDraw (x, y)
int x, y;
{
	static int lastx = -1, lasty = -1, lasttype = -1;

	if((nplot == 0) || (lastx != cur_X) || (lasty != cur_Y)) {
		int flushed = 0;
		if((nplot > 0) && (linetype != SOLIDLINE)) {
			TekFlush();
			flushed = 1;
		}
		if(linetype != lasttype)
			fprintf(Tekout,"t%d\n", lasttype = linetype);
		fprintf(Tekout,"%d %d L\n", cur_X, cur_Y);
		if (flushed)
			nplot = 1;
	} else if(nplot >= MAXPATH) {
		TekFlush();
		fprintf(Tekout,"%d %d L\n", cur_X, cur_Y);
		nplot = 1;
	}
	lastx = cur_X = x;
	lasty = cur_Y = y;
	fprintf(Tekout,"%d %d l\n", x, y);
	nplot++;
}
