c*** File iolib/rcon.f
c*** December 1, 2004
c*** By Doug Mink

c  Right Ascension conversion subroutines

c	RCONR (HRA,RA) hh.mmssss to radians.
c	RCONH (RA,HRA) radians to hh.mmssss
c	RCOND (HRA,DRA) hh.mmssss to fractional degrees.
c	RCONF (DRA,HRA)  fractional degrees to hh.mmssss
c	RCONS (RA,ID,IM,RSEC) radians to hour, min, sec
c	RCONT (ID,IM,RSEC,RA) hour, min, sec to radians
c	RCONI (RA,ID,IM,IRS) radians to hour, min, int. sec
c	RCSTR (RA,NDEC,NPAD,UNITS,STRING) radians to string
c	RCSTRT (RA,NDEC,STRING) radians to string; drop zeroes (hms)
c	RCSTRC (RA,NDEC,STRING) radians to string; drop zeroes (:)
c	RCSTRP (RA,PREC,STRING) radians to string with superscript labels

c--- RCONR  Convert from RA as hh.mmssss to radians

	Subroutine RCONR (HRA,RA)

	Real*8 HRA
c			RA in hh.mmsssss
	Real*8 RA
c			RA in radians (returned)

	Include 'constants.cmb'

	Call HCONR (HRA,RA)
	RA = RA * 15.d0
	If (RA .lt. 0) RA = RA + TPI
	If (RA .gt. TPI) RA = RA - TPI

	Return
	End


c--- RCONH  Convert RA in radians to hh.mmssssss.

	Subroutine RCONH (RA,HRA)

	Real*8 RA
c			RA in radians
	Real*8 HRA
c			RA in hours (as hh.mmsssss)

	Real*8 RA1

	Include 'constants.cmb'

	If (RA .lt. 0) RA = RA + TPI
	If (RA .gt. TPI) RA = RA - TPI
	RA1 = RA / 15.d0
	Call HCONS (RA1,HRA)

	Return
	End


c--- RCOND  Convert from RA as hh.mmssss to degrees

	Subroutine RCOND (HRA,DRA)

	Real*8 HRA
c			RA in hh.mmsssss
	Real*8 DRA
c			RA in degrees (returned)

	Call HCON (HRA,DRA)
	DRA = DRA * 15.d0

	Return
	End


c--- RCONF  Convert from RA as degrees to hh.mmssss

	Subroutine RCONF (DRA,HRA)

	Real*8 DRA
c			RA in degrees (returned)
	Real*8 HRA
c			RA in hh.mmsssss

	Real*8 RA1

	Include 'constants.cmb'

	If (RCON .lt. 0.01d0) Call CONSET
	RA1 = DRA * RCON / 15.d0
	Call HCONS (RA1,HRA)

	Return
	End


c--- RCONS  Convert RA in radians to hours, minutes, and seconds.

	Subroutine RCONS (RA,IRH,IRM,RSEC)

	Real*8 RA
c			RA in radians
	Integer*4 IRH
c			Hours (returned)
	Integer*4 IRM
c			Minutes (returned)
	Real*8 RSEC
c			Seconds (returned)

	Real*8 RA1
	Integer*4 ISGN

	Include 'constants.cmb'

	If (RCON .lt. 0.01d0) Call CONSET
	If (RA .lt. 0) RA = RA + TPI
	If (RA .gt. TPI) RA = RA - TPI
	RA1 = RA / (15.d0 * RCON)
	Call HCON1 (RA1,ISGN,IRH,IRM,RSEC)

	Return
	End


c--- RCONT  Convert RA in hours, minutes, and seconds to radians.

	Subroutine RCONT (IRH,IRM,RSEC,RA)

	Integer*4 IRH
c			Hours
	Integer*4 IRM
c			Minutes
	Real*8 RSEC
c			Seconds
	Real*8 RA
c			RA in radians (returned)

	Real*8 HRA

	Include 'constants.cmb'

	If (RCON .lt. 0.01d0) Call CONSET
	HRA = Dble (IRH) + (1.d-2 * Dble (IRM)) + (1.d-4 * RSEC)
	Call RCONR (HRA, RA)
	If (RA .lt. 0) RA = RA + TPI
        If (RA .gt. TPI) RA = RA - TPI

	Return
	End


c--- RCONI  Convert RA in radians to hours, minutes, and integer seconds.

	Subroutine RCONI (RA,IRH,IRM,IRS)

	Real*8 RA
c			RA in radians
	Integer*4 IRH
c			Hours (returned)
	Integer*4 IRM
c			Minutes (returned)
	Integer*4 IRS
c			Seconds (returned)

	Real*8 RA1
	Integer*4 ISGN

	Include 'constants.cmb'

	If (RCON .lt. 0.01d0) Call CONSET
	If (RA .lt. 0) RA = RA + TPI
	If (RA .gt. TPI) RA = RA - TPI
	RA1 = RA / (15.d0 * RCON)
	Call HCON3 (RA1,ISGN,IRH,IRM,IRS)

	Return
	End


c--- RCSTR  Convert RA to string

	Subroutine RCSTR (RA,NDEC,NPAD,UNITS,STRING)

	Real*8 RA
c			RA in radians
	Integer*4 NDEC
c			Number of decimal places in seconds
	Integer*4 NPAD
c			Number of spaces between entries
	Logical UNITS
c			If true, print units
	Character*(*) STRING
c			String (returned)

	Integer*4 NFIELD
	Character*10 LFORMAT
	Character*15 TEMP
	Integer*4 IRH,IRM,IRS,I
	Real*8 RSEC

	Include 'constants.cmb'

	If (RCON .lt. 0.01d0) Call CONSET
	If (RA .lt. 0) RA = RA + TPI
	If (RA .gt. TPI) RA = RA - TPI

	If (NDEC .gt. 0) Then
	    Call RCONS (RA,IRH,IRM,RSEC)
	Else
	    Call RCONI (RA,IRH,IRM,IRS)
	Endif

c  Hours

        Write(STRING,10) IRH
10	Format(I2,'$')
	If (STRING(1:1) .eq. ' ') STRING(1:1) = '0'
	Call PADNULL (STRING)
	If (UNITS) Call CONSTR (STRING,'h')
	If (NPAD .gt. 0) Then
	    Do 15 I = 1, NPAD
15		Call CONSTR (STRING,' $')
	    Endif

c  Minutes

        Write(TEMP,20) IRM
20	Format(I2,'$')
	If (TEMP(1:1) .eq. ' ') TEMP(1:1) = '0'
	Call CONSTR (STRING,TEMP)
	If (UNITS) Call CONSTR (STRING,'m')
	If (NPAD .gt. 0) Then
	    Do 25 I = 1, NPAD
25		Call CONSTR (STRING,' $')
	    Endif

c  Seconds

	NFIELD = NDEC + 3
	If (NDEC .le. 0) Then
	    Write(TEMP,'(I2)') IRS
	Else
	    If (NFIELD .gt. 9) Then
		Write(LFORMAT,35) NFIELD,NDEC
35		Format('(F',I2,'.',I1,')')
	    Else
		Write(LFORMAT,36) NFIELD,NDEC
36		Format('(F',I1,'.',I1,')')
	    Endif
	    Write(TEMP,LFORMAT) RSEC
	    If (TEMP(NFIELD:NFIELD) .eq. ' ') Then
		Do I = NFIELD, 1, -1
		    If (TEMP(I:I) .eq. ' ') TEMP(I:I) = '0'
		    Enddo
		Endif
	Endif
	If (TEMP(1:1) .eq. ' ') TEMP(1:1) = '0'
	Call PADNULL (TEMP)
	Call CONSTR (STRING,TEMP)
	If (UNITS) Call CONSTR (STRING,'s')
	If (NPAD .gt. 0) Then
	    Do 45 I = 1, NPAD
45		Call CONSTR (STRING,' $')
	    Endif

	Return
	End


c--- RCSTRT  Convert right ascension to string, drop minutes and seconds if zero

	Subroutine RCSTRT (RA,NDEC0,STRING)

	Real*8 RA
c			RA in radians
	Integer*4 NDEC
c			Number of decimal places in seconds
c			* If negative, do not drop trailing zeroes
	Character*(*) STRING
c			String (returned)
	Integer*4 LT
	Integer*4 NPAD, NDEC0
	Logical UNITS
	Character*1 NULL

	NULL = Char (0)
	If (NDEC0 .lt. 0) Then
	    NDEC = -NDEC0
	Else
	    NDEC = NDEC0
	Endif

	NPAD = 0
	UNITS = .true.
	Call RCSTR (RA,NDEC,NPAD,UNITS,STRING)

	If (NDEC0 .ge. 0) Then

c  Remove trailing zeroes in seconds

	    Call VLEN (STRING,LT)
	    If (STRING(LT-4:LT) .eq. '.000s') Then
		STRING(LT-4:LT-4) = 's'
		STRING(LT-3:LT-3) = NULL
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Elseif (STRING(LT-3:LT) .eq. '.00s') Then
		STRING(LT-3:LT-3) = 's'
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Elseif (STRING(LT-2:LT) .eq. '.0s') Then
		STRING(LT-2:LT-2) = 's'
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Endif

c  Remove seconds if zero

	    Call VLEN (STRING,LT)
	    If (STRING(LT-2:LT) .eq. '00s') Then
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
		Endif

c  Remove minutes if zero

	    Call VLEN (STRING,LT)
	    If (STRING(LT-2:LT) .eq. '00m') Then
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
		Endif
	    Endif

	Return
	End


c--- RCSTRC  Convert right ascension to colon-separated string,
c	     dropping minutes and seconds if zero

	Subroutine RCSTRC (RA,NDEC0,STRING)

	Real*8 RA
c			RA in radians
	Integer*4 NDEC0
c			Number of decimal places in seconds
c			* If negative, do not drop trailing zeroes
	Character*(*) STRING
c			String (returned)
	Integer*4 LT
	Integer*4 NPAD, I, NDEC
	Logical UNITS
	Character*1 NULL

	NULL = Char (0)
	If (NDEC0 .lt. 0) Then
	    NDEC = -NDEC0
	Else
	    NDEC = NDEC0
	Endif

	NPAD = 1
	UNITS = .false.
	Call RCSTR (RA,NDEC,NPAD,UNITS,STRING)

	If (NDEC0 .ge. 0) Then

c  Remove trailing zeroes in seconds

	    Call VLEN (STRING,LT)
	    If (STRING(LT-4:LT) .eq. '.000 ') Then
		STRING(LT-4:LT-4) = NULL
		STRING(LT-3:LT-3) = NULL
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Elseif (STRING(LT-3:LT) .eq. '.00 ') Then
		STRING(LT-3:LT-3) = NULL
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Elseif (STRING(LT-2:LT) .eq. '.0 ') Then
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
	    Endif

c  Remove seconds if zero

	    Call VLEN (STRING,LT)
	    If (STRING(LT-2:LT) .eq. ' 00') Then
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
		Endif

c  Remove minutes if zero

	    Call VLEN (STRING,LT)
	    If (STRING(LT-2:LT) .eq. ' 00') Then
		STRING(LT-2:LT-2) = NULL
		STRING(LT-1:LT-1) = NULL
		STRING(LT:LT) = NULL
		Endif
	    Endif

	Call VLEN (STRING,LT)
	If (STRING(LT:LT) .eq. ' ') STRING(LT:LT) = NULL
	Call VLEN (STRING,LT)
	Do I = 2, LT
	    If (STRING(I:I) .eq. ' ') STRING(I:I) = ':'
	    Enddo

	Return
	End


c--- RCSTRH  Convert right ascension to vigesimal string; drop trailing zeroes

	Subroutine RCSTRH (RA,NDEC,STRING)

	Real*8 RA
c			Fractional degrees or hours
	Integer*4 NDEC
c			Number of decimal places in seconds
c			* If negative, do not drop trailing zeroes
	Character*(*) STRING
c			String (returned)
	Integer*4 IC, I, LSTR

	Call RCSTRC (RA,NDEC,STRING)

c  Change first : to .

	IC = Index (STRING,':')
	STRING(IC:IC) = '.'

c  Delete second :

	IC = Index (STRING,':')
	Call VLEN (STRING,LSTR)
	STRING(IC:IC) = STRING(IC+1:IC+1)
	STRING(IC+1:IC+1) = STRING(IC+2:IC+2)

c  If there are decimal seconds, delete .

	If (STRING(IC+2:IC+2) .eq. '.') Then
	    Do I = IC+3, LSTR
		STRING(I-2:I-2) = STRING(I:I)
		Enddo
	    STRING(LSTR-1:LSTR-1) = '$'
	Else
	    STRING(IC+2:IC+2) = '$'
	Endif
	Call PADNULL (STRING)

	Return
	End


	Subroutine RCSTRP (RA,PREC,STRING)

c--- Write RA in degrees to minimum-length hms string with superscripts

	Real*8 RA
c				Right ascension in degrees
	Integer*4 PREC
c				0= Drop both minutes and seconds if 0
c				1= Drop seconds, keep minutes if 0
c				2= Keep seconds even if 0
	Character*(*) STRING
c				Output string, terminated by "$" (returned)

	Real*8 RS
	Integer*4 IRH,IRM,LTEMP,LSTR,LH,LM,LS,LZ,L,IRS
	Character TEMP*60,HLAB*8,MLAB*8,SLAB*8
	Include 'plx1.cmb'

c  Convert right ascension to hours, minutes, and seconds

	Call RCONS (RA,IRH,IRM,RS)

c  Set labels

	If (DRAW) then
	    HLAB = '|sup{h}'
	    LH = 7
	    MLAB = '|sup{m}'
	    LM = 7
	    SLAB = '|sup{s}'
	    LS = 7
	    LTEMP = 24
	Else
	    HLAB = 'h '
	    LH = 2
	    MLAB = 'm '
	    LM = 2
	    SLAB = 's'
	    LS = 1
	    LTEMP = 14
	Endif

c  Set up string and fill in blanks with zeroes

	IRS = Idnint (RS * 1000.d0)
	If ((PREC .eq. 2) .or. (IRS .gt. 0)) Then
	    Write(TEMP,10) IRH,HLAB(1:LH),IRM,MLAB(1:LM),RS
10	    Format(I2,A,I2,A,F6.3)
	    If (TEMP(LTEMP:LTEMP) .eq. '0') LTEMP = LTEMP - 1
	    If (TEMP(LTEMP:LTEMP) .eq. '0') LTEMP = LTEMP - 1
	    If (TEMP(LTEMP:LTEMP) .eq. '0') LTEMP = LTEMP - 2
	    TEMP(LTEMP+1:LTEMP+1) = '$'
	    Call PADNULL (TEMP)
	    Call CONSTR (TEMP,SLAB(1:LS))
	    LZ = LH+LM+5
	    If (TEMP(LZ:LZ) .eq. ' ') TEMP(LZ:LZ) = '0'
	    LZ = LH+3
	    If (TEMP(LZ:LZ) .eq. ' ') TEMP(LZ:LZ) = '0'

c  If seconds are zero, don't bother with them

	Elseif ((PREC .eq. 1) .or. (IRM .gt. 0)) Then
	    Write(TEMP,20) IRH,HLAB(1:LH),IRM,MLAB(1:LM)
20	    Format(I2,A,I2,A,'$')
	    LZ = LH+3
	    If (TEMP(LZ:LZ) .eq. ' ') TEMP(LZ:LZ) = '0'

	Else
	    Write(TEMP,30) IRH,HLAB(1:LH)
30	    Format(I2,A,'$')
	Endif

c  Move temporary string to return string

	LSTR = Len (STRING)
	Call ZFILLC (STRING,LSTR)
	Call VLEN (TEMP,LTEMP)
	L = LTEMP
	If (LSTR .lt. LTEMP) L = LSTR
	If (TEMP(1:1) .ne. ' ') Then
	    STRING(1:L) = TEMP(1:L)
	Elseif (TEMP(2:2) .ne. ' ') Then
	    STRING(1:L-1) = TEMP(2:L)
	Elseif (TEMP(3:3) .ne. ' ') Then
	    STRING(1:L-2) = TEMP(3:L)
	Else
	    STRING(1:L-3) = TEMP(4:L)
	Endif

	Return
	End

c Oct 16 1992	Declare undeclared variables; fix bug in RCSTR
c Nov 18 1992	Fix bug in RCONH

c Jan 26 1993	Add RCONF for fractional degrees to hh.mmssss

c Feb  7 1995	Add RCSTRT to drop unused part of strings
c Feb 11 1995	Add RCSTRC to drop unused part of strings with colons
c Dec  8 1995	Add RCSTRH to mkae vigesimal string

c Jan 16 1996	If NDEC is negative, do not drop trailing zeroes
c Feb  1 1996	Handle bug dealing with negative ndec0
c Sep 12 1996	Keep RA between 0 and two pi
c Sep 13 1996	Add RCSTRP for plotted string
c Oct  1 1996	Add leading 0 to hours in RCSTR if only one digit

c Apr  9 1998	Add RCONT to translate table entry to radians
c Aug 20 1998	Declare undeclared variables

c Dec  1 2004	Include only from current directory
