/*** File iolib/svec.c
 *** March 10, 1999
 *** By Doug Mink, Center for Astrophysics

 *** Vector operations

	svadd2_ (vec1,vec2, vecout, lvec)	Add two I*2 vectors
	svadd4_ (vec1, vec2, vecout, lvec)	Add two I*4 vectors

	svadd2c_ (vec1, num, vecout, lvec)	Add I*2 constant to I*2 vector
	svadd4c_ (vec1, num, vecout, lvec)	Add I*4 constant to I*4 vector
	svaddrc_ (vec1, num, vecout, lvec)	Add R*4 constant to R*4 vector
	svadddc_ (vec1, num, vecout, lvec)	Add R*8 constant to R*8 vector

	svsub2_ (vec1, vec2, vecout, lvec)	Subtract two I*2 vectors
	svsub4_ (vec1, vec2, vecout, lvec)	Subtract two I*4 vectors

	svsub2c_ (vec1, num, vecout, lvec)	Sub. constant from I*2 vector
	svsub4c_ (vec1, num, vecout, lvec)	Sub. constant from I*4 vector

	svset1c_ (num,vecout,offset,lvec)	Set I*1 vector to constant
	svset2c_ (num,vecout,offset,lvec)	Set I*2 vector to constant
	svset4c_ (num,vecout,offset,lvec)	Set I*4 vector to constant
	svsetrc_ (num,vecout,offset,lvec)	Set R*4 vector to constant
	svsetdc_ (num,vecout,offset,lvec)	Set R*8 vector to constant

	svinv1_ (vecout,offset,lvec)		Invert I*1 vector
	svinv2_ (vecout,offset,lvec)		Invert I*2 vector
	svinv4_ (vecout,offset,lvec)		Invert I*4 vector

	svmul4c_ (vec1, num, vecout, lvec)	Mult. I*4 vector by I*4 constant
	svmulrc_ (vec1, num, vecout, lvec)	Mult. R*4 vector by R*4 constant
	svmuldc_ (vec1, num, vecout, lvec)	Mult. R*8 vector by R*8 constant

	svec24_ (vec2,vec4,lvec)		Move I*2 to I*4 vector
	svec42_ (vec4,vec2,lvec)		Move I*4 to I*2 vector
	svec21_ (vec2,vec1,lvec)		Move I*2 to I*1 vector
	svec12_ (vec1,vec2,lvec)		Move I*1 to I*2 vector
	svec1r_ (vec1,vec2,lvec)		Move I*1 to R*4 vector
	svec2r_ (vec2,vecr,lvec)		Move I*2 to R*4 vector
	svec4r_ (vec4,vecr,lvec)		Move I*4 to R*4 vector
	svecdr_ (veca,vecb,lvec)		Move R*8 to R*4 vector
	svecrr_ (veca,vecb,lvec)		Move R*4 to R*4 vector

 */


/*  Add vectors of 2-byte integers */

svadd2_ (vec1,vec2, vecout, lvec)

short *vec1;	/* Address of input Integer*2 vector */
short *vec2;	/* Address of second Integer*2 vector */
short *vecout;	/* Address of output Integer*2 vector */
int *lvec;	/* Address of number of elements in vector */

{
short *vo, *v1, *v2, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	v2 = vec2;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ + *v2++; }
	
	return;
}


/*  Add vectors of 4-byte integers */

svadd4_ (vec1, vec2, vecout, lvec)

int *vec1;	/* Address of input Integer*4 vector */
int *vec2;	/* Address of second Integer*4 vector */
int *vecout;	/* Address of output Integer*4 vector */
int *lvec;	/* Address of number of elements in vector */

{
int *vo, *v1, *v2, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	v2 = vec2;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ + *v2++; }
	
	return;
}


/*  Add constant to vector of 2-byte integers */

svadd2c_ (vec1, num, vecout, lvec)

short *vec1;	/* Address of input Integer*2 vector */
int *num;	/* Address of Integer*2 constant */
short *vecout;	/* Address of output Integer*2 vector */
int *lvec;	/* Address of number of elements in vector */

{
register short *vo, *v1, n, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	n = (short) *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ + n; }

	return;
}


/*  Add constant to vector of 4-byte integers */

svadd4c_ (vec1, num, vecout, lvec)

int *vec1;	/* Address of output Integer*4 vector */
int *num;	/* Address of Integer*4 constant */
int *vecout;	/* Address of output Integer*4 vector */
int *lvec;	/* Address of number of elements in vector */

{
register int *vo, *v1, n, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	n = *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ + n; }
	
	return;
}


/*  Add constant to vector of 4-byte floating point numbers */

svaddrc_ (vec1, num, vecout, lvec)

float *vec1;	/* Address of output Integer*4 vector */
float *num;	/* Address of Integer*4 constant */
float *vecout;	/* Address of output Integer*4 vector */
int *lvec;	/* Address of number of elements in vector */

{
register float *vo, *v1, n, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	n = *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ + n; }
	
	return;
}


/*  Add constant to vector of 8-byte floating point numbers */

svadddc_ (vec1, num, vecout, lvec)

double *vec1;	/* Address of output Integer*4 vector */
double *num;	/* Address of Integer*4 constant */
double *vecout;	/* Address of output Integer*4 vector */
int *lvec;	/* Address of number of elements in vector */

{
register double *vo, *v1, n, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	n = *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ + n; }
	
	return;
}


/*  Set vector of 1-byte integers to constant */

svset1c_ (num, vecout, offset, lvec)

int *num;	/* Address of Integer*2 constant */
unsigned char *vecout;	/* Address of output Integer*2 vector */
int *offset;	/* Address of number of elements offset in vector */
int *lvec;	/* Address of number of elements in vector */

{
register unsigned char *vo, n, *veclast;

	vo = vecout - 1 + *offset;
	n = (char) *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = n; }

	return;
}


/*  Set vector of 2-byte integers to constant */

svset2c_ (num, vecout, offset, lvec)

int *num;	/* Address of Integer*2 constant */
short *vecout;	/* Address of output Integer*2 vector */
int *offset;	/* Address of number of elements offset in vector */
int *lvec;	/* Address of number of elements in vector */

{
register short *vo, n, *veclast;

	vo = vecout - 1 + *offset;
	n = (short) *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = n; }

	return;
}


/*  Set vector of 4-byte integers to constant */

svset4c_ (num, vecout, offset, lvec)

int *num;	/* Address of Integer*4 constant */
int *vecout;	/* Address of output Integer*4 vector */
int *offset;	/* Address of number of elements offset in vector */
int *lvec;	/* Address of number of elements in vector */

{
register int *vo, n, *veclast;

	vo = vecout - 1 + *offset;
	n = *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = n; }

	return;
}


/*  Set vector of 4-byte floating point numbers to a constant */

svsetrc_ (num, vecout, offset, lvec)

float *num;	/* Address of Integer*4 constant */
float *vecout;	/* Address of output Integer*4 vector */
int *offset;	/* Address of number of elements offset in vector */
int *lvec;	/* Address of number of elements in vector */

{
register float *vo, n, *veclast;

	vo = vecout - 1 + *offset;
	n = *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = n; }

	return;
}


/*  Set vector of 8-byte floating point numbers to a constant */

svsetdc_ (num, vecout, offset, lvec)

double *num;	/* Address of Integer*4 constant */
double *vecout;	/* Address of output Integer*4 vector */
int *offset;	/* Address of number of elements offset in vector */
int *lvec;	/* Address of number of elements in vector */

{
register double *vo, n, *veclast;

	vo = vecout - 1 + *offset;
	n = *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = n; }

	return;
}


/*  Invert vector of 1-byte integers */

svinv1_ (vecout, offset, lvec)

int *offset;		/* Address of offset to invert */
unsigned char *vecout;	/* Address of output Integer*1 vector */
int *lvec;		/* Address of number of elements in vector */

{
register unsigned char *vo, *veclast;

	vo = vecout - 1 + *offset;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = ~*vo; }
	
	return;
}


/*  Invert vector of 2-byte integers */

svinv2_ (vecout, offset, lvec)

int *offset;	/* Address of offset to invert */
short *vecout;	/* Address of output Integer*2 vector */
int *lvec;	/* Address of number of elements in vector */

{
register short *vo, *veclast;

	vo = vecout - 1 + *offset;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = ~*vo; }
	
	return;
}


/*  Invert vector of 4-byte integers */

svinv4_ (vecout, offset, lvec)

int *offset;	/* Address of offset to invert */
int *vecout;	/* Address of output Integer*4 vector */
int *lvec;	/* Address of number of elements in vector */

{
register int *vo, *veclast;

	vo = vecout - 1 + *offset;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = ~*vo; }
	
	return;
}


/*   Move vector of 2-byte integers into 4-byte integers */

svec24_ (vec2,vec4,lvec)

short *vec2;	/* Address of input Integer*2 vector */
int *vec4;	/* Address of output Integer*4 vector */
int *lvec;	/* Address of number of integers to convert */

{
int *v4, *veclast;
short *v2;

	v2 = vec2;
	v4 = vec4 - 1;
	veclast = v4 + *lvec;

	while (v4++ < veclast) { *v4 = (int) *v2++; }

	return;
}


/*   Move vector of 1-byte integers into 4-byte floating point numbers */

svec1r_ (vec1,vecr,lvec)

char *vec1;	/* Address of input Integer*1 vector */
float *vecr;	/* Address of output Real*4 vector */
int *lvec;	/* Address of number of integers to convert */

{
float *vr, *veclast;
char *v1;

	v1 = vec1;
	vr = vecr - 1;
	veclast = vr + *lvec;

	while (vr++ < veclast) { *vr = (float) *v1++; }

	return;
}


/*   Move vector of 2-byte integers into 4-byte floating point numbers */

svec2r_ (vec2,vecr,lvec)

short *vec2;	/* Address of input Integer*2 vector */
float *vecr;	/* Address of output Real*4 vector */
int *lvec;	/* Address of number of integers to convert */

{
float *vr, *veclast;
short *v2;

	v2 = vec2;
	vr = vecr - 1;
	veclast = vr + *lvec;

	while (vr++ < veclast) { *vr = (float) *v2++; }

	return;
}


/*   Move vector of 4-byte integers into 4-byte floating point numbers */

svec4r_ (vec4,vecr,lvec)

int *vec4;	/* Address of input Integer*4 vector */
float *vecr;	/* Address of output Real*4 vector */
int *lvec;	/* Address of number of integers to convert */

{
float *vr, *veclast;
int *v4;

	v4 = vec4;
	vr = vecr - 1;
	veclast = vr + *lvec;

	while (vr++ < veclast) { *vr = (float) *v4++; }

	return;
}


/*   Move one vector of 4-byte floating point numbers to another */

svecrr_ (veca,vecb,lvec)

float *veca;	/* Address of input Real*4 vector */
float *vecb;	/* Address of output Real*4 vector */
int *lvec;	/* Address of number of integers to convert */

{
float *va, *vb, *veclast;

	va = veca;
	vb = vecb - 1;
	veclast = vb + *lvec;

	while (vb++ < veclast) { *vb = *va++; }

	return;
}


/*   Move a vector of 8-byte f.p. numbers to one of 4-byte f.p. numbers */

svecdr_ (vecd,vecr,lvec)

double *vecd;	/* Address of input Real*8 vector */
float *vecr;	/* Address of output Real*4 vector */
int *lvec;	/* Address of number of integers to convert */

{
double *vd;
float *vr, *veclast;

	vd = vecd;
	vr = vecr - 1;
	veclast = vr + *lvec;

	while (vr++ < veclast) { *vr = *vd++; }

	return;
}


/*   Move vector of 4-byte integers into 2-byte integers */

svec42_ (vec4,vec2,lvec)

int *vec4;	/* Address of input Integer*4 vector */
short *vec2;	/* Address of output Integer*2 vector */
int *lvec;	/* Address of number of integers to convert */

{
int v4s,*v4, *veclast;
short *v2, v2i;

	v4 = vec4 - 1;
	v2 = vec2;
	veclast = v4 + *lvec;

	while (v4++ < veclast) {
	    v4s = *v4;
	    if (v4s > 32767) v4s = 32767;
	    else if (v4s < -32768) v4s = -32768;
	    *v2++ = (short) v4s;
	    }

	return;
}


/*   Move vector of 2-byte integers into 1-byte integers */

svec21_ (vec2,vec1,lvec)

short *vec2;		/* Address of input Integer*2 vector */
unsigned char *vec1;	/* Address of output Integer*1 vector */
int *lvec;		/* Address of number of integers to convert */

{
register short v2c,*v2, v2min, v2max, *veclast;
register unsigned char *v1, v1c, *v21;

	v2 = vec2 - 1;
	v1 = vec1;
	veclast = v2 + *lvec;
	v2max = (short) 255;
	v2min = (short) 0;

	while (v2++ < veclast) {
	    v2c = *v2;
	    v21 = (unsigned char*) v2;
	    v21++;
	    v1c = *v21;
	    if (v2c > v2max) v2c = v2max;
	    *v1++ = v1c;
	    }

	return;
}


/*   Move vector of 1-byte integers into 2-byte integers */

svec12_ (vec1,vec2,lvec)

unsigned char *vec1;	/* Address of output Integer*1 vector */
short *vec2;		/* Address of input Integer*2 vector */
int *lvec;		/* Address of number of integers to convert */

{
register short v2c, *v2;
register unsigned char *v1, *veclast, v1c;

	v1 = vec1 - 1;
	v2 = vec2;
	veclast = v1 + *lvec;

	while (v1++ < veclast) {
	    v1c = *v1;
	    v2c = (short) v1c;
	    *v2++ = v2c;
	    }

	return;
}


/*  Multiply vector of 4-byte integers by constant */

svmul4c_ (vec1, num, vecout, lvec)

int *vec1;	/* Address of output Integer*4 vector */
int *num;	/* Address of Integer*4 constant */
int *vecout;	/* Address of output Integer*4 vector */
int *lvec;	/* Address of number of elements in vector */

{
register int *vo, *v1, n, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	n = *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ * n; }

	return;
}


/*  Multiply vector of 4-byte floating point numbers by constant */

svmulrc_ (vec1, num, vecout, lvec)

float *vec1;	/* Address of output Integer*4 vector */
float *num;	/* Address of Integer*4 constant */
float *vecout;	/* Address of output Integer*4 vector */
int *lvec;	/* Address of number of elements in vector */

{
register float *vo, *v1, n, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	n = *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ * n; }

	return;
}


/*  Multiply vector of 8-byte floating point numbers by constant */

svmuldc_ (vec1, num, vecout, lvec)

double *vec1;	/* Address of output Integer*4 vector */
double *num;	/* Address of Integer*4 constant */
double *vecout;	/* Address of output Integer*4 vector */
int *lvec;	/* Address of number of elements in vector */

{
register double *vo, *v1, n, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	n = *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ * n; }

	return;
}


/*  Subtract vectors of 2-byte integers */

svsub2_ (vec1, vec2, vecout, lvec)

short *vec1;	/* Address of input Integer*2 vector */
short *vec2;	/* Address of second Integer*2 vector */
short *vecout;	/* Address of output Integer*2 vector */
int *lvec;	/* Address of number of elements in vector */

{
register short *vo, *v1, *v2, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	v2 = vec2;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ - *v2++; }

	return;
}


/*  Subtract constant from vector of 2-byte integers */

svsub2c_ (vec1, num, vecout, lvec)

short *vec1;	/* Address of input Integer*2 vector */
int *num;	/* Address of Integer*2 constant */
short *vecout;	/* Address of output Integer*2 vector */
int *lvec;	/* Address of number of elements in vector */

{
register short *vo, *v1, n, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	n = (short) *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ - n; }
	
	return;
}


/*  Subtract vectors of 4-byte integers */

svsub4_ (vec1, vec2, vecout, lvec)

int *vec1;	/* Address of input Integer*4 vector */
int *vec2;	/* Address of second Integer*4 vector */
int *vecout;	/* Address of output Integer*4 vector */
int *lvec;	/* Address of number of elements in vector */

{
register int *vo, *v1, *v2, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	v2 = vec2;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ - *v2++; }

	return;
}


/*  Subtract constant from vector of 4-byte integers */

svsub4c_ (vec1, num, vecout, lvec)

int *vec1;	/* Address of output Integer*4 vector */
int *num;	/* Address of Integer*4 constant */
int *vecout;	/* Address of output Integer*4 vector */
int *lvec;	/* Address of number of elements in vector */

{
register int *vo, *v1, n, *veclast;

	vo = vecout - 1;
	v1 = vec1;
	n = *num;
	veclast = vo + *lvec;

	while (vo++ < veclast) { *vo = *v1++ - n; }
	
	return;
}


/*   divide Integer*4 vector by Integer*2 vector returning Integer*2 vector */
/*   *if numerator is zero, result is set to zero */

svdiv42_ (vec1, vec2, vecout, lvec)

int *vec1;	/* Address of input Integer*2 vector */
short *vec2;	/* Address of second Integer*4 vector */
short *vecout;	/* Address of output Integer*2 vector */
int *lvec;	/* Address of number of elements in vector */

{
short *vo, *v2, *veclast;
int *v1, v;

	vo = vecout - 1;
	v1 = vec1;
	v2 = vec2;
	veclast = vo + *lvec;

	while (vo++ < veclast) {
	    v = (int) *v2;
	    if (v != 0)
		*vo = (short) (*v1 / v);
	    else
		*vo = 0;
	    v1++;
	    v2++;
	    }
	
	return;
}
/* Sep 15 1994	New file
 *
 * Jun 18 1996	Add floating point operations
 *
 * Mar 10 1999	Add integer to floating conversions
 */
