c*** File iolib/tcon.f
c*** January 20, 1988
c*** By Doug Mink

c  TCON (DAY,TIME,YSEC)
c	Convert from day of year and time as hh.mmsss to seconds of year

c  TCON1 (YSEC,DAY,TIME)
c	Convert seconds of year to day of year and time as hh.mmssss

c  TCONS (YSEC,STRING)
c	Convert seconds of year to day of year/hours:minutes:seconds string

c  TCON2 (YSEC,DAY,HOURS,MINUTES,SECONDS)
c	Convert seconds of year to days, hours, minutes, seconds

c  TCON3 (YSEC,DAY,HOURS,MINUTES,SECONDS)
c	Convert seconds of year to days, hours, minutes, integer seconds


c  TCON-- Convert from day of year and time as hh.mmsss to seconds of year

	Subroutine TCON (DAY,TIME,YSEC)

	Integer*4 DAY
c			Day of year
	Real*8 TIME
c			Time as hh.mmssss
	Real*8 YSEC
c			Seconds of year (returned)

	Integer*4 HOURS
c			Hours of day
	Integer*4 MINUTES
c			Minutes of hour

	Real*8 XNUM,XHR,XMN,XS

	XNUM = TIME

	HOURS = Idint (TIME + 1.d-9)
	XHR = Dble (HOURS)
	XNUM = (XNUM - XHR) * 1.d2

	MINUTES = Idint (XNUM + 1.d-9)
	XMN = Dble (MINUTES)
	XS = (XNUM - XMN) * 1.d2

	YSEC = 3.6d3 * XHR + 6.d1 * XMN + XS
	If (DAY .gt. 0) YSEC = YSEC + 8.64d4 * (DAY - 1)

	Return
	End


c  TCON1-- Convert seconds of year to day of year and time as hh.mmssss

	Subroutine TCON1 (YSEC,DAY,TIME)

	Real*8 YSEC
c			Seconds of year
	Integer*4 DAY
c			Day of year (returned)
	Real*8 TIME
c			Time as hh.mmssss (returned)

	Integer*4 HOURS
c			Hours of day (returned)
	Integer*4 MINUTES
c			Minutes of hour (returned)
	Real*8 SECONDS
c			Seconds of minute (returned)

	Call TCON2 (YSEC,DAY,HOURS,MINUTES,SECONDS)

	TIME = Dble (HOURS) + 1.d-2 * Dble (MINUTES) + 1.d-4 * SECONDS

	Return
	End


c  TCONS-- Convert seconds of year to day of year/hours:minutes:seconds string

	Subroutine TCONS (YSEC,STRING)

	Real*8 YSEC
c			Seconds of year
	Character*(*) STRING
c			Date/time string (returned)

	Integer*4 DAY
c			Day of year
	Integer*4 HOURS
c			Hours of day (returned)
	Integer*4 MINUTES
c			Minutes of hour (returned)
	Real*8 SECONDS
c			Seconds of minute (returned)
	Integer*4 LS
c			Integer seconds

	Call TCON2 (YSEC,DAY,HOURS,MINUTES,SECONDS)

	LS = Idnint (SECONDS)
	If (SECONDS .lt. 1.d-3) Then
	    Write(STRING,10) DAY,HOURS,MINUTES
10	    Format(I3,'/',I2,':',I2)
	    If (STRING(5:5) .eq. ' ') STRING(5:5) = '0'
	    If (STRING(8:8) .eq. ' ') STRING(8:8) = '0'
	Elseif (Dabs (SECONDS - Dble (LS)) .lt. 1.d-3) Then
	    Write(STRING,20) DAY,HOURS,MINUTES,LS
20	    Format(I3,'/',I2,':',I2,':',I2)
	    If (STRING(5:5) .eq. ' ') STRING(5:5) = '0'
	    If (STRING(8:8) .eq. ' ') STRING(8:8) = '0'
	    If (STRING(11:11) .eq. ' ') STRING(11:11) = '0'
	Else
30	    Format(I3,'/',I2,':',I2,':',F6.3)
	    Write(STRING,30) DAY,HOURS,MINUTES,SECONDS
	    If (STRING(5:5) .eq. ' ') STRING(5:5) = '0'
	    If (STRING(8:8) .eq. ' ') STRING(8:8) = '0'
	    If (STRING(11:11) .eq. ' ') STRING(11:11) = '0'
	Endif

c  Pad string with zeroes

	Call PADNULL (STRING)

	Return
	End

c  TCON2-- Convert seconds of year to days, hours, minutes, seconds

	Subroutine TCON2 (YSEC,DAY,HOURS,MINUTES,SECONDS)

	Real*8 YSEC
c			Seconds of year
	Integer*4 DAY
c			Day of year (returned)
	Integer*4 HOURS
c			Hours of day (returned)
	Integer*4 MINUTES
c			Minutes of hour (returned)
	Real*8 SECONDS
c			Seconds of minute (returned)

	Real*8 YSR

	DAY = Idint (1.d0 + YSEC / 8.64d4)
	YSR = YSEC - (8.64d4 * Dble (DAY - 1))

	SECONDS = Dmod (YSR, 6.d1)
	If (SECONDS .gt. 5.999999d1) SECONDS = 0.d0
	YSR = (YSR - SECONDS) / 6.d1

	MINUTES = Idnint (Dmod (YSR, 6.d1))
	If (MINUTES .eq. 60) MINUTES = 0
	YSR = (YSR - Dble (MINUTES)) / 6.d1

	HOURS = Idnint (YSR)

	Return
	End


c  TCON3-- Convert seconds of year to day of year, hours, minutes, and
c          rounded seconds

	Subroutine TCON3 (YSEC,DAY,HOURS,MINUTES,SECONDS)

	Real*8 YSEC
c			Seconds of year
	Integer*4 DAY
c			Day of year (returned)
	Integer*4 HOURS
c			Hours of day (returned)
	Integer*4 MINUTES
c			Minutes of hour (returned)
	Integer*4 SECONDS
c			Seconds of minute (returned)

	Real*8 YSR

	YSR = Dnint (YSEC)

	DAY = Idint (1.d0 + YSEC / 8.64d4)
	YSR = YSR - (8.64d4 * Dble (DAY - 1))

	SECONDS = Idnint (Dmod (YSR, 60.d0))
	YSR = (YSR - Dble (SECONDS)) / 60.d0

	MINUTES = Idnint (Dmod (YSR, 60.d0))
	YSR = (YSR - Dble (MINUTES)) / 60.d0

	HOURS = Idnint (YSR)

	Return
	End

c Jul  1 1987	Add string conversion

c Jan 20 1988	Fix string conversion
