c*** File iolib/tokens.f
c*** October 17, 1995
c*** By Doug Mink, Center for Astrophysics

c-- SETOKEN -- tokenize a string for easy decoding

	Subroutine SETOKEN (STRING, NTMAX, CWHITE, NTOK, TOK1, TOK2)

	Character*(*) STRING
c				Character string to tokenize
	Integer*4 NTMAX
c				Maximum number of tokens
	Character*(*) CWHITE
c				Additional whitespace characters
c				* If = TAB, disallow spaces and commas
	Integer*4 NTOK
c				Number of tokens found (returned)
	Integer*4 TOK1(*)
c				Indices of first characters of tokens (returned)
	Integer*4 TOK2(*)
c				Indices of last characters of tokens (returned)

	Character*1 SQUOTE,DQUOTE,JCH
	Character*20 WHITE
	Character*32 TOKEN
	Integer*4 I0,ITYPE,I,IQ,J,LSTR,NWHITE,NADDW,L1,L2

	SQUOTE = Char (39)
	DQUOTE = Char (34)
	Call VLEN (STRING, LSTR)
	Call VLEN (CWHITE, NADDW)

c  If CHITE is TAB, allow only tabs and NULLs as separators

	If (CWHITE(1:3) .eq. 'TAB') Then
	    WHITE(1:1) = Char(9)
	    WHITE(2:2) = Char (0)
	    NWHITE = 2

c  Otherwise, allow spaces, tabs, commas, NULLs, and CWHITE

	Else
	    NWHITE = 3 + NADDW
	    WHITE(1:1) = ' '
	    WHITE(2:2) = Char (9)
	    WHITE(3:3) = ','
	    WHITE(4:4) = Char (0)
	    If (NWHITE .gt. 20) NWHITE = 20
	    If (NADDW .gt. 0) Then
		I = 1
		Do 10 J = 4, NWHITE
		    WHITE(J:J) = CWHITE(I:I)
10		    I = I + 1
		Endif
	Endif
	Call PADNULL (WHITE)

	NTOK = 0
	IQ = -1
	L1 = 0
	L2 = 0

c  Process string one character at a time

	Do 100 I0 = 1, LSTR
	    If (I0 .le. IQ) Go to 100
	    JCH = STRING(I0:I0)

c	Handle quoted strings

	    If (JCH .eq. SQUOTE) Then
		IQ = Index (STRING(I0+1:LSTR),SQUOTE)
	    Elseif (JCH .eq. DQUOTE) Then
		IQ = Index (STRING(I0+1:LSTR),DQUOTE)
	    Else
		IQ = 0
	    Endif
	    If (IQ .gt. 0) Then
		NTOK = NTOK + 1
		If (NTOK .gt. NTMAX) Return
		TOK1(NTOK) = I0 + 1
		TOK2(NTOK) = I0 + IQ - 1
		TOKEN = STRING(TOK1(NTOK):TOK2(NTOK))
		L1 = TOK2(NTOK) + 1
		L2 = 0
		Go to 100
		Endif

c	Search for unquoted tokens
  
	    ITYPE = Index (WHITE,JCH)

c	If one of the additional whitespace characters, pass as a separate token

	    If (ITYPE .gt. 3) Then

c	Terminate token before whitespace

		If (L2 .gt. 0) Then
		    NTOK = NTOK + 1
		    If (NTOK .gt. NTMAX) Return
		    TOK1(NTOK) = L1
		    L2 = I0 - 1
		    TOK2(NTOK) = L2
		    TOKEN = STRING(TOK1(NTOK):TOK2(NTOK))
		    Endif

c	Make whitespace character next token; start new one

		NTOK = NTOK + 1
		If (NTOK .gt. NTMAX) Return
		TOK1(NTOK) = I0
		TOK2(NTOK) = I0
		If (TOK2(NTOK) .gt. LSTR) TOK2(NTOK) = LSTR
		TOKEN = STRING(TOK1(NTOK):TOK2(NTOK))
		L1 = I0 + 1
		L2 = 0
	    
	    Elseif (ITYPE .gt. 0) Then

c	Ignore leading whitespace

		If (L2 .le. 0) Then
		    L1 = I0 + 1

c	Terminate token before whitespace; start new one

		Else
		    L2 = I0 - 1
		    NTOK = NTOK + 1
		    If (NTOK .gt. NTMAX) Return
		    TOK1(NTOK) = L1
		    TOK2(NTOK) = L2
		    If (TOK2(NTOK) .gt. LSTR) TOK2(NTOK) = LSTR
		    TOKEN = STRING(TOK1(NTOK):TOK2(NTOK))
		    L1 = I0 + 1
		    L2 = 0
		Endif
	    Else
		If (L1 .le. 0) L1 = 1
		L2 = I0
	    Endif

100	    Continue

c  Add token terminated by end of line

	If (L2 .gt. 0) Then
	    NTOK = NTOK + 1
	    If (NTOK .gt. NTMAX) Return
	    TOK1(NTOK) = L1
	    TOK2(NTOK) = L2
	    If (TOK2(NTOK) .gt. LSTR) TOK2(NTOK) = LSTR
	    TOKEN = STRING(TOK1(NTOK):TOK2(NTOK))
	    Endif

	Return
	End


c NEXTOKEN -- Get next token from tokenized string

	Subroutine NEXTOKEN (STRING, NTOK, TOK1, TOK2, ITOK, TOKEN, LTOK)

	Character*(*) STRING
c				Tokenized character string
	Integer*4 NTOK
c				Number of tokens found
	Integer*4 TOK1(*)
c				Indices of first characters of tokens
	Integer*4 TOK2(*)
c				Indices of last characters of tokens
	Integer*4 ITOK
c				Token sequence number of token (incremented)
	Character*(*) TOKEN
c				Token (returned)space characters
	Integer*4 LTOK
c				Length of token string (returned)
	Integer*4 I1, I2

	ITOK = ITOK + 1
	I1 = TOK1(ITOK)
	I2 = TOK2(ITOK)
	TOKEN = STRING(I1:I2)
	Call PADNULL (TOKEN)
	LTOK = TOK2(ITOK) - TOK1(ITOK) + 1

	Return
	End


c GETOKEN -- Get specified token from tokenized string

	Subroutine GETOKEN (STRING, NTOK, TOK1, TOK2, ITOK, TOKEN, LTOK)

	Character*(*) STRING
c				Tokenized character string
	Integer*4 NTOK
c				Number of tokens found
	Integer*4 TOK1(*)
c				Indices of first characters of tokens
	Integer*4 TOK2(*)
c				Indices of last characters of tokens
	Integer*4 ITOK
c				Token sequence number of token
c				* If <0, get whole string after token -ITOK
c				* If =0, get whole string
	Character*(*) TOKEN
c				Token (returned)space characters
	Integer*4 LTOK
c				Length of token string (returned)
	Integer*4 I1, I2

	If (ITOK .gt. 0) Then
	    I1 = TOK1(ITOK)
	    I2 = TOK2(ITOK)
	Elseif (ITOK .lt. 0) Then
	    I1 = TOK1(1-ITOK)
	    I2 = TOK2(NTOK)
	Else
	    I1 = TOK1(1)
	    I2 = TOK2(NTOK)
	Endif
	TOKEN = STRING(I1:I2)
	Call PADNULL (TOKEN)
	Call VLEN (TOKEN,LTOK)

	Return
	End

c Jul 19 1993	New version of tokenize to allow arbitrary whitespace characters
c Aug 11 1993	Fix bug so setoken sets tok2(ntok) correctly
c Aug 17 1993	Drop unused variable isp

c Jun 28 1994	Add new subroutine getoken
c Dec  6 1994	Add options to getoken if token number is negative or 0

c Oct 17 1995	Add option to setoken to deal with tab tables
