/*** File iolib/vaxcon.c
 *** March 24, 2008
 *** By Doug Mink
 */

#include <stdio.h>
#include <stdlib.h>

static double xexp[257];
static double r4norm = 16777216.0;
static double r8norm = 4294967296.0;

vaxr4_ (buff,xnum)

/*-- Convert VAX Real*4 floating point number to local floating point format

/*	Call VAXR4 (XVAX,XNUM)

	Real*4 XVAX		4-byte VAX floating point number
	Real*4 XNUM		4-byte floating point number (returned)
*/

unsigned char buff[4];
float *xnum;
{
	double mantis;		/* Mantissa for floating point */
	double dsgn;		/* Sign value */
	int iexpt; 		/* Exponent for floating point */
	int inum;		/* Value of mantissa */
	int lbuff[4];
	int i;

/*  Compute all possible exponent values to avoid exponentiation */

	if (xexp[129] == 0) {
	    xexp[129] = 1.0;
	    for (i=1; i < 32; i++) {
		xexp[129+i] = xexp[129+i-1] * 2.0;
		xexp[129-i] = 1.0 / xexp[129+i];
		}
	    }
	for (i = 0; i < 4; i++) lbuff[i] = (int) buff[i];

	/*  Set sign  */

	if (lbuff[0] > 127) {
	    dsgn = -1.0; }
	else {
	    dsgn = 1.0; }

	/*  Exponent (excess 128)  */

	iexpt = 2 * (lbuff[0] & 127) + (lbuff[1] & 128 / 128);
	lbuff[1] = lbuff[1] & 127;

	inum = (((lbuff[1] << 8) + lbuff[2]) << 8) + lbuff[3];

/*  If input is hard zero, return it unchanged  */

	if (inum == 0) *xnum = 0.0;

/*  Otherwise translate it  */

	else {

	/*  Mantissa  */

	    mantis = ((double) inum) / r4norm;

/*  Compute number */

	    if (iexpt > 127) {
		fprintf (stderr,"VAXR4:  Exponent overflow = %d \n",iexpt);
		*xnum = 0.;  }
	    else if (iexpt < -127)
		*xnum= 0.;
	    else
		*xnum = dsgn * mantis * xexp[iexpt];
	    }

	return;
}


vaxr8_ (buff,dnum)

/*-- Convert VAX Real*8 floating point number to local floating point format

/*	Call VAXR8 (DVAX,DNUM)

	Real*8 DVAX		8-byte VAX floating point number
	Real*8 DNUM		8-byte floating point number (returned)
*/

unsigned char buff[8];
double *dnum;
{
	double mantis;		/* Mantissa for floating point */
	double mantis2;		/* Mantissa for floating point */
	double dsgn;		/* Sign value */
	int iexpt; 		/* Exponent for floating point */
	int inum;		/* Value of mantissa */
	int inum2 ;		/* Value of mantissa */
	int lbuff[8];
	int i;

/*  Compute all possible exponent values to avoid exponentiation */


/*  Compute all possible exponent values to avoid exponentiation */

	if (xexp[129] == 0) {
	    xexp[129] = 1.0;
	    for (i=1; i < 32; i++) {
		xexp[129+i] = xexp[129+i-1] * 2.0;
		xexp[129-i] = 1.0 / xexp[129+i];
		}
	    }

	for (i=0; i<8; i++) lbuff[i] = (int) buff[i];

	/*  Set sign  */

	if (lbuff[0] > 127) {
	    dsgn = -1.0; }
	else {
	    dsgn = 1.0; }

	/*  Exponent (excess 128)  */

	iexpt = 2 * (lbuff[0] & 127) + (lbuff[1] & 128 / 128);
	lbuff[1] = lbuff[1] & 127;

	inum = (((lbuff[1] << 8) + lbuff[2]) << 8) + lbuff[3];

/*  If input is hard zero, return it unchanged  */

	if (inum == 0) *dnum = 0.0;

/*  Otherwise translate it  */

	else {

	/*  Mantissa  */

	    mantis = (double) inum;
	    inum2 = (((lbuff[4] << 8) + lbuff[5]) << 8) + lbuff[6];
	    mantis2 = ((double) inum2 * 256.0) + (double) lbuff[7];
	    mantis = (mantis + (mantis2 / r8norm)) / r4norm;

/*  Compute number */

	    if (iexpt > 127) {
		fprintf (stderr,"VAXR8:  Exponent overflow = %d \n",iexpt);
		*dnum = 0.;
		}

	    else if (iexpt < 127) {
		/* fprintf (stderr,"VAXR8:  Exponent underflow = %d \n",iexpt); */
		*dnum= 0.;
		}

	    else {
		*dnum = dsgn * mantis * xexp[iexpt];
		}

	    }

	return;
}

/* Jan 11 1991	New subroutines
 *
 * Mar 24 2008	Add std includes for printf
 */
