c*** File iolib/vcon.f
c*** March 14, 2000
c*** By Doug Mink

c  VCON (DATE,TIME,TSEC)
c	Convert date as yyyy.ddmm and time as hh.mmsss to seconds since 1950.0

c  VCON1 (TSEC,DATE,TIME)
c	Convert seconds since 1950.0 to date as yyyy.ddmm and time as hh.mmsss

c  VCONS (TSEC,STRING)
c	Convert seconds since 1950.0 to year day month hours:minutes:seconds string

c  VCON2 (TSEC,IYR,IMON,IDAY,IHR,IMN,SEC)
c	Convert seconds since 1950.0 to year month day hours minutes seconds .1ms

c  VCONY (YEAR,DATE,TIME,TSEC)
c	Convert year, day, and time as hh.mmsss to seconds since 1950.0

c  VCONJ (DATE,TIME,DJ)
c	Convert date as yyyy.ddmm and time as hh.mmsss to Julian Day


c  VCON-- Convert from date as yyyy.mmdd and time as hh.mmsss to seconds since 1950.0

	Subroutine VCON (DATE,TIME,TSEC)

	Real*8 DATE
c			Date as yyyy.mmdd
c			    yyyy = calendar year (e.g. 1973)
c			    mm = calendar month (e.g. 04 = april)
c			    dd = calendar day (e.g. 15)
	Real*8 TIME
c			Time as hh.mmssxxxx
c		       *if TIME<0, it is time as -(fraction of a day)
c			    hh = hour of day (0 .le. hh .le. 23)
c			    nn = minutes (0 .le. nn .le. 59)
c			    ss = seconds (0 .le. ss .le. 59)
c			  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)
	Real*8 TSEC
c			Seconds past 1950.0 (returned)

	Real*8 DH,DM,DD
	Integer*4 IY,IM,ID

	If (DATE .lt. 0.0301d0) Return

c  Calculate the number of full years, months, and days already
c  elapsed since 0h, March 1, -1 (up to most recent midnight).

c  Convert time of day to elapsed seconds

	If (TIME .lt. 0) Then
	    TSEC = TIME * -86400.d0
	Else
	    DH = Dint (TIME + 1.0d-10)
	    DM = Dint (((TIME-DH) * 1.0d2) + 1.0d-10)
	    TSEC = (TIME * 1.0d4) - (DH * 1.0d4) - (DM * 1.0d2)
	    TSEC = Dint (TSEC * 1.d5 + 1.d-4) / 1.d5
	    TSEC = TSEC + (DM * 60.0d0) + (DH * 3600.0d0)
	Endif

c  Calculate the number of full months elapsed since
c  the current or most recent March

	IY = Idint (DATE + 1.0d-10)
	IM = Idint (((DATE - Dble (IY)) * 1.0d4) + 1.0d-8)
	ID = Mod (IM,100)
	IM = (IM / 100) + 9
	If (IM .lt. 12) IY = IY - 1
	IM = Mod (IM, 12)
	ID = ID - 1

c  Starting with March as month 0 and ending with the following
c  February as month 11, the calculation of the number of days
c  per month reduces to a simple formula. the following statement
c  determines the number of whole days elapsed since 3/1/-1 and then
c  subtracts the 712163 days between then and 1/1/1950.  it converts
c  the result to seconds and adds the accumulated seconds above.

	ID = ID + ((IM+1+IM/6+IM/11)/2 * 31) + ((IM-IM/6-IM/11)/2 * 30)
     2	+ (IY / 4) - (IY / 100) + (IY / 400)
	DD = Dble (ID) + (365.d0 * Dble (IY)) - 712163.d0
	TSEC = TSEC + (DD * 86400.d0)

	Return
	End


c  VCON1-- Convert seconds since 1950.0 to date as yyyy.mmdd and time as hh.mmssss

	Subroutine VCON1 (TSEC,DATE,TIME)

	Real*8 TSEC
c			Seconds past 1950.0
	Real*8 DATE
c			Date as yyyy.mmdd
c			    yyyy = calendar year (e.g. 1973)
c			    mm = calendar month (e.g. 04 = april)
c			    dd = calendar day (e.g. 15)
	Real*8 TIME
c			Time as hh.mmssxxxx
c		       *if TIME<0, it is time as -(fraction of a day)
c			    hh = hour of day (0 .le. hh .le. 23)
c			    nn = minutes (0 .le. nn .le. 59)
c			    ss = seconds (0 .le. ss .le. 59)
c			  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)

	Integer*4 IYR,IMON,IDAY,IHR,IMN
	Real*8 SEC

	Call VCON2 (TSEC,IYR,IMON,IDAY,IHR,IMN,SEC)

c  Convert date to yyyy.mmdd

	DATE = Dble (IYR) + 1.d-2 * Dble (IMON) + 1.d-4 * Dble (IDAY)

c  Convert time to hh.mmssssss

	TIME = Dble (IHR) + 1.d-2 * Dble (IMN) + 1.d-4 * SEC

	Return
	End


c  VCONS-- Convert seconds since 1950.0 to year month day hours:minutes:seconds string

	Subroutine VCONS (TSEC,STRING)

	Real*8 TSEC
c				Seconds since 1/1/1950 0:00
	Character*(*) STRING
c				String to receive formatted date

	Real*8 SEC
	Integer IHR,IMN,IMON,IDAY,IYR
	Character*4 DMON(12)

c  Names of the months of the year

	DMON(1) = 'Jan '
	DMON(2) = 'Feb '
	DMON(3) = 'Mar '
	DMON(4) = 'Apr '
	DMON(5) = 'May '
	DMON(6) = 'Jun '
	DMON(7) = 'Jul '
	DMON(8) = 'Aug '
	DMON(9) = 'Sep '
	DMON(10) = 'Oct '
	DMON(11) = 'Nov '
	DMON(12) = 'Dec '

	If (Len (STRING) .lt. 24) Then
	    Write(*,*) 'VCONS:  STRING too short to receive date and time'
	    Return
	    Endif

c  Calculate time of day (hours, minutes, seconds)

	Call VCON2 (TSEC,IYR,IMON,IDAY,IHR,IMN,SEC)
	If (SEC .lt. 0.d0) SEC = 0.0d0

c  Print time of day

c	With month and date

	Write(STRING,100) IYR,DMON(IMON),IDAY,IHR,IMN,SEC
100	Format(I4,1x,A4,I2,I3,':',I2,':',F6.3)
	If (STRING(16:16) .eq. ' ') STRING(16:16) = '0'
	If (STRING(19:19) .eq. ' ') STRING(19:19) = '0'

	Call PADNULL (STRING)

	Return
	End


c  VCONP-- Convert seconds since 1950.0 to
c	   day month year hours:minutes:seconds string

	Subroutine VCONP (TSEC,STRING)

	Real*8 TSEC
c				Seconds since 1/1/1950 0:00
	Character*(*) STRING
c				String to receive formatted date

	Real*8 SEC
	Integer IHR,IMN,IMON,IDAY,IYR
	Integer LMON,LDAY,LHR,NMON(12)
	Character*12 DMON(12),SMON
	Character*8 SSEC
	Character*3 SMN
	Character*2 SDAY,SHR

c  Names of the months of the year

	DMON(1) = 'January'
	DMON(2) = 'February'
	DMON(3) = 'March'
	DMON(4) = 'April'
	DMON(5) = 'May'
	DMON(6) = 'June'
	DMON(7) = 'July'
	DMON(8) = 'August'
	DMON(9) = 'September'
	DMON(10) = 'October'
	DMON(11) = 'November'
	DMON(12) = 'December'
	NMON(1) = 31
	NMON(2) = 30
	NMON(3) = 31
	NMON(4) = 30
	NMON(5) = 31
	NMON(6) = 30
	NMON(7) = 31
	NMON(8) = 31
	NMON(9) = 30
	NMON(10) = 31
	NMON(11) = 30
	NMON(12) = 31

	If (Len (STRING) .lt. 24) Then
	    Write(*,*) 'VCONS:  STRING to short to receive date and time'
	    Return
	    Endif

c  Calculate time of day (hours, minutes, seconds)

	Call VCON2 (TSEC,IYR,IMON,IDAY,IHR,IMN,SEC)
	If (SEC .lt. 0.d0) SEC = 0.0d0
	If (SEC .gt. 59.999) Then
	    SEC = 0.d0
	    IMN = IMN + 1
	    If (IMN .ge. 60) Then
		IMN = IMN - 60
		IHR = IHR + 1
		If ((IHR.gt.24).or.((IHR.gt.23).and.(IMN.gt.0))) Then
		    IHR = IHR - 24
		    IDAY = IDAY + 1
		    If (IDAY .gt. NMON(IMON)) Then
			IDAY = 1
			IMON = IMON + 1
			If (IMON .gt. 12) Then
			    IYR = IYR + 1
			    IMON = 1
			    Endif
			Endif
		    Endif
		Endif
	    Endif

	SMON = DMON(IMON)
	Call PADNULL (SMON)
	Call VLEN (SMON,LMON)

	If (IDAY .lt. 10) Then
	    Write(SDAY,'(I1)') IDAY
	    LDAY = 1
	Else
	    Write(SDAY,'(I2)') IDAY
	    LDAY = 2
	Endif
	If (IHR .lt. 10) Then
	    Write(SHR,'(I1)') IHR
	    LHR = 1
	Else
	    Write(SHR,'(I2)') IHR
	    LHR = 2
	Endif
	Write(SMN,'(I3)') IMN
	SMN(1:1) = ':'
	If (SMN(2:2) .eq. ' ') SMN(2:2) = '0'

c  Print time of day with month and date

	Write(STRING,100) SDAY(1:LDAY),SMON(1:LMON),IYR,SHR(1:LHR),SMN
100	Format(A,1x,A,I5,1x,A,A3)
	Call PADNULL (STRING)

c  Add seconds only if non-zero

	If (SEC .gt. 0.) Then
	    Write(SSEC,'(F7.3)') SEC
	    SSEC(1:1) = ':'
	    If (SSEC(2:2) .eq. ' ') SSEC(2:2) = '0'
	    Call PADNULL (SSEC)
	    Call CONSTR (STRING,SSEC)
	    Endif

	Call CONSTR (STRING,' UT')

	Return
	End


c  VCON2-- Convert seconds since 1950.0 to year month day hours minutes seconds .1ms

	Subroutine VCON2 (TSEC,IYR,IMON,IDAY,IHR,IMN,SEC)

	Real*8 TSEC
c			Seconds since 1/1/1950 0:00
	Integer*4 IYR
c			Year
	Integer*4 IMON
c			Month
	Integer*4 IDAY
c			Day
	Integer*4 IHR
c			Hours
	Integer*4 IMN
c			Minutes
	Real*8 SEC
c			Seconds

	Real*8 T,DAYS
	Integer*4 ISEC,IHMS,NC,NC4,NLY,NY,M,IM

c  Calculate time of day (hours, minutes, seconds, .1 msec)
	T = Dint ((TSEC + 61530883200.d0) * 1.d4 + .5d0)
	IHR = Idint (Dmod (T/36.d6,24.d0))
	IMN = Idint (Dmod (T/60.d4,60.d0))
	If (TSEC .ge. 0) Then
	    IHMS = Idint (Dmod (TSEC+1.d-6,1.d0) * 1.d4)
	    ISEC = Idint (Dmod (TSEC+1.d-6,60.d0))
	Else
	    IHMS = Idint (Dmod (TSEC-1.d-6,1.d0) * 1.d4)
	    ISEC = Idint (Dmod (TSEC-1.d-6,60.d0))
	Endif
	SEC = Dble (ISEC) + 1.d-4 * Dble (IHMS)

c  Calculate number of days since 0 hr 0/0/0000
	DAYS = Dint ((T / 864.d6) + 1.0d-6)

c  Number of leap centuries (400 years)
	NC4 = Idint ((DAYS / 146097.0d0) + 1.0d-5)

c  Number of centuries since last /400
	DAYS = DAYS - (146097.d0 * Dble (NC4))
	NC = Idint ((DAYS / 36524.d0) + 1.0d-4)
	If (NC .gt. 3) NC = 3

c  Calculate number of leap years since last century
	DAYS = DAYS - (36524.d0 * NC)
	NLY = Idint ((DAYS / 1461.d0) + 1.0d-10)

c  Calculate number of years since last leap year
	DAYS = DAYS - (1461.d0 * Dble (NLY))
	NY = Idint ((DAYS / 365.d0) + 1.d-8)
	If (NY .gt. 3) NY = 3

c  Calculate day of month
	DAYS = DAYS - (365.d0 * Dble (NY))
	If (DAYS .lt. 0) Then
	    M = 0
	    IDAY = 29
	Else
	    IDAY = Idint (DAYS + 1.0d-8) + 1
	    Do M = 1,12
		IM = Mod ((M + ((M - 1) / 5)), 2)
		If (IDAY-1 .lt. IM+30) Go to 20
		IDAY = IDAY - IM - 30
		Enddo
	Endif

c  Month
20	IMON  =  Mod (M+1,12) + 1

c  Year
	IYR = NC4*400 + NC*100 + NLY*4 + NY + M/11

	Return
	End


c VCONY-- Convert year, day, and time as hh.mmsss to seconds since 1950.0

	Subroutine VCONY (YEAR,DAY,TIME,TSEC)

	Integer*4 YEAR
c			Year
	Integer*4 DAY
c			Day of the year (1 - 365 or 366)
	Real*8 TIME
c			Time as hh.mmssxxxx
c		       *if TIME<0, it is time as -(fraction of a day)
c			    hh = hour of day (0 .le. hh .le. 23)
c			    nn = minutes (0 .le. nn .le. 59)
c			    ss = seconds (0 .le. ss .le. 59)
c			  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)
	Real*8 TSEC
c			Seconds past 1950.0

	Real*8 DATE0

	DATE0 = Dble (YEAR) + 0.0101d0
	Call VCON (DATE0,TIME,TSEC)
	TSEC = TSEC + 8.64d4 * Dble (DAY - 1)

	Return
	End


c VCONJ-- Convert from date as yyyy.mmdd and time as hh.mmsss to Julian Day
c	  If DATE > 2000000, assume it to be JD and set DATE, TIME, and DJ

	Subroutine VCONJ (DATE,TIME,DJ)

	Real*8 DATE
c			Date as yyyy.mmdd
c			    yyyy = calendar year (e.g. 1973)
c			    mm = calendar month (e.g. 04 = april)
c			    dd = calendar day (e.g. 15)
	Real*8 TIME
c			Time as hh.mmssxxxx
c		       *if TIME<0, it is time as -(fraction of a day)
c			    hh = hour of day (0 .le. hh .le. 23)
c			    nn = minutes (0 .le. nn .le. 59)
c			    ss = seconds (0 .le. ss .le. 59)
c			  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)
	Real*8 DJ
c			Julian Day (returned)

	Real*8 TSEC
c			Seconds past 1950.0

	If (DATE .lt. 0.0301d0) Return

	If (DATE .gt. 2.d6) Then
	    DJ = DATE
	    TSEC = (DJ - 2433282.5d0) * 86400.d0
	    Call VCON1 (TSEC,DATE,TIME)
	Else
	    Call VCON (DATE,TIME,TSEC)
	    DJ = 2433282.5d0 + (TSEC / 86400.d0)
	Endif

	Return
	End

c Aug 15 1989	Add packed string conversion VCONP for labels

c Aug 13 1990	Add Julian Day conversion

c Nov  8 1996	Fix VCON2 for negative times
c Nov 22 1996	Fix VCONP to round 59.999 seconds correctly

c Mar 14 2000	Fix VCON2 to deal with 2000-0229 correctly
