c*** File svdvec.f
c*** By Mike Kurtz, modified by Doug Mink
c*** August 20, 2004

	Program svd_vec

c  Take an array of sky spectra and decompose it into an orthogonal
c  basis using Singular Value Decomposition

	Character*230400 header
	Integer*4 larg, narg, iarg, ier, Iargc
	Integer*4 ia, ib, m, n, nb, i, ier, ipass, lsfile, lvfile, idev
	Real*8 a(4096,4096), b(4096,4096), s(4096), wk(8192)
        Real*4 vector(4096,4096)
	Character*256 argument, specfile, vecfile, sfile, string
	Character*72 history
	Logical specread, vecread, verbose, printhelp, passnext

	ia = 4096
	m = 4096
	n = 4096
	ib = 4096
	nb = 4096
	specread = .false.
	vecread = .false.
	ipass = 0
	verbose = .false.
	printhelp = .false.
	passnext = .false.

c  Loop through argument list
	narg = Iargc ()
	Do iarg = 1, narg
	    Call Getarg (iarg,argument)
	    Call PADNULL (argument)
	    Call VLEN (argument, larg)
	    If ((argument(1:4) .eq. 'help').or. (argument(1:5) .eq. '-help')) Then
		printhelp = .true.

	    Elseif (argument(1:2) .eq. '-v') Then
		verbose = .true.

	    Elseif (argument(1:2) .eq. '-p') Then
		passnext = .true.
	    Elseif (passnext) Then
		Call ASCBINI4 (larg, argument, ipass, ier)
		passnext = .false.

	    Elseif (.not. specread) Then
		specfile = argument
		specread = .true.
		lsfile = larg

	    Elseif (.not. vecread) Then
		vecfile = argument
		vecread = .true.
		lvfile = larg

	    Elseif (vecread .and. specread) Then
		Call ASCBINI4 (larg, argument, ipass, ier)

	    Endif
	    Enddo

200	If (printhelp .or. ((.not. specread) .and. (.not. vecread))) Then
	    Write(*,*) 'SVDVEC decomposes a set of spectra into their eigenvectors.'
	    Write(*,*) '       The first argument is the name of a 2-dimensional file'

	    Write(*,*) '       of stacked spectra, with dispersion along the X axis.'
	    Write(*,*) '       The second argument is the name of the file of eigenvectors'
	    Write(*,*) '       to be returned in the same format.'
	    Write(*,*) '       An optional third argument gives the number of the current pass.'


	Else
	    if (verbose) Then
		Write(*,*) 'SVDVEC writing eigenvectors of ',specfile(1:lsfile),
     1			    ' to ', vecfile(1:lvfile)
		Endif

c  Read stacked spectra with bad lines zeroed
	    Call READ_SKY (specfile, a, vector, header)
	    If (verbose) Then
		Write (string, *) "FITS spectra read"
		Endif

c  Fill diagonal of eigenvector array
	    Do i = 1, m
		b(i,i) = 1.d0
		End do

c  Decompose spectra into eigenvectors
	    Call LSVDF (a,ia,m,n,b,ib,nb,s,wk,ier)

c  Add input filename to header
	    Write(history,*) 'SVD decomposition of ',specfile(1:lsfile)
	    Call HPUTC (header, "HISTORY", history)

c  Write eigenvectors to output file
	    Call WRITE_FIT (vecfile, ipass, b, vector, header)

	    sfile = vecfile
	    Call CONCAT (sfile,256,'.s',2)
	    Call FOPENW (idev, sfile)
	    Do i = 1, nb
		Write (string, 300) i,s(i)
300		Format(I4,G14.6,'$')
		Call PADNULL (string)
		Call SFPUTL (idev, string)
		Enddo
	    Call SFCLOSE (idev, ier)
	Endif

	stop
	end


c  READ_SKY -- Read FITS file containing the array of sky spectra to decompose

	Subroutine READ_SKY (filename, a, spectra, header)

	Character filename(*)
c				Input FITS file
	Real*8 a(4096,4096)
c				
	Real*4 spectra(4096,4096)
c				Spectra as read
	Character*(*) header
c				FITS header
	Integer*4 nr, nbmax, nbytes, i, j, npix, nspec, nba, iout
	Logical nonzero

	nr = 0
	nbmax = 4096 * 4096 * 4
	nba = 4096 * 4096 * 8
	nbytes = 0

c  Read FITS file of up to 4096 4096-pixel stacked spectra
	Call FITSREAD (filename,header,nr,spectra,nbmax,nbytes)
	Call HGETI4 (header, "NAXIS1", npix)
	Call HGETI4 (header, "NAXIS2", nspec)

c  Zero working array
	Call ZFILL (a, nba)

c  Convert Real*4 input spectra to Real*8 for computation
	iout = 0
	Do i = 1, nspec

c  Check to see if this spectrum has been zeroed
	    nonzero = .false.
	    Do j = 1, npix
		if (spectra(j,i) .ne. 0.0) Then
		    nonzero = .true.
		    Endif
		Enddo

c  If it hasn't, copy it
	    If (nonzero) Then
		iout = iout + 1
		Do j = 1, npix
		    a(j,iout) = spectra(j,i)
		    Enddo
		Endif
	    Enddo

	Return

	End


c  WRITE_FIT -- write FITS file containing eigenvector fits

	Subroutine  WRITE_FIT (filename, ipass, b, vectors, header)

	Character*(*) filename
	Integer*4 ipass, nspec, npix
	Real*8 b(4096,4096)
	Real*4 vectors(4096,4096)
	Character*(*) header
	Character*72 object, history
	Integer*4 nw, i, j

	Call HGETI4 (header, "NAXIS1", npix)
	Call HGETI4 (header, "NAXIS2", nspec)

c  Convert Real*8 input coefficient vectors to Real*4 for output
	Do i = 1, npix
	    Do j = 1, nspec
		vectors(i,j) = b(j,i)
		Enddo
	    Enddo

	nw = 0
	Write(object, 10) ipass
10	Format ('Sky SVD pass ',I1,' eigenvectors$')
	Call PADNULL (object)
	Call HPUTS (header, 'OBJECT', object)
	Write(history, 20) ipass
20	Format ('SVDVEC pass ',I1,' eigenvectors$')
	Call PADNULL (history)
	Call HPUTC (header, 'HISTORY', history)
	Call FITSWRITE (filename, header, vectors, nw)

	Return
	End


c   routine name   - lsvdb
c
c-----------------------------------------------------------------------
c
c   purpose             - singular value decomposition of a bidiagonal
c                           matrix.
c
c   usage               - Call lsvdb (d,e,n,v,iv,nrv,c,ic,ncc,ier)
c
c   arguments    d      - vector of length n. (input/output)
c                         on input, d contains the diagonal elements
c                           of the bidiagonal matrix b. d(i) = b(i,i),
c                           i = 1,...,n.
c                         on output, d contains the n (nonnegative)
c                           singular values of b in nonincreasing
c                           order.
c                e      - vector of length n. (input/output)
c                         on input, e contains the superdiagonal
c                           elements of b. e(1) is arbitrary,
c                           e(i) = b(i-1,i), i=2,...,n.
c                         on output, the contents of e are modified
c                           by the subroutine.
c                n      - order of the matrix b. (input)
c                v      - nrv by n matrix. (input/output)
c                           if nrv.le.0, v is not used. otherwise,
c                           v is replaced by the nrv by n product
c                           matrix v*vb. see remarks.
c                iv     - row dimension of matrix v exactly as
c                           specified in the dimension statement in the
c                           calling program. (input)
c                nrv    - number of rows of v. (input)
c                c      - n by ncc matrix. (input/output)
c                           if ncc.le.0 c is not used. otherwise, c
c                           is replaced by the n by ncc product
c                           matrix ub**(t) * c. see remarks.
c                ic     - row dimension of matrix c exactly as
c                           specified in the dimension statement in the
c                           calling program. (input)
c                ncc    - number of columns in c. (input)
c                ier    - error parameter. (input)
c                         warning error
c                           ier = 33 indicates that matrix b is not full
c                             rank or very ill-conditioned. small
c                             singular values may not be very accurate.
c                         terminal error
c                           ier = 129 indicates that convergence was
c                             not attained after 10*n qr sweeps.
c                             (convergence usually occurs in about
c                             2*n sweeps).
c
c
c   remarks      lsvdb computes the singular value decomposition of
c                an n by n bidiagonal matrix
c                     b = ub * s * vb**(t)    where
c                ub and vb are n by n orthogonal matrices and
c                s is diagonal.
c                if arguments v and c are n by n identity matrices,
c                on exit they are replaced by vb and ub**t,
c                respectively.
c
c-----------------------------------------------------------------------
c
	subroutine lsvdb (d,e,n,v,iv,nrv,c,ic,ncc,ier)
c
c                                  specifications for arguments
	integer            n,iv,nrv,ic,ncc,ier
	real*8             d(n),e(n),v(iv,1),c(ic,1)
c                                  specifications for local variables
	integer            i,ii,j,k,kk,l,ll,lp1,nqrs,n10
	logical            wntv,havers,fail
	real*8             dnorm,zero,one,two,cs,f,g,h,sn,t,x,y,z
	data               zero/0.d0/,one/1.d0/,two/2.d0/
c                                  first executable statement
	ier = 0
	if (n .le. 0) Return
	n10 = 10*n
	wntv = nrv.gt.0
	havers = ncc.gt.0
	fail = .false.
	nqrs = 0
	e(1) = zero
	dnorm = zero
	Do j = 1,n
	    dnorm = max(abs(d(j))+abs(e(j)),dnorm)
	    Enddo
	do 100 kk = 1,n
	    k = n+1-kk

c test for splitting or rank deficiencies first make test for
c last diagonal term, d(k), being small.
   10    if (k .ne. 1) Then
	    t = dnorm + d(k)
	    if (t .eq. dnorm) Then

c Since d(k) is small we will make a special pass to transform e(k) to zero.
		cs = zero
		sn = -one
		Do ii = 2,k
		    i = k+1-ii
		    f = -sn*e(i+1)
		    e(i+1) = cs*e(i+1)
		    t = d(i)
		    Call lsvg1 (t,f,cs,sn,d(i))
c                                  transformation constructed to zero
c                                    position (i,k).
		    if (wntv) Then
			Do j = 1,nrv
			    Call lsvg2 (cs,sn,v(j,i),v(j,k))
			    Enddo
			Endif
c
c                                  accumulate rt. transformations in v.
		    Enddo
		Endif
	    Endif

c  The matrix is now bidiagonal, and of lower order since e(k) .eq. zero
	Do ll = 1,k
	    l = k+1-ll
	    t = dnorm+e(l)
	    if (t.eq.dnorm) go to 50
	    t = dnorm+d(l-1)
	    if (t.eq.dnorm) go to 35
	    Enddo
c  this loop can't complete since e(1) = zero.
	Go to 50

c cancellation of e(l), l.gt.1.
   35   cs = zero
	sn = -one
	Do i = l,k
	    f = -sn*e(i)
	    e(i) = cs*e(i)
	    t = dnorm+f
	    if (t.eq.dnorm) go to 50
	    t = d(i)
	    Call lsvg1 (t,f,cs,sn,d(i))
	    If (havers) Then
		Do j = 1,ncc
		    Call lsvg2 (cs,sn,c(i,j),c(l-1,j))
		    Enddo
		Endif
	    Enddo

c Test for convergence
   50	z = d(k)
	if (l.eq.k) go to 85
c                                  shift from bottom 2 by 2 minor of
c                                    b**(t)*b.
	x = d(l)
	y = d(k-1)
	g = e(k-1)
	h = e(k)
	f = ((y-z)*(y+z)+(g-h)*(g+h))/(two*h*y)
	g = sqrt(one+f**2)
	if (f .ge. zero) Then
	    t = f + g
	Else
	    t = f - g
	Endif
	f = ((x-z)*(x+z)+h*(y/t-h))/x
c  next qr sweep
	cs = one
	sn = one
	lp1 = l+1
	Do i = lp1,k
	    g = e(i)
	    y = d(i)
	    h = sn*g
	    g = cs*g
	    Call lsvg1 (f,h,cs,sn,e(i-1))
	    f = x*cs+g*sn
	    g = -x*sn+g*cs
	    h = y*sn
	    y = y*cs

c accumulate rotations (from the right) in v
	    If (wntv) Then
		do j = 1,nrv
		    Call lsvg2 (cs,sn,v(j,i-1),v(j,i))
		    Enddo
		Endif
	    Call lsvg1 (f,h,cs,sn,d(i-1))
	    f = cs*g+sn*y
	    x = -sn*g+cs*y
	    If (havers) Then
		Do j = 1,ncc
		    Call lsvg2 (cs,sn,c(i-1,j),c(i,j))
		    Enddo
		Endif
	    Enddo

c  apply rotations from the left to right hand sides in c
	e(l) = zero
	e(k) = f
	d(k) = x
	nqrs = nqrs + 1
	if (nqrs .le. n10) go to 10
c                                  return to test for splitting.
	fail = .true.

c cutoff for convergence failure. nqrs will be 2*n usually.
   85	if (z .lt. zero) Then
	    d(k) = -z
	    if (wntv) Then
		Do j = 1,nrv
		    v(j,k) = -v(j,k)
		    Enddo
		Endif
	    Endif

c  convergence. d(k) is made nonnegative
  100	continue

	if (n .eq. 1) go to 140
	Do i = 2,n
	    if (d(i) .gt. d(i-1)) go to 110
	    Enddo

c  Every singular value is in order
	go to 140

  110	Do i = 2, n
	    t = d(i-1)
	    k = i-1
	    Do j = i,n
		If (t .lt. d(j)) Then
		    t = d(j)
		    k = j
		    Endif
		Enddo

	    If (k .ne. i-1) Then
		d(k) = d(i-1)
		d(i-1) = t
		If (havers) Then
		    Do j = 1,ncc
			t = c(i-1,j)
			c(i-1,j) = c(k,j)
			c(k,j) = t
			Enddo
		    Endif
		If (wntv) Then
		    Do j = 1,nrv
			t = v(j,i-1)
			v(j,i-1) = v(j,k)
			v(j,k) = t
			Enddo
		    Endif
		Endif
	    Enddo
c  end of ordering algorithm.

  140	ier = 129

c  Check for possible rank deficiency
	If (.not. fail) Then
	    ier = 33
	    t = 0.d0
	    if (d(1) .ne. zero) t = d(n) / d(1)
	    f = 100.d0 + t
	    if (f .ne. 100.d0) ier = 0
	    Endif

	return
	end


c LSVDF -- singular value decomposition of a real*8 matrix

	subroutine lsvdf  (a,ia,m,n,b,ib,nb,s,wk,ier)
c
	real*8 a(ia,n)
c			Real m by n matrix. (input/output)
c			on input, a contains the matrix to be decomposed.
c			on output, a contains the n by n matrix v
c			in its first n rows. see remarks.
c			Either m.ge.n or m.lt.n is permitted.
	integer ia
c			Row dimension of matrix a exactly as
c			specified in the dimension statement in the
c			calling program. (input)
c			a is used by lsvdf as work storage for an
c			n by n matrix. therefore, ia must be
c			greater than or equal to max(m,n).
	integer n
c			Number of columns in a. (input)
	real*8 b(ib,1)
c			m by nb matrix. (input/output)
c			b is not used if nb.le.0. otherwise, b is
c			replaced by the matrix product u**(t) * b.
c			see remarks.
	integer	ib
c			Row dimension of matrix b exactly as
c			specified in the dimension statement in the
c			calling program. (input)
	real*8 s(n)
c			vector of length n. (output)
c			on output, s contains the ordered singular
c			values of a.  s(1) .ge. s(2),...,
c			.ge. s(n) .ge. 0.
	integer m
c			Number of rows in a. (input)
	integer nb
c			Number of columns in b. (input)
c                           if nb.le.0, b is not used.
	real*8 wk(n,2)
c			Work vector of length 2n.
	integer ier
c			Error parameter. (output)
c			Warning error
c			  ier=33 indicates that matrix a is not
c			         full rank or very ill-conditioned. small
c			         singular values may not be very accurate.
c			  ier=34 indicates that either n.le.0 or m.le.0.
c			Terminal error
c			  ier=129 indicates that convergence was
c			          not obtained by lsvdb and computation
c			          was discontinued.
c
c   remarks  1.  lsvdf computes the singular value decomposition of
c                a real m by n matrix
c                     a = u * q * v**(t)  where
c                u is an m by m orthogonal matrix,
c                v is an n by n orthogonal matrix, and
c                q is an m by n matrix with all elements zero except
c                     q(i,i) = s(i) i=1,...,min(m,n).
c                v is returned in the first n rows of a.
c                u is obtained by setting b to the m by m identity
c                matrix, on input, and setting nb = m. on output, b is
c                replaced by u**(t).
c            2.  the notation u**(t) and v**(t) represents u
c                transpose and v transpose, respectively. q**(+)
c                denotes the generalized inverse of q.
c            3.  lsvdf is useful in analyzing and solving the least
c                squares problem a*x.appr.b (where .appr. means
c                approximately equals). in this case b is a vector of
c                length m and lsvdf is Called with ib = m, nb=1. the
c                solution is x = v*q**(+)*u**(t)*b. u**(t)*b replaces
c                b on output. the solution x is obtained as follows...
c                (the user may wish to set small singular values such as
c                s(i) to zero just prior to computing x. see reference
c                for details.)
c
c                c                  compute q**(+) * u**(t) * b
c                      l = min0(m,n)
c                      do 10 i = 1,l
c                         t = 0.0
c                         if (s(i).ne.0.0) t = b(i)/s(i)
c                         b(i) = t
c                   10 continue
c                c                  compute v * q**(+) * u**(t) * b
c                   15 do 25 i = 1,n
c                         x(i) = 0.0
c                         do 20 j = 1,l
c                   20    x(i) = x(i)+a(i,j)*b(j)
c                   25 continue
c                if b is set to the j-th column of the m by m identity
c                matrix on input, x is the j = th column of a**(+) (the
c                generalized inverse of a).
c            4.  the user should be aware of several practical aspects
c                of the singular value analysis. one of these is the
c                effect of the uncertainty of the data and the need for
c                scaling. see the lawson-hanson reference pages 180-198
c                for details.
c
c-----------------------------------------------------------------------
c
c
c                                  specifications for arguments
	integer            ia,m,n,ib,nb,ier
	real*8             a(ia,n),b(ib,1),s(n),wk(n,2)
c                                  specifications for local variables
	integer            i,j,jp1,k,l,mm,nn,nnp1,ns,nsp1
	integer*4 level, nm
	real*8 f
	real*8             zero,one,t
	data               zero/0.d0/,one/1.d0/
c                                  first executable statement
	ier = 0
c                                  begin special for zero rows and
c                                    cols. pack the nonzero cols to the
c                                    left
	nn = n
	ier = 34
	if ((nn .le. 0) .or. (m .le. 0)) Return
	ier = 0
	j = nn
    5	continue
	Do i = 1, m
	    if (a(i,j).ne.zero) go to 25
	    Enddo

c  If col j is zero. exchange it with col n
	If (j .ne. nn) Then
	    Do i = 1,m
	    a(i,j) = a(i,nn)
		Enddo
	    Endif

	a(1,nn) = j
	nn = nn-1

   25	continue
	j = j - 1
	if (j .ge. 1) go to 5

c if n = 0 then a is entirely zero and svd computation can be skipped
	ns = 0
	if (nn. eq. 0) go to 120

c pack nonzero rows to the top quit packing if find n nonzero rows
	i = 1
	mm = m
   30	if ((i .le. n) .and. (i .lt. mm)) Then
	    if (a(i,i) .eq. zero) Then
		Do j = 1,nn
		    if (a(i,j) .ne. zero) go to 40
		    Enddo
		go to 45
		Endif
   40	    i = i+1
	    go to 30
c                                  row i is zero exchange rows i and m
   45	    if (nb .gt. 0) Then
		Do j = 1,nb
		    t = b(i,j)
		    b(i,j) = b(mm,j)
		    b(mm,j) = t
		    Enddo
		Endif

	    Do j = 1, nn
		a(i,j) = a(mm,j)
		Enddo
	    If (mm .le. nn) Then
		Do j = 1,nn
		    a(mm,j) = zero
		    Enddo
		Endif
c                                  exchange is finished
	    mm = mm-1
	    go to 30
	    Endif
c
   75	continue
c  end special for zero rows and columns
c  begin svd algorithm..
c  (1) reduce the matrix to upper bidiagonal form with householder
c      transformations.
c      h(n)...h(1)aq(1)...q(n-2) = (d**t,0)**t where d is upper bidiagonal.
c  (2) apply h(n)...h(1) to b. here h(n)...h(1)*b replaces b in storage.
c  (3) the matrix product w = q(1)...q(n-2) overwrites the first
c      n rows of a in storage.
c  (4) an svd for d is computed. here k rotations ri and pi are computed
c      so that rk...r1*d*p1**(t)...pk**(t) = diag(s1,...,sm) to working
c      accuracy. the si are nonnegative and nonincreasing. here rk...r1*b
c      overwrites b in storage while a*p1**(t)...pk**(t) overwrites a
c      in storage.
c  (5) it follows that,with the proper definitions, u**(t)*b overwrites
c      b, while v overwrites the first n row and columns of a.

	l = min0(mm,nn)

c  the following loop reduces a to upper bidiagonal and also applies
c  the premultiplying transformations to b.
	Do j = 1,l
	    If (j .lt. mm) then
		jp1 = min0(j+1,nn)
		Call vhs12 (1,j,j+1,mm,a(1,j),1,t,a(1,jp1),1,ia,nn-j)
		Call vhs12 (2,j,j+1,mm,a(1,j),1,t,b,1,ib,nb)
		Endif
	    If (j .lt. nn-1) then
		Call vhs12 (1,j+1,j+2,nn,a(j,1),ia,wk(j,2),a(j+1,1),ia,1,mm-j)
		Endif
	    Enddo
c                                  copy the bidiagonal matrix into the
c                                    array s for lsvdb
	if (l .ne. 1) Then
	    Do j = 2,l
		s(j) = a(j,j)
		wk(j,1) = a(j-1,j)
		Enddo
	    Endif
	s(1) = a(1,1)
c
	ns = nn
	If (mm .lt. nn) then
	    ns = mm+1
	    s(ns) = zero
	    wk(ns,1) = a(mm,mm+1)
	    Endif
c                                  construct the explicit n by n
c                                    product matrix, w = q1*q2*...*ql*i
c                                    in the array a
	Do k = 1,nn
	    i = nn+1-k
	    If (i .le. min0(mm,nn-2)) Then
		Call vhs12 (2,i+1,i+2,nn,a(i,1),ia,wk(i,2),a(1,i+1),1,ia,nn-i)
		Endif
	    Do j = 1,nn
		a(i,j) = zero
		Enddo
	    a(i,i) = one
	    Enddo
c                                  compute the svd of the bidiagonal
c                                    matrix
c
	level = 1
	Call lsvdb (s(1),wk(1,1),ns,a,ia,nn,b,ib,nb,ier)
c                                  test for ier = 33
c
	if (ier .gt. 128) Then
	    Return
	    Endif

	if (ier .eq. 33) Then
	    t = 0.d0
	    nm = min0(m,n)
	    if (s(1).ne.zero) t = s(nm)/s(1)
	    f = 100.d0 + t
	    If (f .ne. 100.d0) Then
		ier = 0
		Endif
	    Endif

120	Continue

	If (ns .lt. nn) Then
	    nsp1 = ns+1
	    Do j = nsp1,nn
		s(j) = zero
		Enddo
	    Endif

	if (nn .ne. n) Then
	    nnp1 = nn+1
c                                  move record of permutations and
c                                    store zeros
	    do j = nnp1,n
		s(j) = a(1,j)
		If (nn .gt. 0) Then
		    Do i = 1, nn
			a(i,j) = zero
			Enddo
		    Endif
		Enddo
		    
c                                  permute rows and set zero singular
c                                    values
	    Do k = nnp1,n
		i = Idnint (s(k))
		s(k) = zero
		Do j = 1,n
		    a(k,j) = a(i,j)
		    a(i,j) = zero
		    Enddo
		a(i,k) = one
		Enddo
	    Endif
c                                  end special for zero rows and
c                                    columns
	If (ier.eq.0) go to 9005
 9000	continue
 9005	return
	end

c-----------------------------------------------------------------------
c
c LSVG1 -- nucleus called only by routine lsvdb

	subroutine lsvg1  (a,b,cos,sin,sig)

	real*8 a
	real*8 b
	real*8 cos
	real*8 sin
	real*8 sig

	real*8 aa,bb

	if (abs(a) .gt. abs(b)) then
	    aa = abs (a+a)
	    sig = aa * sqrt (0.25d0 + (b/aa)**2)
	    cos = a / sig
	    sin = b / sig
	else  if (b .ne. 0.d0) then
	    bb = abs (b+b)
	    sig = bb * sqrt (0.25d0 + (a/bb)**2)
	    cos = a / sig
	    sin = b / sig
	else
	    sig = 0.d0
	    cos = 0.d0
	    sin = 1.d0
	endif
	return
	end

c-----------------------------------------------------------------------

c LSVG2 -- nucleus called by routine lsvdb

	subroutine lsvg2  (cos,sin,x,y)

	real*8 cos
	real*8 sin
	real*8 x
	real*8 y

	real*8 xr

	xr = cos*x+sin*y
	y = -sin*x+cos*y
	x = xr
	return
	end

c-----------------------------------------------------------------------

c DROTG --  construct givens plane rotation (double precision)

	subroutine drotg  (da,db,dc,ds)

	double precision   da
c			first element of double precision vector.
c			(input/output)
c			on output, r = (+/-)dsqrt(da**2 + db**2)
c			overwrites da.
	double precision  db
c			second element of double precision vector.
c			(input/output)
c			on output, z overwrites db.
c			z is defined to be:
c			ds,       if dabs(da).gt.dabs(db)
c			1.0d0/dc, if dabs(db).ge.dabs(da) and dc.ne.0.0d0
c			1.0d0,    if dc.eq.0.0d0.
	double precision dc
c			double precision element of output
c			transformation matrix. see remarks.
	double precision ds
c			double precision element of output
c			transformation matrix. see remarks.
c
c DROTG constructs the givens transformation
c                    ( dc  ds )
c                g = (        ) ,  dc**2 + ds**2 = 1 ,
c                    (-ds  dc )
c                which zeros the second element of (da,db)**t.

	double precision   u,v,r

	if (dabs(da) .gt. dabs(db)) then
c                                  here dabs(da) .gt. dabs(db)
	    u = da + da
	    v = db / u
c                                  note that u and r have the sign of
c                                    da
	    r = dsqrt (.25d0 + v**2) * u
c                                  note that dc is positive
	    dc = da / r
	    ds = v * (dc + dc)
	    db = ds
	    da = r
	    return
c                                  here dabs(da) .le. dabs(db)
	elseif (db .ne. 0.d0) then
	    u = db + db
	    v = da / u
c                                  note that u and r have the sign of
c                                    db (r is immediately stored in da)
	    da = dsqrt (.25d0 + v**2) * u
c                                  note that ds is positive
	    ds = db / da
	    dc = v * (ds + ds)
	    if (dc .ne. 0.d0) then
		db = 1.d0 / dc
	    else
		db = 1.d0
	    endif
	    return

c  da = db = 0.d0
	else
	    dc = 1.d0
	    ds = 0.d0
	    return
	endif
c
	end

c-----------------------------------------------------------------------

c VHS12 -- real householder transformation - computation and applications.

	subroutine vhs12  (mode,lp,l1,m,u,incu,up,c,incc,icv,ncv)

	integer mode
c			option parameter. (input)
c                         if mode = 1, the subroutine computes a
c                           householder transformation and if ncv.gt.0,
c                           multiplies it by the set of ncv vectors
c                           (each of length m) stored in c. for a
c                           given vector v of length m and two integer
c                           indices lp and l1 that satisfy
c                           1 .le. lp .lt. l1 .le. m, the subroutine
c                           defines an m by m householder
c                           transformation q which satifies qv = w where
c                           w(i) = v(i) for i.lt.lp
c                           w(lp) = -sig*sqrt(v(lp)**2+v(l1)**2+...
c                             +v(m)**2)
c                             sig = 1  if v(lp).ge.0
c                             sig = -1 if v(lp).lt.0
c                           w(i) = v(i) for lp.lt.i.lt.l1
c                           w(i) = 0    for i.ge.l1.
c                         if mode = 2, the subroutine assumes that a
c                           householder transformation has already
c                           been defined by a previous Call with
c                           mode = 1, and if ncv.gt.0, multiplies it by
c                           the set of ncv vectors (each of length
c                           m) stored in c.
	integer lp,l1
c			parameters that define the desired
c			householder transformation. (input)
	integer m
c			if the condition 1.le.lp.lt.l1.le.m is
c			not satisfied, the subroutine returns to
c			the calling program without performing
c			any computations.
	real*8 u(1)
c			vector of m elements. (input, and output if
c			mode = 1)
c			the storage increment between elements
c			of u is incu. (i.e., u(1+(j-1)*incu),
c			j = 1,...,m). if mode=1, the array v is
c			defined as v(j) = u(1+(j-1)*incu),
c			j = 1,...,m.
c			on output, u(1+(lp-1)*incu) is set to
c			w(lp) (as defined above in the description
c			of mode = 1).
	integer incu
c			increment between elements of u. (input)
	real*8 up
c			scalar set to v(lp)-w(lp) to define the
c			householder transformation q. (input if
c			mode = 2, output if mode=1)
	real*8 c(1)
c			vector of ncv*m elements. (input/output)
c			if ncv.le.0, c is not used.
c			if ncv.gt.0, c contains ncv vectors
c			of length m with increment incc between
c			elements of vectors and increment icv
c			between vectors. element i of vector j is
c			defined as c(1+(i-1)*incc+(j-1)*icv),
c			i = 1,...,m and j=1,...,ncv.
c			on output, c contains the set of ncv
c			vectors resulting from multiplying
c			the given vectors by q.
	integer incc
c			increment between elements of vectors in c. (input)
	integer icv
c			increment between vectors in c. (input)
	integer ncv
c			number of vectors stored in c. (input)
c
c
c   remarks  1.  if u is a single subscripted array or the j-th column
c                of a matrix, then incu = 1. if u is the i-th row of a
c                matrix then incu is the row dimension of the matrix
c                exactly as specified in the calling program.
c            2.  if c is a double subscripted matrix and the vectors
c                are the first ncv columns of c, then incc = 1 and icv
c                is the row dimension of c exactly as specified in
c                the calling program. in this case c is replaced
c                by qc. if the vectors are successive rows of c
c                then incc is the row dimension of c exactly as
c                specified in the calling program and icv = 1. in this
c                case c is replaced by cq.
c
c-----------------------------------------------------------------------
c
c

	integer            ij,ilp,il1,im,incr,i2,i3,i4,j
	double precision   sm,b
	real*8             one,cl,clinv,sm1

	one = 1.d0
c
	If ((0 .ge. lp) .or. (lp .ge. l1) .or. (l1 .gt. m)) Then
	    Return
	    Endif
	ilp = (lp - 1) * incu + 1
	il1 = (l1 - 1) * incu + 1
	im = (m - 1) * incu + 1
	cl = abs (u(ilp))
c                                  construct the transformation.
	If (mode .ne. 2) Then
	    Do ij = il1, im, incu
		cl = max (abs (u(ij)), cl)
		Enddo
	    if (cl .le. 0.d0) Then
		Return
		Endif
	    clinv = one/cl
	    sm = (dble(u(ilp))*clinv)**2
	    Do ij = il1,im,incu
		sm = sm+(dble(u(ij))*clinv)**2
		Enddo

c  Convert dble. prec. sm to sngl. prec. sm1
	    sm1 = sm
	    cl = cl*sqrt(sm1)
	    if (u(ilp).gt.0.d0) cl = -cl
	    up = u(ilp)-cl
	    u(ilp) = cl

c  apply the transformation i+u*(u**t)/b to c.
	Else if (cl .le. 0.d0) Then
	    Return
	Endif

	If (ncv .le. 0) Then
	    Return
	    Endif
	b = dble(up)*u(ilp)

c  b must be nonpositive here. if b = 0., return.
	if (b .ge. 0.d0) Return
	b = one / b
	i2 = 1 - icv + incc * (lp - 1)
	incr = incc*(l1-lp)
	Do j = 1, ncv
	    i2 = i2 + icv
	    i3 = i2 + incr
	    i4 = i3
	    sm = c(i2) * dble(up)
	    do ij = il1, im, incu
		sm = sm + c(i3) * dble (u(ij))
		i3 = i3 + incc
		Enddo

	    if (sm .ne. 0.d0) Then
		sm = sm*b
		c(i2) = c(i2) + sm * dble (up)
		Do ij = il1,im,incu
		    c(i4) = c(i4) + sm * dble (u(ij))
		    i4 = i4 + incc
		    Enddo
		Endif
	    Enddo

	Return
	End
	
c Jul 26 1999	Added command line parsing
c Sep 22 1999	Initialize input array; drop extraneous args to write_fit()

c Jan  6 2000	Write input file name in HISTORY of output file

c Aug 20 2004	Increase data to 4096 pixels, 4096 spectra max.
c Aug 20 2004	Drop as many Go To's as possible from subroutines
