# File rvsao/Util/fill.x
# May 8, 2008
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1999-2008 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.


# FILLLIST -- Interpolate spectrum across each of list of wavelength ranges

procedure filllist (npts, pix, wav, debug)
int	npts		# Number of points in spectrum
real	pix[ARB]	# Array of points to edit
real	wav[ARB]	# Array of wavelengths corresponding to points to edit
bool	debug		# TRUE print diagnostic messages

real	w1, w2		# Wavelengths between which to interpolate
int	fd, lpath, nlrep
char	linename[SZ_LINE]
char	dirpath[SZ_PATHNAME]
char	filename[SZ_PATHNAME]
char	filepath[SZ_PATHNAME]
int	fscan(), open(), stridx(), strlen()

begin

#  Directory containing line lists
	call clgstr ("linedir", dirpath, SZ_PATHNAME)
	lpath = strlen (dirpath)
	if (dirpath[lpath] != '/') {
	    dirpath[lpath+1] = '/'
	    dirpath[lpath+2] = EOS
	    }

#  Lines to replace with surrounding spectrum
	call clgstr ("badlines", filename, SZ_PATHNAME)
	if (strlen (filename) > 0) {
	    if (stridx ("/",filename) > 0 || stridx ("$",filename) > 0)
		call strcpy (filename,filepath,SZ_PATHNAME)
	    else {
		call strcpy (dirpath,filepath,SZ_PATHNAME)
		call strcat (filename,filepath,SZ_PATHNAME)
		}
	    fd = open (filepath, READ_ONLY, TEXT_FILE)
	    if (fd >0) {
		nlrep = 0
		while (fscan(fd) != EOF) {
		    call gargr (w1)
		    call gargr (w2)
		    call gargstr (linename,SZ_LINE)
		    call fillwav (npts, pix, wav, w1, w2, linename, debug)
		    nlrep = nlrep + 1
		    }
		call close (fd)
		if (debug) {
		    call printf ("FILL_LIST: %d lines replaced from line file %s\n")
			call pargi (nlrep)
			call pargstr (filepath)
		    }
		}
	    else {
		call printf ("FILL_LIST: Cannot find bad line file %s\n")
		    call pargstr (filepath)
		}
	    }
	else
	    call printf ("FILL_LIST: No bad line file specified\n")
end


# FILLWAV -- Interpolate spectrum between two wavelengths

procedure fillwav (npts, pix, wav, w1, w2, linename, debug)
 
int	npts		# Number of points in spectrum
real	pix[ARB]	# Array of points to edit
real	wav[ARB]	# Array of wavelengths corresponding to points to edit
real	w1, w2		# Wavelengths between which to interpolate
char	linename[ARB]	# Name of line to be cut
bool	debug		# true for debugging information

int	p1, p2		# Pixels between which to interpolate
int	i, i1, i2
real	dw2

begin

# Find closest pixel to starting wavelength within limits
	if (w1 <= wav[1])
	    p1 = 1
	else if (w1 >= wav[npts])
	    p1 = npts
	else {
	    i1 = 1
	    i2 = npts - 1
	    do i = i1, i2 {
		if (w1 >= wav[i] && w1 < wav[i+1]) {
		    dw2 = (wav[i+1] - wav[i]) * 0.5
		    if (w1 < wav[i] + dw2)
			p1 = i
		    else
			p1 = i + 1
		    break
		    }
		}
	    }

# Find closest pixel to ending wavelength within limits
	if (w2 <= wav[1])
	    p2 = 1
	else if (w2 >= wav[npts])
	    p2 = npts
	else {
	    i1 = 1
	    i2 = npts - 1
	    do i = i1, i2 {
		if (w2 >= wav[i] && w2 < wav[i+1]) {
		    dw2 = (wav[i+1] - wav[i]) * 0.5
		    if (w2 < wav[i] + dw2)
			p2 = i
		    else
			p2 = i + 1
		    break
		    }
		}
	    }
	if (debug) {
	    call printf ("FILL_LIST: %s replaced from %.2dA to %.2dA = %d to %d\n")
		call pargstr (linename)
		call pargr (w1)
		call pargr (w2)
		call pargi (p1)
		call pargi (p2)
	    }

	call fillpix (npts, pix, p1, p2)
end


# FILLPIX -- Interpolate spectrum between two pixels
 
procedure fillpix (npts, pix, p1, p2)
 
int	npts		# Number of points in spectrum
real	pix[ARB]	# Array of points to edit
int	p1, p2		# Pixels between which to interpolate
 
int	i, temp
real	p0, slope
 
begin
 
# If p2 < p1, switch the order.
	if ( p2 < p1 ) {
	    temp = p1
	    p1 = p2
	    p2 = temp
	    }
 
# If p2 = p1, increase it by one.
	if ( p2 == p1 )
	    p2 = p2 + 1
 
	if ( p1 > 1 && p2 < npts ) {
	    p0 = pix[p1 - 1]
	    slope = (pix[p2 + 1] - p0) / (p2 + 2 - p1)
	    }
	else if ( p1 == 1 && p2 < npts ) {
	    p0 = pix[p2 + 1]
	    slope = 0.
	    }
	else if ( p1 > 1 && p2 == npts ) {
	    p0 = pix[p1 - 1]
	    slope = 0.
	    }
 
	for ( i = p1; i <= p2; i = i + 1) {
	    pix[i] = p0 + (i - p1) * slope
	    }
end


# ZEROLIST -- Zero spectrum across each of list of wavelength ranges

procedure zerolist (npts, pix, wav, debug)
int	npts		# Number of points in spectrum
real	pix[ARB]	# Array of points to edit
real	wav[ARB]	# Array of wavelengths corresponding to points to edit
bool	debug		# TRUE print diagnostic messages

real	w1, w2		# Wavelengths between which to interpolate
int	fd, lpath, nlrep
char	linename[SZ_LINE]
char	dirpath[SZ_PATHNAME]
char	filename[SZ_PATHNAME]
char	filepath[SZ_PATHNAME]
int	fscan(), open(), stridx(), strlen()

begin

#  Directory containing line lists
	call clgstr ("linedir", dirpath, SZ_PATHNAME)
	lpath = strlen (dirpath)
	if (dirpath[lpath] != '/') {
	    dirpath[lpath+1] = '/'
	    dirpath[lpath+2] = EOS
	    }

#  Lines to replace with zeroes
	call clgstr ("badlines", filename, SZ_PATHNAME)
	if (stridx ("/",filename) > 0 || stridx ("$",filename) > 0)
	    call strcpy (filename,filepath,SZ_PATHNAME)
	else {
	    call strcpy (dirpath,filepath,SZ_PATHNAME)
	    call strcat (filename,filepath,SZ_PATHNAME)
	    }
	fd = open (filepath, READ_ONLY, TEXT_FILE)
	if (fd >0) {
	    nlrep = 0
	    while (fscan(fd) != EOF) {
		call gargr (w1)
		call gargr (w2)
		call gargstr (linename,SZ_LINE)
		call zerowav (npts, pix, wav, w1, w2, linename, debug)
		nlrep = nlrep + 1
		}
	    call close (fd)
	    if (debug) {
		call printf ("ZERO_LIST: %d lines zeroed from line file %s\n")
		    call pargi (nlrep)
		    call pargstr (filepath)
		}
	    }
	else {
	    call printf ("ZERO_LIST: Cannot find bad line file %s\n")
		call pargstr (filepath)
	    }
end


# ZEROWAV -- Zero spectrum between two wavelengths

procedure zerowav (npts, pix, wav, w1, w2, linename, debug)
 
int	npts		# Number of points in spectrum
real	pix[ARB]	# Array of points to edit
real	wav[ARB]	# Array of wavelengths corresponding to points to edit
real	w1, w2		# Wavelengths between which to interpolate
char	linename[ARB]	# Name of line to be cut
bool	debug		# true for debugging information

int	p1, p2		# Pixels between which to interpolate
int	i, i1, i2
real	dw2

begin

# Find closest pixel to starting wavelength within limits
	if (w1 <= wav[1])
	    p1 = 1
	else if (w1 >= wav[npts])
	    p1 = npts
	else {
	    i1 = 1
	    i2 = npts - 1
	    do i = i1, i2 {
		if (w1 >= wav[i] && w1 < wav[i+1]) {
		    dw2 = (wav[i+1] - wav[i]) * 0.5
		    if (w1 < wav[i] + dw2)
			p1 = i
		    else
			p1 = i + 1
		    break
		    }
		}
	    }

# Find closest pixel to ending wavelength within limits
	if (w2 <= wav[1])
	    p2 = 1
	else if (w2 >= wav[npts])
	    p2 = npts
	else {
	    i1 = 1
	    i2 = npts - 1
	    do i = i1, i2 {
		if (w2 >= wav[i] && w2 < wav[i+1]) {
		    dw2 = (wav[i+1] - wav[i]) * 0.5
		    if (w2 < wav[i] + dw2)
			p2 = i
		    else
			p2 = i + 1
		    break
		    }
		}
	    }
	if (debug) {
	    call printf ("ZEROWAV: %s zeroed from %.2dA to %.2dA = %d to %d\n")
		call pargstr (linename)
		call pargr (w1)
		call pargr (w2)
		call pargi (p1)
		call pargi (p2)
	    }

	call zeropix (npts, pix, p1, p2)
end

 
# ZEROPIX -- Zero spectrum between two pixels

procedure zeropix (npts, pix, p1i, p2i)
 
int	npts		# Number of points in spectrum
real	pix[ARB]	# Array of points to edit
int	p1i, p2i	# Pixels between which to interpolate
 
int	i
real	p1, p2
 
begin
 
# If p2 < p1, switch the order.
	if (p2i < p1i) {
	    p1 = p2i
	    p2 = p1i
	    }
 
# If p2 = p1, increase it by one.
	else if (p2 == p1) {
	    p1 = p1i
	    p2 = p2i + 1
	    }

	else {
	    p1 = p1i
	    p2 = p2i
	    }
 
# Check limits
	if (p1 < 1)
	    p1 = 1
	if (p2 > npts)
	    p2 = npts
 
# Fill range of pixels with zeroes
	do i = p1, p2 {
	    pix[i] = 0.0
	    }
end


# RENORMSPEC -- normalize a spectrum to its mean and multiply by a constant

procedure renormspec (spectrum, rconst, npix, ier)

real	spectrum[ARB]	# Spectrum to normalize
double  rconst          # renormalize by dividing by mean/this
			# if zero do not change spectrum
int	npix		# Number of pixels in spectrum
int	ier		# return 0 if OK, else 1

int	nsp		# Number of pixels summed
double	sptot		# Sum of values of pixels in spectrum
double	sptot2		# Sum of squares of values of pixels in spectrum
double	spmean		# Mean value of pixels in spectrum
double	sprms		# RMS value of pixels in spectrum
real	smin, smax
real	speci		# Value of individual pixel
int	i

begin
	ier = 0
	
	if (rconst == 0.0) 
	    return

# Find RMS value of spectrum for scaling
	sptot = 0.d0
	sptot2 = 0.d0
	nsp = 0
	do i = 1, npix {
	    speci = spectrum[i]
	    if (speci != 0.) {
		sptot2 = sptot2 + (double (speci) * double (speci))
		sptot = sptot + double (speci)
		nsp = nsp + 1
		}
	    }
	if (nsp > 0) {
	    sprms = sqrt (sptot2 / double (nsp)) / rconst
	    if (sprms != 0.0) {
		do i = 1, npix {
		    if (spectrum[i] != 0.) {
			spectrum[i] = spectrum[i] / sprms
			}
		    }
		}
	    }
	else {
	    call eprintf ("RENORM: Spectrum is all zeroes\n")
	    ier = 1
	    }
	return
end

# May  4 1999	New file from rvsao/Util/fill.x
# Sep  8 1999	Fix bug interpolating wavelength limits

# Jan 18 2000	Note that lines are being zeroed, not filled

# Jan 31 1997	New file renamed subroutine FIXIT to FILLPIX

# May 21 1999	Test for existence of bad pixel file
# Sep  8 1999	Fix bug interpolating wavelength limits

# Apr 26 2000	Add zero.x and renormalization to this file; rename procspec.x

# Nov 20 2006	Change renorm() to renormspec()

# May  8 2008	Scale by RMS, not mean, so continuum-less data is scaled
